@@ -19,17 +19,17 @@ class FakeGTKClient(FakeClient):
         self.timeout_add = glib.timeout_add
         self.idle_add = glib.idle_add
 
-    def get_mouse_position(self, *args):
+    def get_mouse_position(self, *_args):
         root = gdk.get_default_root_window()
         p = root.get_pointer()
         return p[0], p[1]
 
-    def get_current_modifiers(self, *args):
+    def get_current_modifiers(self, *_args):
         #root = gdk.get_default_root_window()
         #modifiers_mask = root.get_pointer()[-1]
         #return self.mask_to_names(modifiers_mask)
         return []
-    def window_close_event(self, *args):
+    def window_close_event(self, *_args):
         gtk.main_quit()
 
     def server_ok(self):
