@@ -5,7 +5,7 @@ pygtk.require('2.0')
 import gtk
 
 
-from xpra.x11.gtk_x11 import gdk_display_source			 #@UnusedImport
+from xpra.x11.bindings import posix_display_source      #@UnusedImport
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings		#@UnresolvedImport
 keyboard_bindings = X11KeyboardBindings()
 keycode_mappings = keyboard_bindings.get_keycode_mappings()
