@@ -78,6 +78,7 @@ XPRA_VERSION = ""
 for x in XPRA_VERSION_OUTPUT.splitlines():
     if x.startswith("xpra v"):
         XPRA_VERSION = x[len("xpra v"):].replace("\n", "").replace("\r", "")
+XPRA_VERSION = XPRA_VERSION.split("-")[0]
 XPRA_VERSION_NO = [int(x) for x in XPRA_VERSION.split(".")]
 XPRA_SERVER_STOP_COMMANDS = [
                              [XPRA_BIN, "stop", ":%s" % config.DISPLAY_NO],
@@ -95,10 +96,6 @@ if STRICT_ENCODINGS:
     os.environ["XPRA_MAX_PIXELS_PREFER_RGB"] = "0"
     os.environ["XPRA_MAX_NONVIDEO_PIXELS"] = "0"
 
-#webp leaks - don't test it:
-#if XPRA_VERSION_NO>=[0, 7]:
-#    config.XPRA_TEST_ENCODINGS.append("webp")
-
 XPRA_SPEAKER_OPTIONS = [None]
 XPRA_MICROPHONE_OPTIONS = [None]
 if config.TEST_SOUND:
@@ -458,9 +455,9 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                     #start the test command:
                     if config.USE_VIRTUALGL:
                         if type(test_command)==str:
-                            cmd = config.VGLRUN_BIN + " -- "+ test_command
+                            cmd = config.VGLRUN_BIN + "-d "+os.environ.get("DISPLAY")+" -- "+ test_command
                         elif type(test_command) in (list, tuple):
-                            cmd = [config.VGLRUN_BIN, "--"] + list(test_command)
+                            cmd = [config.VGLRUN_BIN, "-d", os.environ.get("DISPLAY"), "--"] + list(test_command)
                         else:
                             raise Exception("invalid test command type: %s for %s" % (type(test_command), test_command))
                     else:
@@ -577,6 +574,7 @@ def get_auth_args():
         if XPRA_VERSION_NO>=[0, 14]:
             cmd.append("--auth=file")
         if XPRA_VERSION_NO>=[0, 15]:
+            cmd.append("--auth=none")
             cmd.append("--tcp-auth=file")
     return cmd
 
