@@ -15,7 +15,8 @@ from xpra.x11.xroot_props import XRootPropWatcher
 
 def main():
     ROOT_PROPS = ["RESOURCE_MANAGER", "_NET_WORKAREA"]
-    xrpw = XRootPropWatcher(ROOT_PROPS)
+    root = gtk.gdk.get_default_root_window()
+    xrpw = XRootPropWatcher(ROOT_PROPS, root)
     gobject.timeout_add(1000, xrpw.notify_all)
     try:
         gtk.main()
