@@ -5,15 +5,14 @@
 # later version. See the file COPYING for details.
 
 import unittest
-from unit.client.x11_clipboard_test_util import X11ClipboardTestUtil
+from unit.client.x11_clipboard_test_util import X11ClipboardTestUtil, has_xclip
 from xpra.os_util import OSX, POSIX, PYTHON2
 
 
 class OSXLikeClipboardTest(X11ClipboardTestUtil):
 
-	@classmethod
-	def get_run_env(cls):
-		env = X11ClipboardTestUtil.get_run_env()
+	def get_run_env(self):
+		env = X11ClipboardTestUtil.get_run_env(self)
 		env.update({
 					"XPRA_CLIPBOARD_WANT_TARGETS"	: "1",
 					"XPRA_CLIPBOARD_GREEDY"			: "1",
@@ -36,7 +35,7 @@ class OSXLikeClipboardTest(X11ClipboardTestUtil):
 
 
 def main():
-	if POSIX and PYTHON2 and not OSX:
+	if POSIX and PYTHON2 and not OSX and has_xclip():
 		unittest.main()
 
 
