@@ -10,17 +10,25 @@ import unittest
 import tempfile
 from xpra.util import repr_ellipsized
 from xpra.os_util import load_binary_file, pollwait, OSX, POSIX, PYTHON2
-from xpra.exit_codes import EXIT_OK, EXIT_CONNECTION_LOST, EXIT_SSL_FAILURE
+from xpra.exit_codes import EXIT_OK, EXIT_CONNECTION_LOST, EXIT_SSL_FAILURE, EXIT_STR
 from xpra.net.net_util import get_free_tcp_port
 from unit.server_test_util import ServerTestUtil, log
 
 
+def estr(r):
+	s = EXIT_STR.get(r)
+	if s:
+		return "%s : %s" % (r, s)
+	return str(r)
+
+
 class ServerSocketsTest(ServerTestUtil):
 
-	@classmethod
-	def start_server(cls, *args):
-		server_proc = cls.run_xpra(["start", "--no-daemon"]+list(args))
-		assert pollwait(server_proc, 5) is None, "server failed to start, returned %s" % server_proc.poll()
+	def start_server(self, *args):
+		server_proc = self.run_xpra(["start", "--no-daemon"]+list(args))
+		if pollwait(server_proc, 5) is not None:
+			r = server_proc.poll()
+			raise Exception("server failed to start with args=%s, returned %s" % (args, estr(r)))
 		return server_proc
 
 	def _test_connect(self, server_args=[], auth="none", client_args=[], password=None, uri_prefix=":", exit_code=0):
@@ -31,9 +39,11 @@ class ServerSocketsTest(ServerTestUtil):
 		#we should always be able to get the version:
 		uri = uri_prefix + str(display_no)
 		client = self.run_xpra(["version", uri] + server_args)
-		assert pollwait(client, 5)==0, "version client failed to connect"
-		if client.poll() is None:
-			client.terminate()
+		if pollwait(client, 5)!=0:
+			r = client.poll()
+			if client.poll() is None:
+				client.terminate()
+			raise Exception("version client failed to connect, returned %s" % estr(r))
 		#try to connect
 		cmd = ["connect-test", uri] + client_args
 		f = None
@@ -44,22 +54,26 @@ class ServerSocketsTest(ServerTestUtil):
 		r = pollwait(client, 5)
 		if f:
 			f.close()
-		assert r==exit_code, "expected info client to return %s but got %s" % (exit_code, client.poll())
 		if client.poll() is None:
 			client.terminate()
 		server.terminate()
+		if r!=exit_code:
+			raise Exception("expected info client to return %s but got %s" % (estr(exit_code), estr(r)))
 
 	def test_default_socket(self):
 		self._test_connect([], "allow", [], "hello", ":", EXIT_OK)
 
 	def test_tcp_socket(self):
 		port = get_free_tcp_port()
-		self._test_connect(["--bind-tcp=0.0.0.0:%i" % port], "allow", [], "hello", "tcp/127.0.0.1:%i/" % port, EXIT_OK)
-		self._test_connect(["--bind-tcp=0.0.0.0:%i" % port], "allow", [], "hello", "ws/127.0.0.1:%i/" % port, EXIT_OK)
+		self._test_connect(["--bind-tcp=0.0.0.0:%i" % port], "allow", [], "hello", "tcp://127.0.0.1:%i/" % port, EXIT_OK)
+		self._test_connect(["--bind-tcp=0.0.0.0:%i" % port], "allow", [], "hello", "ws://127.0.0.1:%i/" % port, EXIT_OK)
 
 	def test_ws_socket(self):
+		#the client will need "websocket-client":
+		import websocket
+		assert websocket
 		port = get_free_tcp_port()
-		self._test_connect(["--bind-ws=0.0.0.0:%i" % port], "allow", [], "hello", "ws/127.0.0.1:%i/" % port, EXIT_OK)
+		self._test_connect(["--bind-ws=0.0.0.0:%i" % port], "allow", [], "hello", "ws://127.0.0.1:%i/" % port, EXIT_OK)
 
 
 	def test_ssl(self):
@@ -114,19 +128,19 @@ class ServerSocketsTest(ServerTestUtil):
 				assert r==exit_code, "expected info client to return %s but got %s" % (exit_code, client.poll())
 			noverify = "--ssl-server-verify-mode=none"
 			#connect to ssl socket:
-			test_connect("ssl/127.0.0.1:%i/" % ssl_port, EXIT_OK, noverify)
+			test_connect("ssl://127.0.0.1:%i/" % ssl_port, EXIT_OK, noverify)
 			#tcp socket should upgrade to ssl:
-			test_connect("ssl/127.0.0.1:%i/" % tcp_port, EXIT_OK, noverify)
+			test_connect("ssl://127.0.0.1:%i/" % tcp_port, EXIT_OK, noverify)
 			#tcp socket should upgrade to ws and ssl:
-			test_connect("wss/127.0.0.1:%i/" % tcp_port, EXIT_OK, noverify)
+			test_connect("wss://127.0.0.1:%i/" % tcp_port, EXIT_OK, noverify)
 			#ws socket should upgrade to ssl:
-			test_connect("wss/127.0.0.1:%i/" % ws_port, EXIT_OK, noverify)
+			test_connect("wss://127.0.0.1:%i/" % ws_port, EXIT_OK, noverify)
 			
 			#self signed cert should fail without noverify:
-			test_connect("ssl/127.0.0.1:%i/" % ssl_port, EXIT_SSL_FAILURE)
-			test_connect("ssl/127.0.0.1:%i/" % tcp_port, EXIT_SSL_FAILURE)
-			test_connect("wss/127.0.0.1:%i/" % ws_port, EXIT_SSL_FAILURE)
-			test_connect("wss/127.0.0.1:%i/" % wss_port, EXIT_SSL_FAILURE)
+			test_connect("ssl://127.0.0.1:%i/" % ssl_port, EXIT_SSL_FAILURE)
+			test_connect("ssl://127.0.0.1:%i/" % tcp_port, EXIT_SSL_FAILURE)
+			test_connect("wss://127.0.0.1:%i/" % ws_port, EXIT_SSL_FAILURE)
+			test_connect("wss://127.0.0.1:%i/" % wss_port, EXIT_SSL_FAILURE)
 
 		finally:
 			shutil.rmtree(tmpdir)
