@@ -9,7 +9,7 @@ import os
 import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
-from xpra.os_util import bytestostr, PYTHON2, OSX
+from xpra.os_util import bytestostr, PYTHON2, PYTHON3, OSX, WIN32
 from xpra.util import typedict, envbool, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
 log = Logger("window")
@@ -20,21 +20,23 @@ workspacelog = Logger("workspace")
 keylog = Logger("keyboard")
 metalog = Logger("metadata")
 geomlog = Logger("geometry")
+iconlog = Logger("icon")
 
 
 REPAINT_ALL = os.environ.get("XPRA_REPAINT_ALL", "")
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
+AWT_DIALOG_WORKAROUND = envbool("XPRA_AWT_DIALOG_WORKAROUND", WIN32)
 
 
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size, default_cursor_data, pixel_depth):
-        log("%s%s", type(self), (client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size, default_cursor_data, pixel_depth))
-        ClientWidgetBase.__init__(self, client, wid, metadata.boolget("has-alpha"))
+    def __init__(self, client, group_leader, watcher_pid, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size, default_cursor_data, pixel_depth):
+        log("%s%s", type(self), (client, group_leader, watcher_pid, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size, default_cursor_data, pixel_depth))
+        ClientWidgetBase.__init__(self, client, watcher_pid, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
-        self._pos = (x, y)
+        self._pos = (wx, wy)
         self._size = (ww, wh)
         self._client_properties = client_properties
         self._set_initial_position = metadata.boolget("set-initial-position", False)
@@ -57,6 +59,7 @@ class ClientWindowBase(ClientWidgetBase):
         self.max_window_size = max_window_size
         self.button_state = {}
         self.pixel_depth = pixel_depth      #0 for default
+        self.window_offset = None           #actual vs reported coordinates
 
         self.init_window(metadata)
         self.setup_window(bw, bh)
@@ -94,12 +97,6 @@ class ClientWindowBase(ClientWidgetBase):
         return None
 
 
-    def set_cursor_data(self, cursor_data):
-        self.cursor_data = cursor_data
-        b = self._backing
-        if b:
-            b.set_cursor_data(cursor_data)
-
     def new_backing(self, bw, bh):
         backing_class = self.get_backing_class()
         log("new_backing(%s, %s) backing_class=%s", bw, bh, backing_class)
@@ -108,7 +105,6 @@ class ClientWindowBase(ClientWidgetBase):
         self._backing = self.make_new_backing(backing_class, w, h, bw, bh)
         self._backing.border = self.border
         self._backing.default_cursor_data = self.default_cursor_data
-        self._backing.set_cursor_data(self.cursor_data)
         return self._backing._backing
 
 
@@ -129,7 +125,7 @@ class ClientWindowBase(ClientWidgetBase):
         #(with a bit of magic to collapse the missing namespace from encoding_defaults)
         backing_props = self._backing.get_encoding_properties()
         encoding_defaults = self._client.encoding_defaults
-        for k in list(backing_props.keys()):
+        for k in tuple(backing_props.keys()):
             v = backing_props[k]
             try:
                 #ie: "encodings.rgb_formats" -> "rgb_formats"
@@ -148,28 +144,28 @@ class ClientWindowBase(ClientWidgetBase):
 
     def reset_icon(self):
         current_icon = self._current_icon
+        iconlog("reset_icon() current icon=%s", current_icon)
         if current_icon:
-            self.update_icon(*current_icon)
+            self.update_icon(current_icon)
 
-    def update_icon(self, width, height, coding, data):
-        raise Exception("override me!")
+    def update_icon(self, img):
+        raise NotImplementedError("override me!")
 
     def is_realized(self):
-        raise Exception("override me!")
+        raise NotImplementedError("override me!")
 
     def apply_transient_for(self, wid):
-        raise Exception("override me!")
+        raise NotImplementedError("override me!")
 
     def paint_spinner(self, context, area):
-        raise Exception("override me!")
+        raise NotImplementedError("override me!")
 
     def _pointer_modifiers(self, event):
-        raise Exception("override me!")
+        raise NotImplementedError("override me!")
 
 
     def xget_u32_property(self, target, name):
-        raise Exception("override me!")
-
+        raise NotImplementedError("override me!")
 
 
     def is_OR(self):
@@ -192,7 +188,8 @@ class ClientWindowBase(ClientWidgetBase):
         #WARNING: "class-instance" needs to go first because others may realize the window
         #(and GTK doesn't set the "class-instance" once the window is realized)
         if b"class-instance" in metadata:
-            self.set_class_instance(*self._metadata.strlistget("class-instance", ("xpra", "Xpra")))
+            self.set_class_instance(*self._metadata.strlistget("class-instance", ("xpra", "Xpra"), 2, 2))
+            self.reset_icon()
 
         if b"title" in metadata:
             try:
@@ -219,7 +216,8 @@ class ClientWindowBase(ClientWidgetBase):
                 else:
                     utf8_title = title
             except Exception as e:
-                log.error("error parsing window title: %s", e)
+                log.error("Error parsing window title:")
+                log.error(" %s", e)
                 utf8_title = b""
             self.set_title(utf8_title)
 
@@ -273,7 +271,13 @@ class ClientWindowBase(ClientWidgetBase):
         if b"has-alpha" in metadata:
             new_alpha = metadata.boolget("has-alpha")
             if new_alpha!=self._has_alpha:
-                log.warn("window %s changed its alpha flag from %s to %s (unsupported)", self._id, self._has_alpha, new_alpha)
+                l = metalog
+                if PYTHON3 and not WIN32:
+                    #win32 without opengl can't do transparency,
+                    #so it triggers too many warnings
+                    l = log.warn
+                l("Warning: window %s changed its transparency attribute", self._id)
+                l(" from %s to %s, behaviour is undefined", self._has_alpha, new_alpha)
                 self._has_alpha = new_alpha
 
         if b"maximized" in metadata:
@@ -462,7 +466,7 @@ class ClientWindowBase(ClientWidgetBase):
             self.geometry_hints = hints
             self.apply_geometry_hints(hints)
         except:
-            geomlog("set_size_constraints%s", (size_constraints, max_window_size))
+            geomlog("set_size_constraints%s", (size_constraints, max_window_size), exc_info=True)
             geomlog.error("Error setting window hints:")
             for k,v in hints.items():
                 geomlog.error(" %s=%s", k, v)
@@ -475,6 +479,14 @@ class ClientWindowBase(ClientWidgetBase):
     def set_window_type(self, window_types):
         hints = 0
         for window_type in window_types:
+            #win32 workaround:
+            if AWT_DIALOG_WORKAROUND and window_type=="DIALOG" and self._metadata.boolget("skip-taskbar"):
+                wm_class = self._metadata.strlistget("class-instance")
+                if wm_class and len(wm_class)==2 and wm_class[0].startswith("sun-awt-X11"):
+                    #replace "DIALOG" with "NORMAL":
+                    if "NORMAL" in window_types:
+                        continue
+                    window_type = "NORMAL"
             hint = self.NAME_TO_HINT.get(window_type, None)
             if hint is not None:
                 hints |= hint
@@ -494,7 +506,7 @@ class ClientWindowBase(ClientWidgetBase):
         pass
 
 
-    def toggle_debug(self, *args):
+    def toggle_debug(self, *_args):
         b = self._backing
         if not b:
             return
@@ -503,7 +515,7 @@ class ClientWindowBase(ClientWidgetBase):
         else:
             b.paint_box_line_width = b.default_paint_box_line_width
 
-    def increase_quality(self, *args):
+    def increase_quality(self, *_args):
         if self._client.quality>0:
             #change fixed quality:
             self._client.quality = min(100, self._client.quality + 10)
@@ -514,7 +526,7 @@ class ClientWindowBase(ClientWidgetBase):
             self._client.send_min_quality()
             log("new min-quality=%s", self._client.min_quality)
 
-    def decrease_quality(self, *args):
+    def decrease_quality(self, *_args):
         if self._client.quality>0:
             #change fixed quality:
             self._client.quality = max(1, self._client.quality - 10)
@@ -525,7 +537,7 @@ class ClientWindowBase(ClientWidgetBase):
             self._client.send_min_quality()
             log("new min-quality=%s", self._client.min_quality)
 
-    def increase_speed(self, *args):
+    def increase_speed(self, *_args):
         if self._client.speed>0:
             #change fixed speed:
             self._client.speed = min(100, self._client.speed + 10)
@@ -536,7 +548,7 @@ class ClientWindowBase(ClientWidgetBase):
             self._client.send_min_speed()
             log("new min-speed=%s", self._client.min_speed)
 
-    def decrease_speed(self, *args):
+    def decrease_speed(self, *_args):
         if self._client.speed>0:
             #change fixed speed:
             self._client.speed = max(1, self._client.speed - 10)
@@ -547,26 +559,30 @@ class ClientWindowBase(ClientWidgetBase):
             self._client.send_min_speed()
             log("new min-speed=%s", self._client.min_speed)
 
-    def scaleup(self, *args):
+    def scaleup(self, *_args):
         self._client.scaleup()
 
-    def scaledown(self, *args):
+    def scaledown(self, *_args):
         self._client.scaledown()
 
     def scalingoff(self):
         self._client.scalingoff()
 
-    def scalereset(self, *args):
+    def scalereset(self, *_args):
         self._client.scalereset()
 
     def magic_key(self, *args):
-        log.info("magic_key(%s) not handled in %s", args, type(self))
+        b = self.border
+        if b:
+            b.toggle()
+            log("magic_key%s border=%s", args, b)
+            self.queue_draw(0, 0, *self._size)
 
     def refresh_window(self, *args):
         log("refresh_window(%s) wid=%s", args, self._id)
         self._client.send_refresh(self._id)
 
-    def refresh_all_windows(self, *args):
+    def refresh_all_windows(self, *_args):
         #this method is only here because we may want to fire it
         #from a --key-shortcut action and the event is delivered to
         #the "ClientWindow"
@@ -587,17 +603,26 @@ class ClientWindowBase(ClientWidgetBase):
             backing = self._backing
             if backing and backing.draw_needs_refresh:
                 if REPAINT_ALL=="1" or self._client.xscale!=1 or self._client.yscale!=1:
-                    w, h = self.get_size()
-                    rect = 0, 0, w, h
+                    rw, rh = self.get_size()
+                    rx, ry = 0, 0
                 else:
-                    rect = self._client.srect(x, y, width, height)
-                self.idle_add(self.queue_draw, *rect)
+                    rx, ry, rw, rh = self._client.srect(x, y, width, height)
+                if self.window_offset:
+                    rx += self.window_offset[0]
+                    ry += self.window_offset[1]
+                self.idle_add(self.queue_draw, rx, ry, rw, rh)
         #only register this callback if we actually need it:
         if backing.draw_needs_refresh:
             callbacks.append(after_draw_refresh)
         backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
-    def spinner(self, ok):
+    def eos(self):
+        """ Note: this runs from the draw thread (not UI thread) """
+        backing = self._backing
+        if backing:
+            backing.eos()
+
+    def spinner(self, _ok):
         if not self.can_have_spinner():
             return
         log("spinner(%s) queueing redraw")
@@ -664,18 +689,21 @@ class ClientWindowBase(ClientWidgetBase):
         return self._client.rpc_call("dbus", rpc_args, **kwargs)
 
 
-    def get_mouse_event_wid(self, x, y):
+    def get_mouse_event_wid(self, _x, _y):
         #overriden in GTKClientWindowBase
         return self._id
 
     def do_motion_notify_event(self, event):
-        if self._client.readonly:
+        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
             return
-        pointer, modifiers, buttons = self._pointer_modifiers(event)
+        pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
         wid = self.get_mouse_event_wid(*pointer)
-        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
-        self._client.send_mouse_position(["pointer-position", wid,
-                                          pointer, modifiers, buttons])
+        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, relative pointer=%s, modifiers=%s, buttons=%s", event, wid, self._client._focused, self._id, self._device_info(event), pointer, relative_pointer, modifiers, buttons)
+        pdata = pointer
+        if self._client.server_pointer_relative:
+            pdata = list(pointer)+list(relative_pointer)
+        packet = ["pointer-position", wid, pdata, modifiers, buttons]
+        self._client.send_mouse_position(packet)
 
     def _device_info(self, event):
         try:
@@ -684,9 +712,9 @@ class ClientWindowBase(ClientWidgetBase):
             return ""
 
     def _button_action(self, button, event, depressed, *args):
-        if self._client.readonly:
+        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
             return
-        pointer, modifiers, buttons = self._pointer_modifiers(event)
+        pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
         wid = self.get_mouse_event_wid(*pointer)
         mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
         #map wheel buttons via translation table to support inverted axes:
@@ -703,8 +731,11 @@ class ClientWindowBase(ClientWidgetBase):
                     continue
                 b = sb
             server_buttons.append(b)
+        pdata = pointer
+        if self._client.server_pointer_relative:
+            pdata = list(pointer)+list(relative_pointer)
         def send_button(pressed):
-            self._client.send_button(wid, server_button, pressed, pointer, modifiers, server_buttons, *args)
+            self._client.send_button(wid, server_button, pressed, pdata, modifiers, server_buttons, *args)
         pressed_state = self.button_state.get(button, False)
         if SIMULATE_MOUSE_DOWN and pressed_state is False and depressed is False:
             mouselog("button action: simulating a missing mouse-down event for window %s before sending the mouse-up event", wid)
