@@ -1,47 +1,27 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2012 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import logging
-from xpra.util import envbool
-from xpra.os_util import OSX, WIN32, PYTHON3
+from xpra.util import envbool, csv
+from xpra.os_util import POSIX, OSX, WIN32, PYTHON3, bytestostr
 from xpra.log import Logger, CaptureHandler
+from xpra.client.gl.gl_drivers import WHITELIST, GREYLIST, VERSION_REQ, BLACKLIST
 log = Logger("opengl")
 
 required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
-WHITELIST = {
-    "renderer"  : ["Skylake", "Kabylake"],
-    }
-GREYLIST = {
-            "vendor"    : ["Intel", "Humper"]
-            }
-VERSION_REQ = {
-               "nouveau" : [3, 0],      #older versions have issues
-               }
-BLACKLIST = {
-             "renderer" : ["Software Rasterizer", "Mesa DRI Intel(R) Ivybridge Desktop"],
-             "vendor"    : ["VMware, Inc."]
-             }
-
-from xpra.os_util import getUbuntuVersion
-uv = getUbuntuVersion()
-if uv and uv<[15]:
-    #Ubuntu 14.x drivers are just too old
-    GREYLIST.setdefault("vendor", []).append("X.Org")
-if False:
-    #for testing:
-    GREYLIST["vendor"].append("NVIDIA Corporation")
-    WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
-
-    if OSX:
-        #frequent crashes on osx with GT 650M: (see ticket #808)
-        GREYLIST.setdefault("vendor", []).append("NVIDIA Corporation")
+#for testing:
+#GREYLIST["vendor"].append("NVIDIA Corporation")
+#WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
+#frequent crashes on OSX with GT 650M: (see ticket #808)
+#if OSX:
+#    GREYLIST.setdefault("vendor", []).append("NVIDIA Corporation")
 
 
 #alpha requires gtk3 or *nix only for gtk2:
@@ -53,78 +33,6 @@ CAN_DOUBLE_BUFFER = not PYTHON3
 DEFAULT_DOUBLE_BUFFERED = WIN32 or CAN_DOUBLE_BUFFER
 DOUBLE_BUFFERED = envbool("XPRA_OPENGL_DOUBLE_BUFFERED", DEFAULT_DOUBLE_BUFFERED)
 
-from xpra.gtk_common.gtk_util import STATIC_GRAY, GRAYSCALE, STATIC_COLOR, PSEUDO_COLOR, TRUE_COLOR, DIRECT_COLOR
-VISUAL_NAMES = {
-                STATIC_GRAY      : "STATIC_GRAY",
-                GRAYSCALE        : "GRAYSCALE",
-                STATIC_COLOR     : "STATIC_COLOR",
-                PSEUDO_COLOR     : "PSEUDO_COLOR",
-                TRUE_COLOR       : "TRUE_COLOR",
-                DIRECT_COLOR     : "DIRECT_COLOR",
-                }
-
-from xpra.gtk_common.gtk_util import LSB_FIRST, MSB_FIRST
-
-VISUAL_TYPES = {
-                LSB_FIRST   : "LSB",
-                MSB_FIRST   : "MSB",
-                }
-
-from xpra.client.gl.gtk_compat import MODE_RGBA, MODE_ALPHA, MODE_RGB, MODE_DOUBLE, MODE_SINGLE, MODE_DEPTH
-
-
-def get_visual_name(visual):
-    if not visual:
-        return ""
-    global VISUAL_NAMES
-    return VISUAL_NAMES.get(visual.type, "unknown")
-
-def get_visual_byte_order(visual):
-    if not visual:
-        return ""
-    global VISUAL_TYPES
-    return VISUAL_TYPES.get(visual.byte_order, "unknown")
-
-def visual_to_str(visual):
-    if not visual:
-        return ""
-    d = {
-         "type"         : get_visual_name(visual),
-         "byte_order"   : get_visual_byte_order(visual),
-         }
-    for k in ("bits_per_rgb", "depth"):
-        d[k] = getattr(visual, k)
-    return str(d)
-
-def get_DISPLAY_MODE(want_alpha=GL_ALPHA_SUPPORTED):
-    #MODE_DEPTH
-    if want_alpha:
-        mode = MODE_RGBA | MODE_ALPHA
-    else:
-        mode = MODE_RGB
-    if DOUBLE_BUFFERED:
-        mode = mode | MODE_DOUBLE
-    else:
-        mode = mode | MODE_SINGLE
-    return mode
-
-FRIENDLY_MODE_NAMES = {
-                       MODE_RGB         : "RGB",
-                       MODE_RGBA        : "RGBA",
-                       MODE_ALPHA       : "ALPHA",
-                       MODE_DEPTH       : "DEPTH",
-                       MODE_DOUBLE      : "DOUBLE",
-                       MODE_SINGLE      : "SINGLE",
-                       }
-
-def get_MODE_names(mode):
-    global FRIENDLY_MODE_NAMES
-    friendly_modes = [v for k,v in FRIENDLY_MODE_NAMES.items() if k>0 and (k&mode)==k]
-    #special case for single (value is zero!)
-    if not (mode&MODE_DOUBLE==MODE_DOUBLE):
-        friendly_modes.append("SINGLE")
-    return friendly_modes
-
 
 #by default, we raise an ImportError as soon as we find something missing:
 def raise_error(msg):
@@ -171,18 +79,12 @@ def check_functions(*functions):
         else:
             available.append(name)
     if len(missing)>0:
-        gl_check_error("some required OpenGL functions are not available: %s" % (", ".join(missing)))
+        gl_check_error("some required OpenGL functions are not available: %s" % csv(missing))
     else:
-        log("All the required OpenGL functions are available: %s " % (", ".join(available)))
+        log("All the required OpenGL functions are available: %s " % csv(available))
 
 
-#sanity checks: OpenGL version and fragment program support:
-def check_GL_support(widget, force_enable=False):
-    from xpra.client.gl.gtk_compat import GLContextManager
-    with GLContextManager(widget):
-        return do_check_GL_support(force_enable)
-
-def do_check_GL_support(force_enable):
+def check_PyOpenGL_support(force_enable):
     props = {}
     try:
         #log redirection:
@@ -208,8 +110,8 @@ def do_check_GL_support(force_enable):
         if gl_version_str is None:
             gl_check_error("OpenGL version is missing - cannot continue")
             return  {}
-        gl_major = int(gl_version_str[0])
-        gl_minor = int(gl_version_str[2])
+        gl_major = int(bytestostr(gl_version_str)[0])
+        gl_minor = int(bytestostr(gl_version_str)[2])
         props["opengl"] = gl_major, gl_minor
         MIN_VERSION = (1,1)
         if (gl_major, gl_minor) < MIN_VERSION:
@@ -234,6 +136,7 @@ def do_check_GL_support(force_enable):
             global _version_warning_shown
             if not _version_warning_shown:
                 log.warn("Warning: version mismatch between PyOpenGL and PyOpenGL-accelerate")
+                log.warn(" %s vs %s", pyopengl_version, accel_version)
                 log.warn(" this may cause crashes")
                 _version_warning_shown = True
         vsplit = pyopengl_version.split('.')
@@ -249,7 +152,7 @@ def do_check_GL_support(force_enable):
             log("error querying extensions", exc_info=True)
             extensions = []
             gl_check_error("OpenGL could not find the list of GL extensions - does the graphics driver support OpenGL?")
-        log("OpenGL extensions found: %s", ", ".join(extensions))
+        log("OpenGL extensions found: %s", csv(extensions))
         props["extensions"] = extensions
 
         from OpenGL.arrays.arraydatatype import ArrayDatatype
@@ -313,12 +216,10 @@ def do_check_GL_support(force_enable):
                 log.info("%s '%s' enabled (found in both blacklist and whitelist)", *whitelisted)
             elif force_enable:
                 log.warn("Warning: %s '%s' is blacklisted!", *blacklisted)
+                log.warn(" force enabled by option")
             else:
                 gl_check_error("%s '%s' is blacklisted!" % (blacklisted))
         safe = bool(whitelisted) or not bool(blacklisted)
-        if safe and PYTHON3:
-            log.warn("Warning: OpenGL python3 support is not enabled by default")
-            safe = False
         if greylisted and not whitelisted:
             log.warn("Warning: %s '%s' is greylisted,", *greylisted)
             log.warn(" you may want to turn off OpenGL if you encounter bugs")
@@ -403,12 +304,12 @@ def do_check_GL_support(force_enable):
 
         log("Texture size GL_MAX_RECTANGLE_TEXTURE_SIZE=%s, GL_MAX_TEXTURE_SIZE=%s", rect_texture_size, texture_size)
         texture_size_limit = min(rect_texture_size, texture_size)
-        props["texture-size-limit"] = texture_size_limit
+        props["texture-size-limit"] = int(texture_size_limit)
 
         try:
             from OpenGL.GL import GL_MAX_VIEWPORT_DIMS
             v = glGetIntegerv(GL_MAX_VIEWPORT_DIMS)
-            max_viewport_dims = v[0], v[1]
+            max_viewport_dims = int(v[0]), int(v[1])
             assert max_viewport_dims[0]>=texture_size_limit and max_viewport_dims[1]>=texture_size_limit
             log("GL_MAX_VIEWPORT_DIMS=%s", max_viewport_dims)
         except ImportError as e:
@@ -443,7 +344,7 @@ def do_check_GL_support(force_enable):
                 format_handler = format_handler[:p]
                 missing_handlers.append(format_handler)
         if len(missing_handlers)>0:
-            log.warn("PyOpenGL warning: missing array format handlers: %s", ", ".join(missing_handlers))
+            log.warn("PyOpenGL warning: missing array format handlers: %s", csv(missing_handlers))
 
         for x in elogger.handlers[0].records:
             msg = x.getMessage()
@@ -467,7 +368,7 @@ def do_check_GL_support(force_enable):
             else:
                 log.info(msg)
         if missing_accelerators:
-            log.info("OpenGL accelerate missing: %s", ", ".join(missing_accelerators))
+            log.info("OpenGL accelerate missing: %s", csv(missing_accelerators))
 
         def restore_logger(logger):
             logger.handlers = logger.saved_handlers
@@ -479,113 +380,7 @@ def do_check_GL_support(force_enable):
         restore_logger(clogger)
 
 
-def check_support(force_enable=False, check_colormap=False):
-    #platform checks:
-    from xpra.platform.gui import gl_check
-    warning = gl_check()
-    if warning:
-        if force_enable:
-            log.warn("Warning: trying to continue despite '%s'" % warning)
-        else:
-            gl_check_error(warning)
-
-    props = {}
-    #this will import gtk.gtkgl / gdkgl or gi.repository.GtkGLExt / GdkGLExt:
-    try:
-        from xpra.client.gl.gtk_compat import get_info, gdkgl, Config_new_by_mode, GLDrawingArea
-    except RuntimeError as e:
-        gl_check_error(str(e))
-        return {}
-    props.update(get_info())
-    display_mode = get_DISPLAY_MODE()
-    glconfig = Config_new_by_mode(display_mode)
-    if glconfig is None and CAN_DOUBLE_BUFFER:
-        log("trying to toggle double-buffering")
-        display_mode &= ~MODE_DOUBLE
-        glconfig = Config_new_by_mode(display_mode)
-    if not glconfig:
-        gl_check_error("cannot setup an OpenGL context")
-        return {}
-    props["display_mode"] = get_MODE_names(display_mode)
-    #on OSX, we had to patch out get_depth...
-    #so take extra precautions when querying properties:
-    for x,fn_name in {
-        "has_alpha"           : "has_alpha",
-        "rgba"                : "is_rgba",
-        "stereo"              : "is_stereo",
-        "double-buffered"     : "is_double_buffered",
-        "depth"               : "get_depth",
-        "has-depth-buffer"    : "has_depth_buffer",
-        "has-stencil-buffer"  : "has_stencil_buffer",
-        }.items():
-        fn = getattr(glconfig, fn_name, None)
-        if fn:
-            props[x] = fn()
-        else:
-            log("%s does not support %s()", glconfig, fn_name)
-    for x in ("RED_SIZE", "GREEN_SIZE", "BLUE_SIZE", "ALPHA_SIZE",
-              "AUX_BUFFERS", "DEPTH_SIZE", "STENCIL_SIZE",
-              "ACCUM_RED_SIZE", "ACCUM_GREEN_SIZE", "ACCUM_BLUE_SIZE",
-              "SAMPLE_BUFFERS", "SAMPLES"):
-        prop = getattr(gdkgl, x)
-        if not prop:
-            continue
-        try:
-            v = glconfig.get_attrib(prop)[0]
-            props[x.lower().replace("_", "-")] = v
-        except:
-            pass
-    log("GL props=%s", props)
-
-    from xpra.gtk_common.gtk_util import import_gtk, gdk_window_process_all_updates
-    gtk = import_gtk()
-    assert gdkgl.query_extension()
-    glext, w = None, None
-    try:
-        #ugly code for win32 and others (virtualbox broken GL drivers)
-        #for getting a GL drawable and context: we must use a window...
-        #(which we do not even show on screen)
-        #
-        #here is the old simpler alternative which does not work on some platforms:
-        # glext = gtk.gdkgl.ext(gdk.Pixmap(gdk.get_default_root_window(), 1, 1))
-        # gldrawable = glext.set_gl_capability(glconfig)
-        # glcontext = gtk.gdkgl.Context(gldrawable, direct=True)
-        w = gtk.Window()
-        w.set_decorated(False)
-        vbox = gtk.VBox()
-        glarea = GLDrawingArea(glconfig)
-        glarea.set_size_request(32, 32)
-        vbox.add(glarea)
-        vbox.show_all()
-        w.add(vbox)
-        #we don't need to actually show the window!
-        #w.show_all()
-        glarea.realize()
-        gdk_window_process_all_updates()
-
-        gl_props = check_GL_support(glarea, force_enable)
-
-        if check_colormap:
-            s = w.get_screen()
-            for x in ("rgb_visual", "rgba_visual", "system_visual"):
-                try:
-                    visual = getattr(s, "get_%s" % x)()
-                    gl_props[x] = visual_to_str(visual)
-                except:
-                    pass
-            #i = 0
-            #for v in s.list_visuals():
-            #    gl_props["visual[%s]" % i] = visual_to_str(v)
-            #    i += 1
-    finally:
-        if w:
-            w.destroy()
-        del glext, glconfig
-    props.update(gl_props)
-    return props
-
-
-def main(force_enable=False):
+def main():
     from xpra.platform import program_context
     from xpra.platform.gui import init as gui_init
     from xpra.util import print_nested_dict
@@ -596,8 +391,14 @@ def main(force_enable=False):
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
         if verbose:
             log.enable_debug()
-            from xpra.client.gl.gtk_compat import log as clog
-            clog.enable_debug()
+        if POSIX and not OSX:
+            from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+            init_display_source()
+        force_enable = "-f" in sys.argv or "--force" in sys.argv
+        from xpra.platform.gl_context import GLContext
+        log("testing %s", GLContext)
+        gl_context = GLContext()
+        log("GLContext=%s", gl_context)
         #replace ImportError with a log message:
         global gl_check_error
         errors = []
@@ -605,15 +406,21 @@ def main(force_enable=False):
             log.error("ERROR: %s", msg)
             errors.append(msg)
         gl_check_error = log_error
-        props = check_support(force_enable, verbose)
+        try:
+            props = gl_context.check_support(force_enable)
+        except Exception as e:
+            props = {}
+            log("check_support", exc_info=True)
+            errors.append(e)
         log.info("")
         if len(errors)>0:
             log.info("OpenGL errors:")
             for e in errors:
                 log.info("  %s", e)
-        log.info("")
-        log.info("OpenGL properties:")
-        print_nested_dict(props)
+        if props:
+            log.info("")
+            log.info("OpenGL properties:")
+            print_nested_dict(props)
         return len(errors)
 
 
