@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2013 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,7 +8,7 @@ import os
 import struct
 import time, math
 
-from xpra.os_util import monotonic_time, OSX, POSIX, DummyContextManager
+from xpra.os_util import monotonic_time, strtobytes, hexstr, POSIX, DummyContextManager
 from xpra.util import envint, envbool, repr_ellipsized
 from xpra.log import Logger
 log = Logger("opengl", "paint")
@@ -17,8 +17,9 @@ fpslog = Logger("opengl", "fps")
 OPENGL_DEBUG = envbool("XPRA_OPENGL_DEBUG", False)
 SCROLL_ENCODING = envbool("XPRA_SCROLL_ENCODING", True)
 PAINT_FLUSH = envbool("XPRA_PAINT_FLUSH", True)
-HIGH_BIT_DEPTH = envbool("XPRA_HIGH_BIT_DEPTH", True)
-FORCE_HIGH_BIT_DEPTH = envbool("XPRA_FORCE_HIGH_BIT_DEPTH", False)
+JPEG_YUV = envbool("XPRA_JPEG_YUV", True)
+WEBP_YUV = envbool("XPRA_WEBP_YUV", True)
+FORCE_CLONE = envbool("XPRA_OPENGL_FORCE_CLONE", False)
 
 CURSOR_IDLE_TIMEOUT = envint("XPRA_CURSOR_IDLE_TIMEOUT", 6)
 TEXTURE_CURSOR = envbool("XPRA_OPENGL_TEXTURE_CURSOR", False)
@@ -32,16 +33,13 @@ if SAVE_BUFFERS:
     import numpy
     from PIL import Image, ImageOps
 
-from xpra.gtk_common.gtk_util import is_realized
-from xpra.gtk_common.paint_colors import get_paint_box_color
+from xpra.client.paint_colors import get_paint_box_color
 from xpra.codecs.codec_constants import get_subsampling_divs
-from xpra.client.window_backing_base import fire_paint_callbacks
-from xpra.gtk_common.gtk_util import POINTER_MOTION_MASK, POINTER_MOTION_HINT_MASK
-from xpra.gtk_common.gtk_spinner import cv
-from xpra.client.gtk_base.gtk_window_backing_base import GTKWindowBacking
-from xpra.client.gl.gtk_compat import Config_new_by_mode, MODE_DOUBLE, GLContextManager, GLDrawingArea
-from xpra.client.gl.gl_check import get_DISPLAY_MODE, GL_ALPHA_SUPPORTED, CAN_DOUBLE_BUFFER, is_pyopengl_memoryview_safe
-from xpra.client.gl.gl_colorspace_conversions import YUV2RGB_shader, RGBP2RGB_shader
+from xpra.client.window_backing_base import fire_paint_callbacks, WEBP_PILLOW
+from xpra.client.spinner import cv
+from xpra.client.window_backing_base import WindowBackingBase
+from xpra.client.gl.gl_check import GL_ALPHA_SUPPORTED, is_pyopengl_memoryview_safe
+from xpra.client.gl.gl_colorspace_conversions import YUV2RGB_shader, YUV2RGB_FULL_shader, RGBP2RGB_shader
 from OpenGL import version as OpenGL_version
 from OpenGL.error import GLError
 from OpenGL.GL import \
@@ -54,11 +52,11 @@ from OpenGL.GL import \
     GL_DONT_CARE, GL_TRUE, GL_DEPTH_TEST, GL_SCISSOR_TEST, GL_LIGHTING, GL_DITHER, \
     GL_RGB, GL_RGBA, GL_BGR, GL_BGRA, GL_RGBA8, GL_RGB8, GL_RGB10_A2, GL_RGB565, GL_RGB5_A1, GL_RGBA4, \
     GL_UNSIGNED_INT_2_10_10_10_REV, GL_UNSIGNED_INT_10_10_10_2, GL_UNSIGNED_SHORT_5_6_5, \
-    GL_BLEND, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA, \
+    GL_BLEND, GL_ONE, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA, \
     GL_TEXTURE_MAX_LEVEL, GL_TEXTURE_BASE_LEVEL, \
     GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST, \
     glLineStipple, GL_LINE_STIPPLE, GL_POINTS, \
-    glTexEnvi, GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE, GL_TEXTURE_2D, \
+    glTexEnvi, GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE, \
     glHint, \
     glBlendFunc, \
     glActiveTexture, glTexSubImage2D, \
@@ -78,7 +76,6 @@ from OpenGL.GL.ARB.fragment_program import GL_FRAGMENT_PROGRAM_ARB
 from OpenGL.GL.ARB.framebuffer_object import GL_FRAMEBUFFER, GL_DRAW_FRAMEBUFFER, GL_READ_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1, \
     glGenFramebuffers, glBindFramebuffer, glFramebufferTexture2D, glBlitFramebuffer
 
-from ctypes import c_uint
 
 PIXEL_FORMAT_TO_CONSTANT = {
     "r210"  : GL_BGRA,
@@ -143,7 +140,7 @@ if OPENGL_DEBUG:
         from OpenGL.GL.GREMEDY.frame_terminator import glInitFrameTerminatorGREMEDY, glFrameTerminatorGREMEDY
         from OpenGL.GL import GLDEBUGPROC #@UnresolvedImport
         def py_gl_debug_callback(source, error_type, error_id, severity, length, message, param):
-            log.error("src %x type %x id %x severity %x length %d message %s", source, error_type, error_id, severity, length, message)
+            log.error("src %x type %x id %x severity %x length %d message %s, param=%s", source, error_type, error_id, severity, length, message, param)
         gl_debug_callback = GLDEBUGPROC(py_gl_debug_callback)
     except ImportError:
         # This is normal- GREMEDY_string_marker is only available with OpenGL debuggers
@@ -203,6 +200,7 @@ else:
 # Shader number assignment
 YUV2RGB_SHADER = 0
 RGBP2RGB_SHADER = 1
+YUV2RGB_FULL_SHADER = 2
 
 """
 The logic is as follows:
@@ -213,15 +211,13 @@ textured quad when requested, that is: after each YUV or RGB painting operation,
 The use of a intermediate framebuffer object is the only way to guarantee that the client keeps an always fully up-to-date
 window image, which is critical because of backbuffer content losses upon buffer swaps or offscreen window movement.
 """
-class GLWindowBackingBase(GTKWindowBacking):
+class GLWindowBackingBase(WindowBackingBase):
 
     RGB_MODES = ["YUV420P", "YUV422P", "YUV444P", "GBRP", "BGRA", "BGRX", "RGBA", "RGBX", "RGB", "BGR"]
     HAS_ALPHA = GL_ALPHA_SUPPORTED
 
     def __init__(self, wid, window_alpha, pixel_depth=0):
         self.wid = wid
-        self.size = 0, 0
-        self.render_size = 0, 0
         self.texture_pixel_format = None
         #this is the pixel format we are currently updating the fbo with
         #can be: "YUV420P", "YUV422P", "YUV444P", "GBRP" or None when not initialized yet.
@@ -239,7 +235,8 @@ class GLWindowBackingBase(GTKWindowBacking):
         self.pending_fbo_paint = []
         self.last_flush = monotonic_time()
 
-        GTKWindowBacking.__init__(self, wid, window_alpha)
+        WindowBackingBase.__init__(self, wid, window_alpha and self.HAS_ALPHA)
+
         self.init_gl_config(window_alpha)
         self.init_backing()
         self.bit_depth = self.get_bit_depth(pixel_depth)
@@ -248,51 +245,23 @@ class GLWindowBackingBase(GTKWindowBacking):
         self._backing.show()
 
     def init_gl_config(self, window_alpha):
-        #setup gl config:
-        alpha = GL_ALPHA_SUPPORTED and window_alpha
-        display_mode = get_DISPLAY_MODE(want_alpha=alpha)
-        self.glconfig = Config_new_by_mode(display_mode)
-        if self.glconfig is None and CAN_DOUBLE_BUFFER:
-            log("trying to toggle double-buffering")
-            display_mode &= ~MODE_DOUBLE
-            self.glconfig = Config_new_by_mode(display_mode)
-        if not self.glconfig:
-            raise Exception("cannot setup an OpenGL context")
+        raise NotImplementedError()
 
     def init_backing(self):
-        self._backing = GLDrawingArea(self.glconfig)
-        #must be overriden in subclasses to setup self._backing
-        assert self._backing
-        log("init_backing() backing=%s, alpha_enabled=%s", self._backing, self._alpha_enabled)
-        if self._alpha_enabled:
-            assert GL_ALPHA_SUPPORTED, "BUG: cannot enable alpha if GL backing does not support it!"
-            screen = self._backing.get_screen()
-            rgba = screen.get_rgba_colormap()
-            display = screen.get_display()
-            if not display.supports_composite() and not OSX:
-                log.warn("display %s does not support compositing, transparency disabled", display.get_name())
-                self._alpha_enabled = False
-            elif rgba:
-                log("%s.__init__() using rgba colormap %s", self, rgba)
-                self._backing.set_colormap(rgba)
-            else:
-                log.warn("Warning: failed to enable transparency, no RGBA colormap")
-                self._alpha_enabled = False
-        self._backing.set_events(self._backing.get_events() | POINTER_MOTION_MASK | POINTER_MOTION_HINT_MASK)
+        raise NotImplementedError()
+
+    def gl_context(self):
+        raise NotImplementedError()
+
+    def do_gl_show(self, rect_count):
+        raise NotImplementedError()
+
+    def is_double_buffered(self):
+        raise NotImplementedError()
+
 
     def get_bit_depth(self, pixel_depth=0):
-        gl_depth = self.glconfig.get_depth()
-        log("get_bit_depth() glconfig depth=%i, HIGH_BIT_DEPTH=%s, requested pixel depth=%i", gl_depth, HIGH_BIT_DEPTH, pixel_depth)
-        bit_depth = 24
-        if HIGH_BIT_DEPTH:
-            if pixel_depth==0:
-                #auto detect
-                if POSIX and gl_depth>=24:
-                    bit_depth = gl_depth
-            elif pixel_depth>0:
-                bit_depth = pixel_depth
-        log("get_bit_depth()=%i", bit_depth)
-        return bit_depth
+        return pixel_depth or 24
 
     def init_formats(self):
         self.RGB_MODES = list(GLWindowBackingBase.RGB_MODES)
@@ -325,7 +294,7 @@ class GLWindowBackingBase(GTKWindowBacking):
         log("init_formats() texture pixel format=%s, internal format=%s, rgb modes=%s", ["GL_RGB", "GL_RGBA"][self._alpha_enabled], INTERNAL_FORMAT_TO_STR.get(self.internal_format), self.RGB_MODES)
 
     def get_encoding_properties(self):
-        props = GTKWindowBacking.get_encoding_properties(self)
+        props = WindowBackingBase.get_encoding_properties(self)
         if SCROLL_ENCODING:
             props["encoding.scrolling"] = True
         props["encoding.bit-depth"] = self.bit_depth
@@ -391,25 +360,22 @@ class GLWindowBackingBase(GTKWindowBacking):
     def gl_init_textures(self):
         assert self.offscreen_fbo is None
         assert self.shaders is None
+        assert glGenFramebuffers, "no framebuffer support"
         self.textures = glGenTextures(N_TEXTURES)
-        self.offscreen_fbo = self._gen_fbo()
-        self.tmp_fbo = self._gen_fbo()
+        self.offscreen_fbo = glGenFramebuffers(1)
+        self.tmp_fbo = glGenFramebuffers(1)
         log("%s.gl_init_textures() textures: %s, offscreen fbo: %s, tmp fbo: %s", self, self.textures, self.offscreen_fbo, self.tmp_fbo)
 
-    def _gen_fbo(self):
-        if hasattr(glGenFramebuffers, "pyConverters") and len(glGenFramebuffers.pyConverters)==1:
-            #single argument syntax:
-            return glGenFramebuffers(1)
-        fbo = c_uint(1)
-        glGenFramebuffers(1, fbo)
-        return fbo
-
     def gl_init_shaders(self):
         assert self.shaders is None
         # Create and assign fragment programs
-        self.shaders = [ 1, 2 ]
-        glGenProgramsARB(2, self.shaders)
-        for name, progid, progstr in (("YUV2RGB", YUV2RGB_SHADER, YUV2RGB_shader), ("RGBP2RGB", RGBP2RGB_SHADER, RGBP2RGB_shader)):
+        self.shaders = [ 1, 2, 3 ]
+        glGenProgramsARB(3, self.shaders)
+        for name, progid, progstr in (
+            ("YUV2RGB",     YUV2RGB_SHADER,         YUV2RGB_shader),
+            ("YUV2RGBFULL", YUV2RGB_FULL_SHADER,    YUV2RGB_FULL_shader),
+            ("RGBP2RGB",    RGBP2RGB_SHADER,        RGBP2RGB_shader),
+            ):
             glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, self.shaders[progid])
             glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, len(progstr), progstr)
             err = glGetString(GL_PROGRAM_ERROR_STRING_ARB)
@@ -420,26 +386,6 @@ class GLWindowBackingBase(GTKWindowBacking):
             else:
                 log("%s shader initialized", name)
 
-    def gl_context(self):
-        b = self._backing
-        if not b:
-            log("cannot get an OpenGL context: no backing defined")
-            return None
-        if not is_realized(b):
-            log.error("Error: OpenGL backing %s is not realized", b)
-            return None
-        w, h = self.size
-        if w<=0 or h<=0:
-            log.error("Error: invalid OpenGL backing size: %ix%i", w, h)
-            return None
-        try:
-            context = GLContextManager(b)
-        except Exception as e:
-            log.error("Error: %s", e)
-            return None
-        log("%s.gl_context() GL Pixmap backing size: %d x %d, context=%s", self, w, h, context)
-        return context
-
     def gl_init(self):
         #must be called within a context!
         #performs init if needed
@@ -524,9 +470,9 @@ class GLWindowBackingBase(GTKWindowBacking):
         if b:
             self._backing = None
             b.destroy()
-        self.glconfig = None
 
-    def paint_scroll(self, x, y, w, h, scroll_data, options, callbacks):
+
+    def paint_scroll(self, scroll_data, options, callbacks):
         flush = options.intget("flush", 0)
         self.idle_add(self.do_scroll_paints, scroll_data, flush, callbacks)
 
@@ -617,8 +563,8 @@ class GLWindowBackingBase(GTKWindowBacking):
             glBindTexture(target, 0)
             glDisable(target)
             self.paint_box("scroll", True, x+xdelta, y+ydelta, x+w+xdelta, y+h+ydelta)
-            self.present_fbo(0, 0, bw, bh, flush)
             fire_paint_callbacks(callbacks, True)
+            self.present_fbo(0, 0, bw, bh, flush)
 
     def present_fbo(self, x, y, w, h, flush=0):
         log("present_fbo: adding %s to pending paint list (size=%i), flush=%s, paint_screen=%s", (x, y, w, h), len(self.pending_fbo_paint), flush, self.paint_screen)
@@ -639,7 +585,7 @@ class GLWindowBackingBase(GTKWindowBacking):
         bw, bh = self.size
         ww, wh = self.render_size
         rect_count = len(self.pending_fbo_paint)
-        if self.glconfig.is_double_buffered() or bw!=ww or bh!=wh:
+        if self.is_double_buffered() or bw!=ww or bh!=wh:
             #refresh the whole window:
             rectangles = ((0, 0, bw, bh), )
         else:
@@ -656,16 +602,21 @@ class GLWindowBackingBase(GTKWindowBacking):
         glBindFramebuffer(GL_FRAMEBUFFER, 0)
         glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0)
 
+        left, top, right, bottom = self.offsets
+
+        #viewport for clearing the whole window:
+        glViewport(0, 0, left+ww+right, top+wh+bottom)
         if self._alpha_enabled:
             # transparent background:
             glClearColor(0.0, 0.0, 0.0, 0.0)
         else:
-            # plain white no alpha:
-            glClearColor(1.0, 1.0, 1.0, 1.0)
+            # black, no alpha:
+            glClearColor(0.0, 0.0, 0.0, 1.0)
+        if left or top or right or bottom:
+            glClear(GL_COLOR_BUFFER_BIT)
 
-        #viewport for painting to window:
-        x, _, _, y = self.offsets
-        glViewport(x, y, ww, wh)
+        #from now on, take the offsets into account:
+        glViewport(left, top, ww, wh)
         target = GL_TEXTURE_RECTANGLE_ARB
         if ww!=bw or wh!=bh:
             glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
@@ -707,6 +658,7 @@ class GLWindowBackingBase(GTKWindowBacking):
             self.draw_border()
 
         # Show the backbuffer on screen
+        glFlush()
         self.gl_show(rect_count)
         self.gl_frame_terminator()
 
@@ -747,14 +699,15 @@ class GLWindowBackingBase(GTKWindowBacking):
         glDisable(GL_TEXTURE_RECTANGLE_ARB)
 
     def draw_pointer(self):
-        x, y, _, _, size, start_time = self.pointer_overlay
+        px, py, _, _, size, start_time = self.pointer_overlay
         elapsed = monotonic_time()-start_time
         log("pointer_overlay=%s, elapsed=%.1f, timeout=%s, cursor-data=%s", self.pointer_overlay, elapsed, CURSOR_IDLE_TIMEOUT, (self.cursor_data or [])[:7])
         if elapsed>=CURSOR_IDLE_TIMEOUT:
             #timeout - stop showing it:
             self.pointer_overlay = None
             return
-
+        x = px
+        y = py
         if not self.cursor_data:
             #paint a fake one:
             alpha = max(0, (5.0-elapsed)/5.0)
@@ -771,37 +724,37 @@ class GLWindowBackingBase(GTKWindowBacking):
 
         cw = self.cursor_data[3]
         ch = self.cursor_data[4]
+        xhot = self.cursor_data[5]
+        yhot = self.cursor_data[6]
+        x = px-xhot
+        y = py-yhot
         if TEXTURE_CURSOR:
             #paint the texture containing the cursor:
-            #glActiveTexture(GL_TEXTURE1)
-            target = GL_TEXTURE_2D
+            glActiveTexture(GL_TEXTURE0)
+            target = GL_TEXTURE_RECTANGLE_ARB
             glEnable(target)
             glBindTexture(target, self.textures[TEX_CURSOR])
-            self.upload_cursor_texture(target, self.cursor_data)
-
-            #glEnablei(GL_BLEND, self.textures[TEX_CURSOR])
-            #glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
-            #from OpenGL.GL import GL_REPLACE, GL_TEXTURE_2D, GL_COMBINE, GL_DECAL, GL_MODULATE, GL_ADD
-            #glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE)  #GL_REPLACE, GL_BLEND, GL_MODULATE
+            glEnablei(GL_BLEND, self.textures[TEX_CURSOR])
+            glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA)
+            glBlendFunc(GL_ONE, GL_ONE)
+            glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)
 
             glBegin(GL_QUADS)
-            glTexCoord2i(0, 0)
-            glVertex2i(x, y)
-            glTexCoord2i(0, 1)
-            glVertex2i(x, y+ch)
-            glTexCoord2i(1, 1)
-            glVertex2i(x+cw, y+ch)
-            glTexCoord2i(1, 0)
-            glVertex2i(x+cw, y)
+            glTexCoord2i(0, 0);     glVertex2i(x, y)
+            glTexCoord2i(0, ch);    glVertex2i(x, y+ch)
+            glTexCoord2i(cw, ch);   glVertex2i(x+cw, y+ch)
+            glTexCoord2i(cw, 0);    glVertex2i(x+cw, y)
             glEnd()
 
             glBindTexture(target, 0)
             glDisable(target)
-            #glActiveTexture(GL_TEXTURE0)
         else:
             #FUGLY: paint each pixel separately..
+            if not self.validate_cursor():
+                return
             pixels = self.cursor_data[8]
-            p = struct.unpack("B"*(cw*ch*4), pixels)
+            blen = cw*ch*4
+            p = struct.unpack(b"B"*blen, pixels)
             glLineWidth(1)
             #TODO: use VBO arrays to make this faster
             for cx in range(cw):
@@ -813,27 +766,6 @@ class GLWindowBackingBase(GTKWindowBacking):
                         glVertex2i(x+cx, y+cy)
                         glEnd()
 
-    def upload_cursor_texture(self, target, cursor_data):
-        width = cursor_data[3]
-        height = cursor_data[4]
-        pixels = cursor_data[8]
-        if len(pixels)<width*4*height:
-            import binascii
-            log.error("Error: invalid cursor pixel buffer for %ix%i", width, height)
-            log.error(" expected %i bytes but got %i", width*height*4, len(pixels))
-            log.error(" %s", repr_ellipsized(binascii.hexlify(pixels)))
-            return
-        upload, pixel_data = self.pixels_for_upload(pixels)
-        rgb_format = "BGRA"
-        self.set_alignment(width, width*4, rgb_format)
-        glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
-        glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
-        set_texture_level(target)
-        glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER)
-        glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER)
-        glTexImage2D(target, 0, GL_RGBA8, width, height, 0, GL_BGRA, GL_UNSIGNED_BYTE, pixel_data)
-        log("GL cursor %ix%i uploaded %i bytes of %s pixel data using %s", width, height, len(pixels), rgb_format, upload)
-
     def draw_spinner(self):
         bw, bh = self.size
         dim = min(bw/3.0, bh/3.0)
@@ -860,6 +792,7 @@ class GLWindowBackingBase(GTKWindowBacking):
     def draw_border(self):
         bw, bh = self.size
         #double size since half the line will be off-screen
+        log("draw_border: %s", self.border)
         glLineWidth(self.border.size*2)
         glBegin(GL_LINE_LOOP)
         glColor4f(self.border.red, self.border.green, self.border.blue, self.border.alpha)
@@ -867,30 +800,57 @@ class GLWindowBackingBase(GTKWindowBacking):
             glVertex2i(px, py)
         glEnd()
 
+
+    def validate_cursor(self):
+        cursor_data = self.cursor_data
+        cw = cursor_data[3]
+        ch = cursor_data[4]
+        pixels = cursor_data[8]
+        blen = cw*ch*4
+        if len(pixels)!=blen:
+            log.error("Error: invalid cursor pixel buffer for %ix%i", cw, ch)
+            log.error(" expected %i bytes but got %i (%s)", blen, len(pixels), type(pixels))
+            log.error(" %s", repr_ellipsized(hexstr(pixels)))
+            return False
+        return True
+
     def set_cursor_data(self, cursor_data):
         if (not cursor_data or len(cursor_data)==1) and self.default_cursor_data:
             cursor_data = ["raw"] + self.default_cursor_data
         if not cursor_data:
             return
         self.cursor_data = cursor_data
+        if not cursor_data or not TEXTURE_CURSOR:
+            return
+        cw = cursor_data[3]
+        ch = cursor_data[4]
+        pixels = cursor_data[8]
+        if not self.validate_cursor():
+            return
+        context = self.gl_context()
+        if not context:
+            return
+        with context:
+            self.gl_init()
+            self.upload_cursor_texture(cw, ch, pixels)
 
-
-    def gl_show(self, rect_count):
-        start = monotonic_time()
-        if self.glconfig.is_double_buffered():
-            # Show the backbuffer on screen
-            log("%s.gl_show() swapping buffers now", self)
-            gldrawable = self.get_gl_drawable()
-            gldrawable.swap_buffers()
-        else:
-            #just ensure stuff gets painted:
-            log("%s.gl_show() flushing", self)
-            glFlush()
-        end = monotonic_time()
-        flush_elapsed = end-self.last_flush
-        self.last_flush = end
-        fpslog("gl_show after %3ims took %2ims, %2i updates", flush_elapsed*1000, (end-start)*1000, rect_count)
-
+    def upload_cursor_texture(self, width, height, pixels):
+        upload, pixel_data = self.pixels_for_upload(pixels)
+        rgb_format = "RGBA"
+        glActiveTexture(GL_TEXTURE0)
+        target = GL_TEXTURE_RECTANGLE_ARB
+        glEnable(target)
+        glBindTexture(target, self.textures[TEX_CURSOR])
+        self.set_alignment(width, width*4, rgb_format)
+        glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
+        glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
+        set_texture_level(target)
+        glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER)
+        glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER)
+        glTexImage2D(target, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixel_data)
+        log("GL cursor %ix%i uploaded %i bytes of %s pixel data using %s", width, height, len(pixels), rgb_format, upload)
+        glBindTexture(target, 0)
+        glDisable(target)
 
     def paint_box(self, encoding, is_delta, x, y, w, h):
         #show region being painted if debug paint box is enabled only:
@@ -919,17 +879,16 @@ class GLWindowBackingBase(GTKWindowBacking):
                 #not safe, make a copy :(
                 return "copy:memoryview.tobytes", img_data.tobytes()
             return "zerocopy:memoryview", img_data
-        elif t in (str, buffer_type) and zerocopy_upload:
+        elif t in (bytes, buffer_type) and zerocopy_upload:
             #we can zerocopy if we wrap it:
             return "zerocopy:buffer-as-memoryview", memoryview(img_data)
-        elif t!=str:
+        elif t==bytes:
+            return "copy:bytes", img_data
+        else:
             if hasattr(img_data, "raw"):
                 return "zerocopy:mmap", img_data.raw
             #everything else.. copy to bytes (aka str):
-            return "copy:str(%s)" % t, str(img_data)
-        else:
-            #str already
-            return  "copy:str", img_data
+            return "copy:bytes(%s)" % t, strtobytes(img_data)
 
     def set_alignment(self, width, rowstride, pixel_format):
         bytes_per_pixel = len(pixel_format)       #ie: BGRX -> 4
@@ -951,11 +910,23 @@ class GLWindowBackingBase(GTKWindowBacking):
 
 
     def paint_jpeg(self, img_data, x, y, width, height, options, callbacks):
-        #img = self.jpeg_decoder.decompress_to_yuv(img_data, width, height, options)
-        #self.idle_add(self.gl_paint_planar, flush, "jpeg", img, x, y, width, height, width, height, callbacks)
-        img = self.jpeg_decoder.decompress_to_rgb("BGRX", img_data, width, height, options)
-        self.idle_add(self.do_paint_rgb, "BGRX", img.get_pixels(), x, y, width, height, img.get_rowstride(), options, callbacks)
+        if JPEG_YUV and width>=2 and height>=2:
+            img = self.jpeg_decoder.decompress_to_yuv(img_data, width, height, options)
+            flush = options.intget("flush", 0)
+            self.idle_add(self.gl_paint_planar, YUV2RGB_FULL_SHADER, flush, "jpeg", img, x, y, width, height, width, height, callbacks)
+        else:
+            img = self.jpeg_decoder.decompress_to_rgb("BGRX", img_data, width, height, options)
+            self.idle_add(self.do_paint_rgb, "BGRX", img.get_pixels(), x, y, width, height, img.get_rowstride(), options, callbacks)
 
+    def paint_webp(self, img_data, x, y, width, height, options, callbacks):
+        subsampling = options.strget("subsampling")
+        has_alpha = options.boolget("has_alpha")
+        if subsampling=="YUV420P" and WEBP_YUV and self.webp_decoder and not WEBP_PILLOW and not has_alpha and width>=2 and height>=2:
+            img = self.webp_decoder.decompress_yuv(img_data)
+            flush = options.intget("flush", 0)
+            self.idle_add(self.gl_paint_planar, YUV2RGB_SHADER, flush, "webp", img, x, y, width, height, width, height, callbacks)
+            return
+        WindowBackingBase.paint_webp(self, img_data, x, y, width, height, options, callbacks)
 
     def do_paint_rgb(self, rgb_format, img_data, x, y, width, height, rowstride, options, callbacks):
         log("%s.do_paint_rgb(%s, %s bytes, x=%d, y=%d, width=%d, height=%d, rowstride=%d, options=%s)", self, rgb_format, len(img_data), x, y, width, height, rowstride, options)
@@ -964,7 +935,7 @@ class GLWindowBackingBase(GTKWindowBacking):
             log("%s._do_paint_rgb(..) no context!", self)
             fire_paint_callbacks(callbacks, False, "no opengl context")
             return
-        if not options.get("paint", True):
+        if not options.boolget("paint", True):
             fire_paint_callbacks(callbacks)
             return
         try:
@@ -1008,26 +979,32 @@ class GLWindowBackingBase(GTKWindowBacking):
 
                 glBindTexture(target, 0)
                 glDisable(target)
-                self.paint_box(options.get("encoding"), options.get("delta", -1)>=0, x, y, width, height)
-
+                self.paint_box(options.strget("encoding"), options.intget("delta", -1)>=0, x, y, width, height)
+                fire_paint_callbacks(callbacks)
                 # Present update to screen
-                self.present_fbo(x, y, width, height, options.get("flush", 0))
+                self.present_fbo(x, y, width, height, options.intget("flush", 0))
                 # present_fbo has reset state already
-            fire_paint_callbacks(callbacks)
             return
         except GLError as e:
-            message = b"OpenGL %s paint failed: %r" % (rgb_format, e)
+            message = "OpenGL %s paint failed: %r" % (rgb_format, e)
+            log("Error in %s paint of %i bytes, options=%s", rgb_format, len(img_data), options, exc_info=True)
         except Exception as e:
-            message = b"OpenGL %s paint error: %s" % (rgb_format, e)
-        log("Error in %s paint of %i bytes, options=%s", rgb_format, len(img_data), options, exc_info=True)
+            message = "OpenGL %s paint error: %s" % (rgb_format, e)
+            log("Error in %s paint of %i bytes, options=%s", rgb_format, len(img_data), options, exc_info=True)
         fire_paint_callbacks(callbacks, False, message)
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
-        #copy so the data will be usable (usually a str)
-        img.clone_pixel_data()
-        self.idle_add(self.gl_paint_planar, options.get("flush", 0), options.get("encoding"), img, x, y, enc_width, enc_height, width, height, callbacks)
+        if not zerocopy_upload or FORCE_CLONE:
+            #copy so the data will be usable (usually a str)
+            img.clone_pixel_data()
+        pixel_format = img.get_pixel_format()
+        if pixel_format=="GBRP":
+            shader = RGBP2RGB_SHADER
+        else:
+            shader = YUV2RGB_SHADER
+        self.idle_add(self.gl_paint_planar, shader, options.intget("flush", 0), options.strget("encoding"), img, x, y, enc_width, enc_height, width, height, callbacks)
 
-    def gl_paint_planar(self, flush, encoding, img, x, y, enc_width, enc_height, width, height, callbacks):
+    def gl_paint_planar(self, shader, flush, encoding, img, x, y, enc_width, enc_height, width, height, callbacks):
         #this function runs in the UI thread, no video_decoder lock held
         log("gl_paint_planar%s", (flush, encoding, img, x, y, enc_width, enc_height, width, height, callbacks))
         try:
@@ -1048,18 +1025,19 @@ class GLWindowBackingBase(GTKWindowBacking):
                 if width!=enc_width or height!=enc_height:
                     x_scale = float(width)/enc_width
                     y_scale = float(height)/enc_height
-                self.render_planar_update(x, y, enc_width, enc_height, x_scale, y_scale)
+                self.render_planar_update(x, y, enc_width, enc_height, x_scale, y_scale, shader)
                 self.paint_box(encoding, False, x, y, width, height)
+                fire_paint_callbacks(callbacks, True)
                 # Present it on screen
                 self.present_fbo(x, y, width, height, flush)
             img.free()
-            fire_paint_callbacks(callbacks, True)
             return
         except GLError as e:
             message = "OpenGL %s paint failed: %r" % (encoding, e)
+            log.error("Error painting planar update", exc_info=True)
         except Exception as e:
             message = "OpenGL %s paint failed: %s" % (encoding, e)
-        log.error("Error: %s", e, exc_info=True)
+            log.error("Error painting planar update", exc_info=True)
         log.error(" flush=%i, image=%s, coords=%s, size=%ix%i", flush, img, (x, y, enc_width, enc_height), width, height)
         fire_paint_callbacks(callbacks, False, message)
 
@@ -1092,32 +1070,36 @@ class GLWindowBackingBase(GTKWindowBacking):
         rowstrides = img.get_rowstride()
         img_data = img.get_pixels()
         assert len(rowstrides)==3 and len(img_data)==3
-        for texture, index in ((GL_TEXTURE0, TEX_Y), (GL_TEXTURE1, TEX_U), (GL_TEXTURE2, TEX_V)):
+        for texture, index, tex_name in ((GL_TEXTURE0, TEX_Y, "Y"), (GL_TEXTURE1, TEX_U, "U"), (GL_TEXTURE2, TEX_V, "V")):
             (div_w, div_h) = divs[index]
+            w = width//div_w
+            h = height//div_h
+            if w==0 or h==0:
+                log.error("Error: zero dimension %ix%i for %s planar texture %s", w, h, pixel_format, tex_name)
+                log.error(" screen update %s dropped", (x, y, width, height))
+                continue
             glActiveTexture(texture)
 
             target = GL_TEXTURE_RECTANGLE_ARB
             glBindTexture(target, self.textures[index])
-            self.set_alignment(width//div_w, rowstrides[index], "YUV"[index])
+            self.set_alignment(w, rowstrides[index], tex_name)
             upload, pixel_data = self.pixels_for_upload(img_data[index])
-            log("texture %s: div=%s, rowstride=%s, %sx%s, data=%s bytes, upload=%s", index, divs[index], rowstrides[index], width//div_w, height//div_h, len(pixel_data), upload)
+            log("texture %s: div=%s, rowstride=%s, %sx%s, data=%s bytes, upload=%s", index, divs[index], rowstrides[index], w, h, len(pixel_data), upload)
             glTexParameteri(target, GL_TEXTURE_BASE_LEVEL, 0)
             try:
                 glTexParameteri(target, GL_TEXTURE_MAX_LEVEL, 0)
             except:
                 pass
-            glTexSubImage2D(target, 0, 0, 0, width//div_w, height//div_h, GL_LUMINANCE, GL_UNSIGNED_BYTE, pixel_data)
+            glTexSubImage2D(target, 0, 0, 0, w, h, GL_LUMINANCE, GL_UNSIGNED_BYTE, pixel_data)
             glBindTexture(target, 0)
         #glActiveTexture(GL_TEXTURE0)    #redundant, we always call render_planar_update afterwards
 
-    def render_planar_update(self, rx, ry, rw, rh, x_scale=1, y_scale=1):
-        log("%s.render_planar_update%s pixel_format=%s", self, (rx, ry, rw, rh, x_scale, y_scale), self.pixel_format)
+    def render_planar_update(self, rx, ry, rw, rh, x_scale=1, y_scale=1, shader=YUV2RGB_SHADER):
+        log("%s.render_planar_update%s pixel_format=%s", self, (rx, ry, rw, rh, x_scale, y_scale, shader), self.pixel_format)
         if self.pixel_format not in ("YUV420P", "YUV422P", "YUV444P", "GBRP"):
             #not ready to render yet
             return
-        if self.pixel_format == "GBRP":
-            # Set GL state for planar RGB: change fragment program
-            glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, self.shaders[RGBP2RGB_SHADER])
+        glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, self.shaders[shader])
         self.gl_marker("painting planar update, format %s", self.pixel_format)
         divs = get_subsampling_divs(self.pixel_format)
         glEnable(GL_FRAGMENT_PROGRAM_ARB)
@@ -1140,7 +1122,27 @@ class GLWindowBackingBase(GTKWindowBacking):
             glActiveTexture(texture)
             glBindTexture(GL_TEXTURE_RECTANGLE_ARB, 0)
         glDisable(GL_FRAGMENT_PROGRAM_ARB)
-        if self.pixel_format == "GBRP":
-            # Reset state to our default (YUV painting)
-            glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, self.shaders[YUV2RGB_SHADER])
         glActiveTexture(GL_TEXTURE0)
+
+
+    def gl_show(self, rect_count):
+        start = monotonic_time()
+        self.do_gl_show(rect_count)
+        end = monotonic_time()
+        flush_elapsed = end-self.last_flush
+        self.last_flush = end
+        fpslog("gl_show after %3ims took %2ims, %2i updates", flush_elapsed*1000, (end-start)*1000, rect_count)
+
+
+    def gl_expose_rect(self, rect=None):
+        if not self.paint_screen:
+            return
+        context = self.gl_context()
+        if not context:
+            return
+        if not rect:
+            w, h = self.size
+            rect = (0, 0, w, h)
+        with context:
+            self.gl_init()
+            self.present_fbo(*rect)
