@@ -1,10 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import gobject
+
 from xpra.util import envbool
 from xpra.client.gtk2.gtk2_window_base import GTK2WindowBase
 
@@ -24,7 +26,13 @@ either using CairoBacking or PixmapBacking.
 """
 class ClientWindow(GTK2WindowBase):
 
-    __common_gsignals__ = GTK2WindowBase.__common_gsignals__
+    __gsignals__ = GTK2WindowBase.__common_gsignals__
 
     def get_backing_class(self):
         return BACKING_CLASS
+
+
+    def toggle_debug(self, *args):
+        pass
+
+gobject.type_register(ClientWindow)
