@@ -178,12 +178,13 @@ class ClientWindow(GTKClientWindowBase):
         self.queue_draw_area(x, y, width, height)
 
     def do_draw(self, context):
+        #Gtk.Window.do_draw(self, context)
         paintlog("do_draw(%s)", context)
         backing = self._backing
         if self.get_mapped() and backing:
             self.paint_backing_offset_border(backing, context)
             self.clip_to_backing(backing, context)
-            self._backing.cairo_draw(context)
+            backing.cairo_draw(context)
         self.cairo_paint_border(context, None)
         if not self._client.server_ok():
             self.paint_spinner(context)
