@@ -6,9 +6,11 @@
 
 from xpra.gtk_common.gobject_compat import import_gtk
 gtk = import_gtk()
-from xpra.gtk_common.gtk_util import WIN_POS_CENTER
-from xpra.x11.gtk_x11.gdk_display_source import init_display_source
-init_display_source()
+from xpra.gtk_common.gtk_util import WIN_POS_CENTER, add_close_accel
+from xpra.os_util import POSIX
+if POSIX:
+    from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+    init_display_source()
 
 
 class BellWindow(gtk.Window):
@@ -22,8 +24,9 @@ class BellWindow(gtk.Window):
         btn.connect('clicked', self.bell)
         self.add(btn)
         self.show_all()
+        add_close_accel(self, gtk.main_quit)
 
-    def bell(self, *args):
+    def bell(self, *_args):
         from xpra.platform.gui import system_bell
         system_bell(self.get_window(), 0, 100, 2000, 1000, 0, 0, "test")
 
