@@ -781,7 +781,8 @@ class SessionInfo(gtk.Window):
 
     def populate_connection(self):
         def settimedeltastr(label, from_time):
-            delta = datetime.timedelta(seconds=(int(monotonic_time())-int(from_time)))
+            import time
+            delta = datetime.timedelta(seconds=(int(time.time())-int(from_time)))
             label.set_text(str(delta))
         if self.client.server_load:
             self.server_load_label.set_text("  ".join([str(x/1000.0) for x in self.client.server_load]))
@@ -872,17 +873,26 @@ class SessionInfo(gtk.Window):
     def all_values_from_info(self, *window_props):
         #newer (2.4 and later) servers can just give us the value directly:
         for window_prop in window_props:
-            v = dictlook(self.client.server_last_info, "client.%s" % window_prop)
+            prop_path = "client.%s" % window_prop
+            v = dictlook(self.client.server_last_info, prop_path)
             if v is not None:
-                v = typedict(v)
-                getv = v.intget
-                return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
+                try:
+                    v = typedict(v)
+                except TypeError:
+                    #backwards compatibility:
+                    #older servers don't expose the correct value or type here
+                    #so don't use this value
+                    log("expected dictionary for %s", prop_path)
+                    log(" got %s: %s", type(v), v)
+                else:
+                    getv = v.intget
+                    return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
 
         #legacy servers: sum up the values for all the windows found
         def avg(values):
             if not values:
                 return ""
-            return sum(values) / len(values)
+            return sum(values) // len(values)
         def getv(suffix, op):
             if self.client.server_last_info is None:
                 return ""
