@@ -21,8 +21,8 @@ gobject.threads_init()
 pango = import_pango()
 
 
-from xpra.gtk_common.gtk_util import gtk_main, add_close_accel, scaled_image, pixbuf_new_from_file, \
-                                    WIN_POS_CENTER, STATE_NORMAL
+from xpra.gtk_common.gtk_util import gtk_main, add_close_accel, scaled_image, pixbuf_new_from_file, window_defaults, \
+                                    WIN_POS_CENTER
 from xpra.platform.paths import get_icon_dir
 from xpra.log import Logger, enable_debug_for
 log = Logger("util")
@@ -41,11 +41,10 @@ class StartNewCommand(object):
     def __init__(self, run_callback=None, can_share=False):
         self.run_callback = run_callback
         self.window = gtk.Window()
+        window_defaults(self.window)
         self.window.connect("destroy", self.close)
         self.window.set_default_size(400, 150)
-        self.window.set_border_width(20)
         self.window.set_title("Start New Command")
-        self.window.modify_bg(STATE_NORMAL, gdk.Color(red=65535, green=65535, blue=65535))
 
         icon_pixbuf = self.get_icon("forward.png")
         if icon_pixbuf:
@@ -63,7 +62,8 @@ class StartNewCommand(object):
         vbox.add(al)
 
         # Actual command:
-        self.entry = gtk.Entry(max=100)
+        self.entry = gtk.Entry()
+        self.entry.set_max_length(255)
         self.entry.set_width_chars(32)
         self.entry.connect('activate', self.run_command)
         vbox.add(self.entry)
@@ -92,7 +92,7 @@ class StartNewCommand(object):
         btn("Run", "Run this command", self.run_command, "forward.png")
         btn("Cancel", "", self.close, "quit.png")
 
-        def accel_close(*args):
+        def accel_close(*_args):
             self.close()
         add_close_accel(self.window, accel_close)
         vbox.show_all()
@@ -138,7 +138,7 @@ class StartNewCommand(object):
         return None
 
 
-    def run_command(self, *args):
+    def run_command(self, *_args):
         self.hide()
         command = self.entry.get_text()
         if self.run_callback:
@@ -158,8 +158,8 @@ def main():
         gtk_main_quit_on_fatal_exceptions_enable()
 
         app = StartNewCommand()
-        app.close = app.quit
-        def app_signal(signum, frame):
+        app.hide = app.quit
+        def app_signal(signum, _frame):
             print("")
             log.info("got signal %s", SIGNAMES.get(signum, signum))
             app.quit()
