@@ -1,25 +1,28 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 from xpra.log import Logger
 log = Logger("keyboard")
 
-from xpra.keyboard.layouts import xkbmap_query_tostring
-from xpra.keyboard.mask import DEFAULT_MODIFIER_MEANINGS, DEFAULT_MODIFIER_NUISANCE
-from xpra.util import nonl, csv, std, print_nested_dict
+from xpra.util import nonl, csv, std, envbool, print_nested_dict, repr_ellipsized
+from xpra.os_util import POSIX, bytestostr
+
+LAYOUT_GROUPS = envbool("XPRA_LAYOUT_GROUPS", True)
 
 
 class KeyboardHelper(object):
 
-    def __init__(self, net_send, keyboard_sync, key_shortcuts, raw, layout, layouts, variant, variants, options):
+    def __init__(self, net_send, keyboard_sync=True, shortcut_modifiers="auto", key_shortcuts=[], raw=False, layout="", layouts=[], variant="", variants=[], options=""):
         self.reset_state()
         self.send = net_send
         self.locked = False
         self.keyboard_sync = keyboard_sync
+        self.shortcut_modifiers = shortcut_modifiers
         self.key_shortcuts = self.parse_shortcuts(key_shortcuts)
         #command line overrides:
         self.xkbmap_raw = raw
@@ -32,6 +35,9 @@ class KeyboardHelper(object):
         from xpra.platform.keyboard import Keyboard
         self.keyboard = Keyboard()
         log("KeyboardHelper(%s) keyboard=%s", (net_send, keyboard_sync, key_shortcuts, raw, layout, layouts, variant, variants, options), self.keyboard)
+        key_repeat = self.keyboard.get_keyboard_repeat()
+        if key_repeat:
+            self.key_repeat_delay, self.key_repeat_interval = key_repeat
 
     def mask_to_names(self, mask):
         return self.keyboard.mask_to_names(mask)
@@ -49,20 +55,21 @@ class KeyboardHelper(object):
         self.xkbmap_layouts = []
         self.xkbmap_variant = ""
         self.xkbmap_variants = []
+        self.xkbmap_options = ""
         self.xkbmap_print = ""
         self.xkbmap_query = ""
         self.xkbmap_query_struct = {}
+        self.xkbmap_layout_groups = LAYOUT_GROUPS
+        self.xkbmap_raw = False
 
         self.hash = None
 
         self.key_repeat_delay = -1
         self.key_repeat_interval = -1
-        self.keys_pressed = {}
         self.keyboard_sync = False
         self.key_shortcuts = {}
 
     def cleanup(self):
-        self.clear_repeat()
         self.reset_state()
         def nosend(*args):
             pass
@@ -88,6 +95,7 @@ class KeyboardHelper(object):
         #whereas at best, we keep "Alt_L" : "mod1" mappings... (xposix)
         #so generate a map from one to the other:
         modifier_names = {}
+        from xpra.keyboard.mask import DEFAULT_MODIFIER_MEANINGS, DEFAULT_MODIFIER_NUISANCE
         meanings = self.xkbmap_mod_meanings or DEFAULT_MODIFIER_MEANINGS
         DEFAULT_MODIFIER_IGNORE_KEYNAMES = ["Caps_Lock", "Num_Lock", "Scroll_Lock"]
         for pub_name,mod_name in meanings.items():
@@ -103,6 +111,37 @@ class KeyboardHelper(object):
             if pub_name.lower()=="control":
                 #alias "control" to "ctrl" as it is often used:
                 modifier_names["ctrl"] = mod_name
+        log("parse_shortcuts: modifier names=%s", modifier_names)
+
+        #figure out the default shortcut modifiers
+        #accept "," or "+" as delimiter:
+        shortcut_modifiers = self.shortcut_modifiers.lower().replace(",", "+").split("+")
+        def mod_defaults():
+            mods = ["meta", "shift"]
+            if POSIX:
+                #gnome intercepts too many of the Alt+Shift shortcuts,
+                #so use control with gnome:
+                if os.environ.get("XDG_CURRENT_DESKTOP")=="GNOME":
+                    mods = ["control", "shift"]
+            return mods
+        if shortcut_modifiers==["auto"]:
+            shortcut_modifiers = mod_defaults()
+        elif shortcut_modifiers==["none"]:
+            shortcut_modifiers = []
+        else:
+            r = []
+            for x in shortcut_modifiers:
+                x = x.lower()
+                if x not in modifier_names:
+                    log.warn("Warning: invalid shortcut modifier '%s'", x)
+                else:
+                    r.append(x)
+            if shortcut_modifiers and not r:
+                log.warn(" using defaults instead")
+                shortcut_modifiers = mod_defaults()
+            else:
+                shortcut_modifiers = r
+        log("shortcut modifiers=%s", shortcut_modifiers)
 
         for s in strs:
             #example for s: Control+F8:some_action()
@@ -131,7 +170,8 @@ class KeyboardHelper(object):
                             args.append(int(x))
                     args = tuple(args)
                 except Exception as e:
-                    log.warn("failed to parse arguments of shortcut '%s': %s", s, e)
+                    log.warn("Warning: failed to parse arguments of shortcut:")
+                    log.warn(" '%s': %s", s, e)
                     continue
             action = action.replace("-", "_")       #must be an object attribute
             log("action(%s)=%s%s", s, action, args)
@@ -143,11 +183,22 @@ class KeyboardHelper(object):
                 valid = True
                 #ie: ["Alt"]
                 for mod in keyspec[:len(keyspec)-1]:
+                    mod = mod.lower()
+                    if mod=="none":
+                        continue
+                    elif mod=="#":
+                        #this is the placeholder for the list of shortcut modifiers:
+                        for x in shortcut_modifiers:
+                            imod = modifier_names.get(x)
+                            if imod:
+                                modifiers.append(imod)
+                        continue
+                    #find the real modifier for this name:
                     #ie: "alt_l" -> "mod1"
-                    imod = modifier_names.get(mod.lower())
+                    imod = modifier_names.get(mod)
                     if not imod:
-                        log.error("Error: invalid modifier '%s' in keyboard shortcut '%s'", mod, s)
-                        log.error(" the modifiers must be one of: %s", csv(modifier_names.keys()))
+                        log.warn("Warning: invalid modifier '%s' in keyboard shortcut '%s'", mod, s)
+                        log.warn(" the modifiers must be one of: %s", csv(modifier_names.keys()))
                         valid = False
                         break
                     modifiers.append(imod)
@@ -156,14 +207,15 @@ class KeyboardHelper(object):
             #TODO: validate keyname
             keyname = keyspec[len(keyspec)-1]
             shortcuts.setdefault(keyname, []).append((modifiers, action, args))
-            log("shortcut(%s)=%s", keyname, (modifiers, action, args))
+            log("shortcut(%s)=%s", s, csv((modifiers, action, args)))
         log("parse_shortcuts(%s)=%s" % (str(strs), shortcuts))
         print_nested_dict(shortcuts, print_fn=log)
-        return  shortcuts
+        return shortcuts
 
     def key_handled_as_shortcut(self, window, key_name, modifiers, depressed):
         #find the shortcuts that may match this key:
         shortcuts = self.key_shortcuts.get(key_name)
+        log("key_handled_as_shortcut: shortcut(%s)=%s", key_name, shortcuts)
         if not shortcuts:
             return False
         if len(shortcuts)>1:
@@ -239,70 +291,12 @@ class KeyboardHelper(object):
         for x in ("keyname", "pressed", "modifiers", "keyval", "string", "keycode", "group"):
             packet.append(getattr(key_event, x))
         self.send(*packet)
-        if self.keyboard_sync and self.key_repeat_delay>0 and self.key_repeat_interval>0:
-            self._key_repeat(key_event)
-
-    def _key_repeat(self, key_event):
-        """ this method takes care of scheduling the sending of
-            "key-repeat" packets to the server so that it can
-            maintain a consistent keyboard state.
-        """
-        wid, keyname, pressed, _, keyval, _, keycode, _ = key_event
-        #we keep track of which keys are still pressed in a dict,
-        if keycode<0:
-            key = keyname
-        else:
-            key = keycode
-        if not pressed and key in self.keys_pressed:
-            """ stop the timer and clear this keycode: """
-            timer = self.keys_pressed[key]
-            log("key repeat: clearing timer %s for %s / %s", timer, keyname, keycode)
-            self.source_remove(timer)
-            del self.keys_pressed[key]
-        elif pressed and key not in self.keys_pressed:
-            """ we must send packets to the server regularly for as long as the key is still pressed: """
-            #TODO: we can have latency measurements (see ping).. use them?
-            LATENCY_JITTER = 100
-            MIN_DELAY = 5
-            delay = max(self.key_repeat_delay-LATENCY_JITTER, MIN_DELAY)
-            interval = max(self.key_repeat_interval-LATENCY_JITTER, MIN_DELAY)
-            log("scheduling key repeat for %s: delay=%s, interval=%s (from %s and %s)", keyname, delay, interval, self.key_repeat_delay, self.key_repeat_interval)
-            def send_key_repeat():
-                modifiers = self.get_current_modifiers()
-                self.send_now("key-repeat", wid, keyname, keyval, keycode, modifiers)
-            def continue_key_repeat(*args):
-                #if the key is still pressed (redundant check?)
-                #confirm it and continue, otherwise stop
-                log("continue_key_repeat for %s / %s", keyname, keycode)
-                if key in self.keys_pressed:
-                    send_key_repeat()
-                    return  True
-                else:
-                    del self.keys_pressed[key]
-                    return  False
-            def start_key_repeat(*args):
-                #if the key is still pressed (redundant check?)
-                #confirm it and start repeat:
-                log("start_key_repeat for %s / %s", keyname, keycode)
-                if key in self.keys_pressed:
-                    send_key_repeat()
-                    self.keys_pressed[key] = self.timeout_add(interval, continue_key_repeat)
-                else:
-                    del self.keys_pressed[key]
-                return  False   #never run this timer again
-            log("key repeat: starting timer for %s / %s with delay %s and interval %s", keyname, keycode, delay, interval)
-            self.keys_pressed[key] = self.timeout_add(delay, start_key_repeat)
-
-    def clear_repeat(self):
-        for timer in self.keys_pressed.values():
-            self.source_remove(timer)
-        self.keys_pressed = {}
 
 
     def get_layout_spec(self):
         """ add / honour overrides """
-        layout, layouts, variant, variants = self.keyboard.get_layout_spec()
-        log("%s.get_layout_spec()=%s", self.keyboard, (layout, layouts, variant, variants))
+        layout, layouts, variant, variants, options = self.keyboard.get_layout_spec()
+        log("%s.get_layout_spec()=%s", self.keyboard, (layout, layouts, variant, variants, options))
         def inl(v, l):
             try:
                 if v in l or v is None:
@@ -316,40 +310,43 @@ class KeyboardHelper(object):
         layouts  = inl(layout, self.layouts_option or layouts)
         variant  = self.variant_option or variant
         variants = inl(variant, self.variants_option or variants)
-        val = (layout, layouts, self.variant_option or variant, self.variants_option or variants)
+        options  = self.options or options
+        val = (layout, layouts, self.variant_option or variant, self.variants_option or variants, self.options)
         log("get_layout_spec()=%s", val)
         return val
 
     def get_keymap_spec(self):
         _print, query, query_struct = self.keyboard.get_keymap_spec()
-        if self.layout_option:
-            query_struct["layout"] = self.layout_option
-        if self.layouts_option:
-            query_struct["layouts"] = csv(self.layouts_option)
-        if self.variant_option:
-            query_struct["variant"] = self.variant_option
-        if self.variants_option:
-            query_struct["variants"] = csv(self.variants_option)
-        if self.options:
-            if self.options.lower()=="none":
-                query_struct["options"] = ""
-            else:
-                query_struct["options"] = self.options
-        if self.layout_option or self.layouts_option or self.variant_option or self.variants_option or self.options:
-            query = xkbmap_query_tostring(query_struct)
+        if query_struct:
+            if self.layout_option:
+                query_struct["layout"] = self.layout_option
+            if self.layouts_option:
+                query_struct["layouts"] = csv(self.layouts_option)
+            if self.variant_option:
+                query_struct["variant"] = self.variant_option
+            if self.variants_option:
+                query_struct["variants"] = csv(self.variants_option)
+            if self.options:
+                if self.options.lower()=="none":
+                    query_struct["options"] = ""
+                else:
+                    query_struct["options"] = self.options
+            if self.layout_option or self.layouts_option or self.variant_option or self.variants_option or self.options:
+                from xpra.keyboard.layouts import xkbmap_query_tostring
+                query = xkbmap_query_tostring(query_struct)
         return _print, query, query_struct
 
     def query_xkbmap(self):
-        self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants = self.get_layout_spec()
+        self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants, self.xkbmap_options = self.get_layout_spec()
         self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct = self.get_keymap_spec()
         self.xkbmap_keycodes = self.get_full_keymap()
         self.xkbmap_x11_keycodes = self.keyboard.get_x11_keymap()
         self.xkbmap_mod_meanings, self.xkbmap_mod_managed, self.xkbmap_mod_pointermissing = self.keyboard.get_keymap_modifiers()
         self.update_hash()
         log("layout=%s, layouts=%s, variant=%s, variants=%s", self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants)
-        log("print=%s, query=%s, struct=%s", nonl(self.xkbmap_print), nonl(self.xkbmap_query), nonl(self.xkbmap_query_struct))
-        log("keycodes=%s", str(self.xkbmap_keycodes)[:80]+"...")
-        log("x11 keycodes=%s", str(self.xkbmap_x11_keycodes)[:80]+"...")
+        log("print=%s, query=%s, struct=%s", nonl(self.xkbmap_print), nonl(self.xkbmap_query), self.xkbmap_query_struct)
+        log("keycodes=%s", repr_ellipsized(str(self.xkbmap_keycodes)))
+        log("x11 keycodes=%s", repr_ellipsized(str(self.xkbmap_x11_keycodes)))
         log("mod managed: %s", self.xkbmap_mod_managed)
         log("mod meanings: %s", self.xkbmap_mod_meanings)
         log("mod pointermissing: %s", self.xkbmap_mod_pointermissing)
@@ -360,12 +357,12 @@ class KeyboardHelper(object):
             self.query_xkbmap()
 
     def layout_str(self):
-        return " / ".join([x for x in (self.layout_option or self.xkbmap_layout, self.variant_option or self.xkbmap_variant) if bool(x)])
+        return " / ".join([bytestostr(x) for x in (self.layout_option or self.xkbmap_layout, self.variant_option or self.xkbmap_variant) if bool(x)])
 
 
     def send_layout(self):
-        log("send_layout() layout_option=%s, xkbmap_layout=%s, variant_option=%s, xkbmap_variant=%s", self.layout_option, self.xkbmap_layout, self.variant_option, self.xkbmap_variant)
-        self.send("layout-changed", self.layout_option or self.xkbmap_layout or "", self.variant_option or self.xkbmap_variant or "")
+        log("send_layout() layout_option=%s, xkbmap_layout=%s, variant_option=%s, xkbmap_variant=%s, xkbmap_options=%s", self.layout_option, self.xkbmap_layout, self.variant_option, self.xkbmap_variant, self.xkbmap_options)
+        self.send("layout-changed", self.layout_option or self.xkbmap_layout or "", self.variant_option or self.xkbmap_variant or "", self.xkbmap_options or "")
 
     def send_keymap(self):
         log("send_keymap()")
@@ -394,7 +391,7 @@ class KeyboardHelper(object):
     def get_keymap_properties(self):
         props = {}
         for x in ("layout", "layouts", "variant", "variants",
-                  "raw",
+                  "raw", "layout_groups",
                   "print", "query", "query_struct", "mod_meanings",
                   "mod_managed", "mod_pointermissing", "keycodes", "x11_keycodes"):
             p = "xkbmap_%s" % x
@@ -424,4 +421,4 @@ class KeyboardHelper(object):
         if len(kb_info)==0:
             log.info(" using default keyboard settings")
         else:
-            log.info(" keyboard settings: %s", ", ".join(["%s=%s" % (std(k), std(v)) for k,v in kb_info.items()]))
+            log.info(" keyboard settings: %s", csv("%s=%s" % (std(k), std(v)) for k,v in kb_info.items()))
