@@ -1,11 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.platform.paths import get_icon_filename
 from xpra.log import Logger
+from xpra.os_util import monotonic_time
 from collections import deque
 log = Logger("tray")
 
@@ -15,7 +16,7 @@ class TrayBase(object):
         Utility superclass for all tray implementations
     """
 
-    def __init__(self, client, app_id, menu, tooltip, icon_filename, size_changed_cb, click_cb, mouseover_cb, exit_cb):
+    def __init__(self, _client, app_id, menu, tooltip, icon_filename, size_changed_cb, click_cb, mouseover_cb, exit_cb):
         #we don't keep a reference to client,
         #because calling functions on the client directly should be discouraged
         self.app_id = app_id
@@ -31,6 +32,10 @@ class TrayBase(object):
         self.geometry_guess = None
         self.tray_event_locations = deque(maxlen=512)
         self.default_icon_extension = "png"
+        self.icon_timestamp = 0
+
+    def __repr__(self):
+        return "Tray(%i:%s)" % (self.app_id, self.tooltip)
 
     def cleanup(self):
         if self.tray_widget:
@@ -78,7 +83,7 @@ class TrayBase(object):
     def set_icon(self, basename=None):
         filename = self.get_icon_filename(basename)
         if not filename:
-            log.error("Error: cannot find icon '%s' for name '%s'", filename, basename)
+            log.error("Error: cannot find icon '%s'", basename)
             return
         log("set_icon(%s) using filename=%s", basename, filename)
         self.set_icon_from_file(filename)
@@ -88,17 +93,21 @@ class TrayBase(object):
         if not self.tray_widget:
             return
         self.do_set_icon_from_file(filename)
+        self.icon_timestamp = monotonic_time()
 
     def do_set_icon_from_file(self, filename):
         raise Exception("override me!")
 
     def recalculate_geometry(self, x, y, width, height):
-        log("recalculate_geometry%s tray event locations: %s", (x, y, width, height), len(self.tray_event_locations))
+        log("recalculate_geometry%s guess=%s, tray event locations: %s", (x, y, width, height), self.geometry_guess, len(self.tray_event_locations))
+        if x is None or y is None:
+            return
         if self.geometry_guess is None:
             #better than nothing!
             self.geometry_guess = x, y, width, height
         if len(self.tray_event_locations)>0 and self.tray_event_locations[-1]==(x,y):
             #unchanged
+            log("tray event location unchanged")
             return
         self.tray_event_locations.append((x, y))
         #sets of locations that can fit together within (size,size) distance of each other:
@@ -129,6 +138,6 @@ class TrayBase(object):
         miny -= pady//2
         oldgeom = self.geometry_guess
         self.geometry_guess = max(0, minx), max(0, miny), width, height
-        log("recalculate_geometry() geometry guess=%s", self.geometry_guess)
+        log("recalculate_geometry() geometry guess=%s (old guess=%s)", self.geometry_guess, oldgeom)
         if self.size_changed_cb and self.geometry_guess!=oldgeom:
             self.size_changed_cb()
