@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -16,12 +16,15 @@ from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
 from xpra.os_util import BytesIOClass, bytestostr, _buffer
-from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
-from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
+try:
+    from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
+except ImportError:
+    from xpra.util import xor as xor_str
 
 DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
 INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
 PAINT_BOX = envint("XPRA_PAINT_BOX", 0) or envint("XPRA_OPENGL_PAINT_BOX", 0)
+WEBP_PILLOW = envbool("XPRA_WEBP_PILLOW", False)
 
 
 #ie:
@@ -67,16 +70,16 @@ def fire_paint_callbacks(callbacks, success=True, message=""):
 
 """
 Generic superclass for all Backing code,
-see CairoBacking and GTKWindowBacking for actual implementations
+see CairoBackingBase and GTK2WindowBacking subclasses for actual implementations
 """
 class WindowBackingBase(object):
-    def __init__(self, wid, window_alpha, idle_add):
+    def __init__(self, wid, window_alpha):
         load_csc_options()
         load_video_decoders()
         self.wid = wid
         self.size = 0, 0
+        self.render_size = 0, 0
         self.offsets = 0, 0, 0, 0       #top,left,bottom,right
-        self.idle_add = idle_add
         self._alpha_enabled = window_alpha
         self._backing = None
         self._delta_pixel_data = [None for _ in range(DELTA_BUCKETS)]
@@ -96,6 +99,10 @@ class WindowBackingBase(object):
         self.mmap = None
         self.mmap_enabled = False
         self.jpeg_decoder = get_codec("dec_jpeg")
+        self.webp_decoder = get_codec("dec_webp")
+
+    def idle_add(self, *_args, **_kwargs):
+        raise NotImplementedError()
 
     def enable_mmap(self, mmap_area):
         self.mmap = mmap_area
@@ -145,15 +152,17 @@ class WindowBackingBase(object):
         #calculate the server CSC modes the server is allowed to use
         #based on the client CSC modes we can convert to in the backing class we use
         #and trim the transparency if we cannot handle it
-        target_rgb_modes = list(rgb_modes)
+        target_rgb_modes = tuple(rgb_modes)
         if not self._alpha_enabled:
-            target_rgb_modes = [x for x in target_rgb_modes if x.find("A")<0]
+            target_rgb_modes = tuple(x for x in target_rgb_modes if x.find("A")<0)
         full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*target_rgb_modes)
+        full_csc_modes["webp"] = [x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA")]
         log("_get_full_csc_modes(%s)=%s (target_rgb_modes=%s)", rgb_modes, full_csc_modes, target_rgb_modes)
         return full_csc_modes
 
 
     def unpremultiply(self, img_data):
+        from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
         if type(img_data) not in (str, _buffer):
             try:
                 unpremultiply_argb_in_place(img_data)
@@ -222,8 +231,8 @@ class WindowBackingBase(object):
         assert PIL.Image, "PIL.Image not found"
         buf = BytesIOClass(img_data)
         img = PIL.Image.open(buf)
-        assert img.mode in ("L", "P", "RGB", "RGBA"), "invalid image mode: %s" % img.mode
-        transparency = options.get("transparency", -1)
+        assert img.mode in ("L", "P", "RGB", "RGBA", "RGBX"), "invalid image mode: %s" % img.mode
+        transparency = options.intget("transparency", -1)
         if img.mode=="P":
             if transparency>=0:
                 #this deals with alpha without any extra work
@@ -251,19 +260,66 @@ class WindowBackingBase(object):
 
         raw_data = img.tobytes("raw", img.mode)
         paint_options = typedict(options)
-        rgb_format = img.mode
-        if rgb_format=="RGB":
+        if img.mode=="RGB":
             #PIL flattens the data to a continuous straightforward RGB format:
             rowstride = width*3
+            rgb_format = options.strget("rgb_format", "")
+            rgb_format = rgb_format.replace("A", "").replace("X", "")
+            #the webp encoder only takes BGRX input,
+            #so we have to swap things around if it was fed "RGB":
+            if rgb_format=="RGB":
+                rgb_format = "BGR"
+            else:
+                rgb_format = "RGB"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-        elif rgb_format=="RGBA":
+        elif img.mode in ("RGBA", "RGBX"):
             rowstride = width*4
+            rgb_format = options.strget("rgb_format", img.mode)
+            if coding=="webp":
+                #the webp encoder only takes BGRX input,
+                #so we have to swap things around if it was fed "RGBA":
+                if rgb_format=="RGBA":
+                    rgb_format = "BGRA"
+                elif rgb_format=="RGBX":
+                    rgb_format = "BGRX"
+                elif rgb_format=="BGRA":
+                    rgb_format = "RGBA"
+                elif rgb_format=="BGRX":
+                    rgb_format = "RGBX"
+                else:
+                    log.warn("Warning: unexpected RGB format '%s'", rgb_format)
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
         else:
             raise Exception("invalid image mode: %s" % img.mode)
         self.idle_add(self.do_paint_rgb, rgb_format, img_data, x, y, width, height, rowstride, paint_options, callbacks)
         return False
 
+    def paint_webp(self, img_data, x, y, width, height, options, callbacks):
+        if not self.webp_decoder or WEBP_PILLOW:
+            #if webp is enabled, then Pillow should be able to take care of it:
+            return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
+        rgb_format = options.strget("rgb_format")
+        has_alpha = options.boolget("has_alpha", False)
+        buffer_wrapper, width, height, stride, has_alpha, rgb_format = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
+        def free_buffer(*_args):
+            buffer_wrapper.free()
+        callbacks.append(free_buffer)
+        data = buffer_wrapper.get_pixels()
+        #if the backing can't handle this format,
+        #ie: tray only supports RGBA
+        if rgb_format not in self.RGB_MODES:
+            from xpra.codecs.rgb_transform import rgb_reformat
+            from xpra.codecs.image_wrapper import ImageWrapper
+            img = ImageWrapper(x, y, width, height, data, rgb_format, len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
+            #log("rgb_reformat(%s, %s, %s) %i bytes", img, self.RGB_MODES, has_alpha and self._alpha_enabled, len(img_data))
+            rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
+            rgb_format = img.get_pixel_format()
+            data = img.get_pixels()
+            stride = img.get_rowstride()
+        #replace with the actual rgb format we get from the decoder:
+        options[b"rgb_format"] = rgb_format
+        return self.paint_rgb(rgb_format, data, x, y, width, height, stride, options, callbacks)
+
     def paint_rgb(self, rgb_format, raw_data, x, y, width, height, rowstride, options, callbacks):
         """ can be called from a non-UI thread
             this method calls process_delta
@@ -278,14 +334,14 @@ class WindowBackingBase(object):
             the actual paint code is in _do_paint_rgb[24|32]
         """
         try:
-            if not options.get("paint", True):
+            if not options.boolget("paint", True):
                 fire_paint_callbacks(callbacks)
                 return
             if self._backing is None:
                 fire_paint_callbacks(callbacks, -1, "no backing")
                 return
             bpp = len(rgb_format)*8
-            assert bpp in (24, 32), "invalid rgb format %s" % rgb_format
+            assert bpp in (24, 32), "invalid rgb format '%s'" % rgb_format
             paint_fn = getattr(self, "_do_paint_rgb%i" % bpp)
             options[b"rgb_format"] = rgb_format
             success = paint_fn(img_data, x, y, width, height, rowstride, options)
@@ -307,6 +363,13 @@ class WindowBackingBase(object):
         raise Exception("override me!")
 
 
+    def eos(self):
+        dl = self._decoder_lock
+        with dl:
+            self.do_clean_csc_decoder()
+            self.do_clean_video_decoder()
+
+
     def make_csc(self, src_width, src_height, src_format,
                        dst_width, dst_height, dst_format_options, speed):
         global CSC_OPTIONS
@@ -345,7 +408,7 @@ class WindowBackingBase(object):
                 v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
                 if v:
                     log.error("       "+v[0], *v[1:])
-        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s)" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
 
     def validate_csc_size(self, spec, src_width, src_height, dst_width, dst_height):
         if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
@@ -394,7 +457,7 @@ class WindowBackingBase(object):
 
             vd = self._video_decoder
             if vd:
-                if options.get("frame", -1)==0:
+                if options.intget("frame", -1)==0:
                     log("paint_with_video_decoder: first frame of new stream")
                     self.do_clean_video_decoder()
                 elif vd.get_encoding()!=coding:
@@ -416,7 +479,7 @@ class WindowBackingBase(object):
 
             img = vd.decompress_image(img_data, options)
             if not img:
-                if options.get("delayed", 0)>0:
+                if options.intget("delayed", 0)>0:
                     #there are further frames queued up,
                     #and this frame references those, so assume all is well:
                     fire_paint_callbacks(callbacks)
@@ -501,10 +564,10 @@ class WindowBackingBase(object):
             coding = bytestostr(coding)
             options["encoding"] = coding            #used for choosing the color of the paint box
             if INTEGRITY_HASH:
-                l = options.get("z.len")
+                l = options.intget("z.len")
                 if l:
                     assert l==len(img_data), "compressed pixel data failed length integrity check: expected %i bytes but got %i" % (l, len(img_data))
-                md5 = options.get("z.md5")
+                md5 = options.strget("z.md5")
                 if md5:
                     h = hashlib.md5(img_data)
                     hd = h.hexdigest()
@@ -514,7 +577,7 @@ class WindowBackingBase(object):
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding == "rgb24" or coding == "rgb32":
                 #avoid confusion over how many bytes-per-pixel we may have:
-                rgb_format = options.get(b"rgb_format")
+                rgb_format = options.strget(b"rgb_format")
                 if rgb_format:
                     Bpp = len(rgb_format)
                 elif coding=="rgb24":
@@ -530,10 +593,12 @@ class WindowBackingBase(object):
                 self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
             elif self.jpeg_decoder and coding=="jpeg":
                 self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
+            elif coding == "webp":
+                self.paint_webp(img_data, x, y, width, height, options, callbacks)
             elif coding in self._PIL_encodings:
                 self.paint_image(coding, img_data, x, y, width, height, options, callbacks)
             elif coding == "scroll":
-                self.paint_scroll(x, y, width, height, img_data, options, callbacks)
+                self.paint_scroll(img_data, options, callbacks)
             else:
                 self.do_draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
         except Exception:
@@ -542,7 +607,7 @@ class WindowBackingBase(object):
             else:
                 raise
 
-    def do_draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
+    def do_draw_region(self, _x, _y, _width, _height, coding, _img_data, _rowstride, _options, callbacks):
         msg = "invalid encoding: '%s'" % coding
         log.error("Error: %s", msg)
         fire_paint_callbacks(callbacks, False, msg)
