@@ -7,31 +7,44 @@
 import sys
 import os
 from xpra.log import Logger
-log = Logger("encoder", "nvenc")
-from xpra.util import pver, print_nested_dict, engs, envbool
-from xpra.os_util import bytestostr
+log = Logger("encoder", "util")
+from xpra.util import pver, print_nested_dict, engs, envbool, csv
+from xpra.os_util import bytestostr, strtobytes
 
+MIN_VERSION = 375
 
-MAX_TESTED = 384
+nvml_init_warned = False
+def wrap_nvml_init(nvmlInit):
+    try:
+        nvmlInit()
+        return True
+    except Exception as e:
+        log("get_nvml_driver_version() pynvml error", exc_info=True)
+        global nvml_init_warned
+        if not nvml_init_warned:
+            log.warn("Warning: failed to initialize NVML:")
+            log.warn(" %s", e)
+            nvml_init_warned = True
+        return False
 
 def get_nvml_driver_version():
     try:
         from pynvml import nvmlInit, nvmlShutdown, nvmlSystemGetDriverVersion
         try:
-            nvmlInit()
-            v = nvmlSystemGetDriverVersion()
-            log("nvmlSystemGetDriverVersion=%s", bytestostr(v))
-            return v.split(b".")
+            if wrap_nvml_init(nvmlInit):
+                v = nvmlSystemGetDriverVersion()
+                log("nvmlSystemGetDriverVersion=%s", bytestostr(v))
+                return v.split(b".")
         except Exception as e:
             log("get_nvml_driver_version() pynvml error", exc_info=True)
-            log.warn("Warning: failed to query the NVidia kernel module version via NVML:")
+            log.warn("Warning: failed to query the NVidia kernel module version using NVML:")
             log.warn(" %s", e)
         finally:
             nvmlShutdown()
     except ImportError as e:
         log("cannot use nvml to query the kernel module version:")
         log(" %s", e)
-    return ""
+    return ()
 
 
 def get_proc_driver_version():
@@ -41,7 +54,7 @@ def get_proc_driver_version():
     if not v:
         log.warn("Warning: NVidia kernel module not installed?")
         log.warn(" cannot open '%s'", proc_file)
-        return ""
+        return ()
     KSTR = b"Kernel Module"
     p = v.find(KSTR)
     if not p:
@@ -65,10 +78,10 @@ def identify_nvidia_module_version():
                     raise
         if numver:
             log.info("NVidia driver version %s", pver(numver))
-            return numver
+            return tuple(numver)
     except Exception as e:
         log.warn("failed to parse Nvidia driver version '%s': %s", v, e)
-    return []
+    return ()
 
 nvidia_module_version = None
 def get_nvidia_module_version(probe=True):
@@ -85,7 +98,8 @@ def identify_cards():
         from pynvml import nvmlInit, nvmlShutdown, nvmlDeviceGetCount, nvmlDeviceGetHandleByIndex
         deviceCount = None
         try:
-            nvmlInit()
+            if not wrap_nvml_init(nvmlInit):
+                return devices
             deviceCount = nvmlDeviceGetCount()
             log("identify_cards() will probe %i cards", deviceCount)
             for i in range(deviceCount):
@@ -106,37 +120,41 @@ def identify_cards():
                         except:
                             pass
                     try:
-                        i["busId"] = str(pci.busId)
+                        i["busId"] = bytestostr(pci.busId)
                     except:
                         pass
                     return i
-                for prop, fn_name, args, conv in (
-                       ("name",                     "nvmlDeviceGetName",                    (),     str),
-                       ("serial",                   "nvmlDeviceGetSerial",                  (),     str),
-                       ("uuid",                     "nvmlDeviceGetUUID",                    (),     str),
-                       ("pci",                      "nvmlDeviceGetPciInfo",                 (),     pciinfo),
-                       ("memory",                   "nvmlDeviceGetMemoryInfo",              (),     meminfo),
-                       ("pcie-link-generation-max", "nvmlDeviceGetMaxPcieLinkGeneration",   (),     int),
-                       ("pcie-link-width-max",      "nvmlDeviceGetMaxPcieLinkWidth",        (),     int),
-                       ("pcie-link-generation",     "nvmlDeviceGetCurrPcieLinkGeneration",  (),     int),
-                       ("pcie-link-width",          "nvmlDeviceGetCurrPcieLinkWidth",       (),     int),
-                       ("clock-info-graphics",      "nvmlDeviceGetClockInfo",               (0,),   int),
-                       ("clock-info-sm",            "nvmlDeviceGetClockInfo",               (1,),   int),
-                       ("clock-info-mem",           "nvmlDeviceGetClockInfo",               (2,),   int),
-                       ("clock-info-graphics-max",  "nvmlDeviceGetMaxClockInfo",            (0,),   int),
-                       ("clock-info-sm-max",        "nvmlDeviceGetMaxClockInfo",            (1,),   int),
-                       ("clock-info-mem-max",       "nvmlDeviceGetMaxClockInfo",            (2,),   int),
-                       ("fan-speed",                "nvmlDeviceGetFanSpeed",                (),     int),
-                       ("temperature",              "nvmlDeviceGetTemperature",             (0,),   int),
-                       ("power-state",              "nvmlDeviceGetPowerState",              (),     int),
-                       ("vbios-version",            "nvmlDeviceGetVbiosVersion",            (),     str),
+                for prefix, prop, fn_name, args, conv in (
+                       ("", "name",                     "nvmlDeviceGetName",                    (),     strtobytes),
+                       ("", "serial",                   "nvmlDeviceGetSerial",                  (),     strtobytes),
+                       ("", "uuid",                     "nvmlDeviceGetUUID",                    (),     strtobytes),
+                       ("", "pci",                      "nvmlDeviceGetPciInfo",                 (),     pciinfo),
+                       ("", "memory",                   "nvmlDeviceGetMemoryInfo",              (),     meminfo),
+                       ("pcie-link", "generation-max",  "nvmlDeviceGetMaxPcieLinkGeneration",   (),     int),
+                       ("pcie-link", "width-max",       "nvmlDeviceGetMaxPcieLinkWidth",        (),     int),
+                       ("pcie-link", "generation",      "nvmlDeviceGetCurrPcieLinkGeneration",  (),     int),
+                       ("pcie-link", "width",           "nvmlDeviceGetCurrPcieLinkWidth",       (),     int),
+                       ("clock-info", "graphics",       "nvmlDeviceGetClockInfo",               (0,),   int),
+                       ("clock-info", "sm",             "nvmlDeviceGetClockInfo",               (1,),   int),
+                       ("clock-info", "mem",            "nvmlDeviceGetClockInfo",               (2,),   int),
+                       ("clock-info", "graphics-max",   "nvmlDeviceGetMaxClockInfo",            (0,),   int),
+                       ("clock-info", "sm-max",         "nvmlDeviceGetMaxClockInfo",            (1,),   int),
+                       ("clock-info", "mem-max",        "nvmlDeviceGetMaxClockInfo",            (2,),   int),
+                       ("", "fan-speed",                "nvmlDeviceGetFanSpeed",                (),     int),
+                       ("", "temperature",              "nvmlDeviceGetTemperature",             (0,),   int),
+                       ("", "power-state",              "nvmlDeviceGetPowerState",              (),     int),
+                       ("", "vbios-version",            "nvmlDeviceGetVbiosVersion",            (),     strtobytes),
                        ):
                     try:
                         fn = getattr(pynvml, fn_name)
                         v = fn(handle, *args)
                         if conv:
                             v = conv(v)
-                        props[prop] = v
+                        if prefix:
+                            d = props.setdefault(prefix, {})
+                        else:
+                            d = props
+                        d[prop] = v
                     except Exception as e:
                         log("identify_cards() cannot query %s using %s on device %i with handle %s: %s", prop, fn, i, handle, e)
                         continue
@@ -146,7 +164,7 @@ def identify_cards():
             #log.info("unitCount=%s", unitCount)
         except Exception as e:
             log("identify_cards() pynvml error", exc_info=True)
-            log.warn("Warning: failed to query the NVidia cards via NVML:")
+            log.warn("Warning: failed to query the NVidia cards using NVML:")
             log.warn(" %s", e)
         finally:
             if deviceCount is not None:
@@ -168,7 +186,7 @@ def get_cards(probe=True):
 def is_blacklisted():
     v = get_nvidia_module_version(True)
     try:
-        if v[0]<=MAX_TESTED:
+        if v[0]>MIN_VERSION:
             return False
     except Exception as e:
         log.warn("Warning: error checking driver version:")
@@ -196,34 +214,38 @@ def validate_driver_yuv444lossless():
             _version_warning = True
         if v:
             l("Warning: NVidia driver version %s is untested with NVENC", pver(v))
-            l(" (this encoder has been tested with versions up to %s.x only)", MAX_TESTED)
+            l(" (this encoder has been tested with versions %s.x and later only)", MIN_VERSION)
         if not envbool("XPRA_NVENC_YUV444P", False):
             l(" disabling YUV444P and lossless mode")
-            l(" use XPRA_NVENC_YUV444P=0 to force enable")
+            l(" use XPRA_NVENC_YUV444P=1 to force enable")
             return False
         l(" force enabling YUV444P and lossless mode")
     return True
 
-_init_message = False
-def nvenc_loaded():
-    global _init_message
-    if not _init_message:
-        log.info("NVENC successfully initialized")
-        _init_message = True
+
+def parse_nvfbc_hex_key(s):
+    #ie: 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10
+    #ie: 0102030405060708090A0B0C0D0E0F10
+    #start by removing spaces and 0x:
+    hexstr = s.replace("0x", "").replace(",", "").replace(" ", "")
+    import binascii
+    return binascii.unhexlify(hexstr)
 
 
-nvenc_license_keys = {}
-def get_nvenc_license_keys(nvenc_version=0):
-    global nvenc_license_keys
-    keys = nvenc_license_keys.get(nvenc_version)
+license_keys = {}
+def get_license_keys(version=0, basefilename="nvenc"):
+    global license_keys
+    filename = "%s%s.keys" % (basefilename, version or "")
+    keys = license_keys.get(filename)
     if keys is not None:
         return keys
-    env_keys = os.environ.get("XPRA_NVENC_CLIENT_KEY", "")
+    env_name = "XPRA_%s_CLIENT_KEY" % basefilename.upper()
+    env_keys = os.environ.get(env_name, "")
     if env_keys:
-        keys = [x.strip() for x in os.environ.get("XPRA_NVENC_CLIENT_KEY", "").split(",")]
-        log("using nvenc keys from environment variable XPRA_NVENC_CLIENT_KEY: %s", nvenc_license_keys)
+        keys = [x.strip() for x in env_keys.split(",")]
+        log("using %s keys from environment variable %s: %s", basefilename, env_name, csv(keys))
     else:
-        #try the license file
+        #try to load the license file
         keys = []
         try:
             #see read_xpra_defaults for an explanation of paths
@@ -232,12 +254,12 @@ def get_nvenc_license_keys(nvenc_version=0):
             for d in dirs:
                 if not d:
                     continue
-                keys_file = os.path.join(d, "nvenc%s.keys" % (nvenc_version or ""))
+                keys_file = os.path.join(d, filename)
                 keys_file = os.path.expanduser(keys_file)
                 if not os.path.exists(keys_file):
-                    log("get_nvenc_license_keys(%s) '%s' does not exist", nvenc_version, keys_file)
+                    log("get_license_keys(%s, %s) '%s' does not exist", basefilename, version, keys_file)
                     continue
-                log("loading nvenc%s keys from %s", nvenc_version, keys_file)
+                log("loading %s version %s keys from %s", basefilename, version, keys_file)
                 with open(keys_file, "rb") as f:
                     fkeys = []
                     for line in f:
@@ -252,10 +274,10 @@ def get_nvenc_license_keys(nvenc_version=0):
                         log("added key: %s", sline)
                     log("added %i key%s from %s", len(fkeys), engs(fkeys), keys_file)
                     keys += fkeys
-        except Exception as e:
-            log.error("error loading nvenc license keys: %s", e, exc_info=True)
-    nvenc_license_keys[nvenc_version] = keys
-    log("get_nvenc_license_keys(%s)=%s", nvenc_version, keys)
+        except Exception:
+            log.error("Error loading %s license keys", basefilename, exc_info=True)
+    license_keys[filename] = keys
+    log("get_nvenc_license_keys(%s)=%s", version, keys)
     return keys
 
 
@@ -271,7 +293,7 @@ def main():
             log.warn("Warning: this driver version is blacklisted")
         log.info("NVENC license keys:")
         for v in (0, 8):
-            keys = get_nvenc_license_keys(v)
+            keys = get_license_keys(v)
             log.info("* version %s: %s key(s)", v or "common", len(keys))
             for k in keys:
                 log.info("  %s", k)
