@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,9 +11,8 @@ if envbool("XPRA_USE_CYTHON_BENCODE", True):
         from xpra.net.bencode.cython_bencode import bencode, bdecode, __version__
         cython_bencode_loaded = True
     except ImportError as e:
-        from xpra.log import Logger
-        log = Logger("network")
-        log.warn("cannot load cython bencode module: %s", e)
+        from xpra.os_util import get_util_logger
+        get_util_logger().warn("Warning: cannot load cython bencode module: %s", e)
 if not cython_bencode_loaded:
     from xpra.net.bencode.bencode import bencode, bdecode, __version__      #@Reimport
 
