@@ -356,6 +356,17 @@ def getUbuntuVersion():
 def is_unity():
     return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().startswith("unity")
 
+def is_WSL():
+    if not POSIX:
+        return False
+    r = None
+    for f in ("/proc/sys/kernel/osrelease", "/proc/version"):
+        r = load_binary_file(f)
+        if r:
+            break
+    return r is not None and r.find(b"Microsoft")>=0
+
+
 def get_generic_os_name():
     for k,v in {
         "linux"     : "linux",
@@ -474,10 +485,8 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
             from xpra.platform.xposix.paths import get_runtime_dir
             d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
     if actual_username:
-        d.update({
-            "USERNAME"  : actual_username,
-            "USER"      : actual_username,
-            })
+        d["USERNAME"] = actual_username
+        d["USER"] = actual_username
     #first, expand the substitutions themselves,
     #as they may contain references to other variables:
     ssub = OrderedDict()
@@ -592,8 +601,14 @@ class nomodule_context(object):
     def __enter__(self):
         pass
     def __exit__(self, *_args):
-        if sys.modules[self.module_name] == None:
-            sys.modules[self.module_name] = self.saved_module
+        if sys.modules.get(self.module_name) == None:
+            if self.saved_module is None:
+                try:
+                    del sys.modules[self.module_name]
+                except KeyError:
+                    pass
+            else:
+                sys.modules[self.module_name] = self.saved_module
     def __repr__(self):
         return "nomodule_context(%s)" % self.module_name
 
