@@ -1,24 +1,34 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 
-from xpra.gtk_common.gtk_util import META_MASK
+from xpra.gtk_common.gtk_util import SHIFT_MASK, LOCK_MASK, META_MASK, CONTROL_MASK, SUPER_MASK, HYPER_MASK
 from xpra.platform.keyboard_base import KeyboardBase, log
 from xpra.platform.darwin.osx_menu import getOSXMenuHelper
 
 
 NUM_LOCK_KEYCODE = 71           #HARDCODED!
 #a key and the keys we want to translate it into when swapping keys
-#(a list so we can hopefully find a good match)
+#(a list so we can hopefully find a good match, best options come first)
 KEYS_TRANSLATION_OPTIONS = {
-                    "Control_L"     : ["Alt_L", "Meta_L"],
-                    "Control_R"     : ["Alt_R", "Meta_R"],
-                    "Meta_L"        : ["Control_L", "Control_R"],
-                    "Meta_R"        : ["Control_R", "Control_L"],
-                    }
+    #try to swap with "Meta" first, fallback to "Alt":
+    "Control_L"     : ["Meta_L", "Meta_R", "Alt_L", "Alt_R"],
+    "Control_R"     : ["Meta_R", "Meta_L", "Alt_R", "Alt_L"],
+    #"Meta" always swapped with "Control"
+    "Meta_L"        : ["Control_L", "Control_R"],
+    "Meta_R"        : ["Control_R", "Control_L"],
+    #"Alt" to "Super" (or "Hyper") so we can distinguish it from "Meta":
+    "Alt_L"         : ["Super_L", "Super_R", "Hyper_L", "Hyper_R"],
+    "Alt_R"         : ["Super_R", "Super_L", "Hyper_R", "Hyper_L"],
+    }
+#keys we always want to swap,
+#irrespective of the swap-keys option:
+ALWAYS_SWAP = os.environ.get("XPRA_MACOS_KEYS_ALWAYS_SWAP", "Alt_L,Alt_R").split(",")
+
 
 #data extracted from:
 #https://support.apple.com/en-us/HT201794
@@ -65,27 +75,40 @@ class Keyboard(KeyboardBase):
     """
 
     def __init__(self):
+        KeyboardBase.__init__(self)
+        self.init_vars()
+
+    def init_vars(self):
         self.swap_keys = True
         self.meta_modifier = None
-        self.control_modifier = None
+        self.control_modifier = "control"
+        self.super_modifier = None
+        self.hyper_modifier = None
         self.num_lock_modifier = None
         self.num_lock_state = True
         self.num_lock_keycode = NUM_LOCK_KEYCODE
         self.key_translations = {}
 
+    def cleanup(self):
+        self.init_vars()
+        KeyboardBase.cleanup(self)
+
 
     def get_layout_spec(self):
         layout = "us"
         layouts = ["us"]
         variant = ""
         variants = []
+        options = ""
         try:
-            from AppKit import NSTextInputContext       #@UnresolvedImport
-            ic = NSTextInputContext.new()
-            current_keyboard = ic.selectedKeyboardInputSource()
+            from AppKit import NSTextInputContext, NSTextView       #@UnresolvedImport
+            text_input_context = NSTextInputContext.alloc()
+            view = NSTextView.new()
+            text_input_context.initWithClient_(view)
+            current_keyboard = text_input_context.selectedKeyboardInputSource()
             code = APPLE_LAYOUTS.get(current_keyboard.split(".")[-1])
             log("get_layout_spec() current_keyboard=%s, code=%s", current_keyboard, code)
-            all_keyboards = ic.keyboardInputSources()
+            all_keyboards = text_input_context.keyboardInputSources()
             log("get_layout_spec() other keyboards=%s", all_keyboards)
             if code:
                 layout = code
@@ -100,12 +123,12 @@ class Keyboard(KeyboardBase):
             else:
                 if code not in layouts:
                     layouts.insert(0, code)
-            log("get_layout_spec() layout=%s, layouts=%s", layout, layouts)
+            log("get_layout_spec() view=%s, input_context=%s, layout=%s, layouts=%s", view, text_input_context, layout, layouts)
         except Exception as e:
             log("get_layout_spec()", exc_info=True)
             log.error("Error querying keyboard layout:")
             log.error(" %s", e)
-        return layout, layouts, variant, variants
+        return layout, layouts, variant, variants, options
 
     def get_keymap_modifiers(self):
         """
@@ -117,23 +140,26 @@ class Keyboard(KeyboardBase):
     def set_modifier_mappings(self, mappings):
         KeyboardBase.set_modifier_mappings(self, mappings)
         self.meta_modifier = self.modifier_keys.get("Meta_L") or self.modifier_keys.get("Meta_R")
-        self.control_modifier = self.modifier_keys.get("Control_L") or self.modifier_keys.get("Control_R")
+        self.control_modifier = self.modifier_keys.get("Control_L") or self.modifier_keys.get("Control_R") or "control"
+        self.super_modifier = self.modifier_keys.get("Super_L") or self.modifier_keys.get("Super_R")
+        self.hyper_modifier = self.modifier_keys.get("Hyper_L") or self.modifier_keys.get("Hyper_R")
         self.num_lock_modifier = self.modifier_keys.get("Num_Lock")
-        log("set_modifier_mappings(%s) meta=%s, control=%s, numlock=%s", mappings, self.meta_modifier, self.control_modifier, self.num_lock_modifier)
+        log("set_modifier_mappings(%s) meta=%s, control=%s, super=%s, hyper=%s, numlock=%s", mappings, self.meta_modifier, self.control_modifier, self.super_modifier, self.hyper_modifier, self.num_lock_modifier)
         #find the keysyms and keycodes to use for each key we may translate:
-        for orig_keysym in KEYS_TRANSLATION_OPTIONS.keys():
-            new_def = self.find_translation(orig_keysym)
+        for orig_keysym, keysyms in KEYS_TRANSLATION_OPTIONS.items():
+            new_def = self.find_translation(keysyms)
             if new_def is not None:
                 self.key_translations[orig_keysym] = new_def
         log("set_modifier_mappings(..) swap keys translations=%s", self.key_translations)
 
-    def find_translation(self, orig_keysym):
+    def find_translation(self, keysyms):
+        log("find_translation(%s)", keysyms)
         new_def = None
         #ie: keysyms : ["Meta_L", "Alt_L"]
-        keysyms = KEYS_TRANSLATION_OPTIONS.get(orig_keysym)
         for keysym in keysyms:
             #ie: "Alt_L":
             keycodes_defs = self.modifier_keycodes.get(keysym)
+            log("modifier_keycodes(%s)=%s", keysym, keycodes_defs)
             if not keycodes_defs:
                 #keysym not found
                 continue
@@ -143,44 +169,58 @@ class Keyboard(KeyboardBase):
                     #no keycode found, but better than nothing:
                     new_def = 0, keycode_def    #ie: (0, 'Alt_L')
                     continue
+                #an int alone is the keycode:
+                if type(keycode_def)==int:
+                    if keycode_def>0:
+                        #exact match, use it:
+                        return keycode_def, keysym
+                    new_def = 0, keysym
+                    continue
+                #below is for compatibility with older servers,
+                #(we may be able to remove some of this code already)
                 #look for a tuple of (keycode, keysym):
-                if type(keycode_def) not in (list, tuple):
+                if type(keycode_def) not in (list, tuple) or len(keycode_def)!=2:
                     continue
                 if type(keycode_def[0])!=int or type(keycode_def[1])!=str:
                     continue
-                #found one, use that:
-                return keycode_def           #(55, 'Alt_L')
+                if keycode_def[0]==0:
+                    new_def = keycode_def
+                    continue
+                #found a valid keycode, use this one:
+                return keycode_def              #ie: (55, 'Alt_L')
         return new_def
 
 
     def mask_to_names(self, mask):
-        names = KeyboardBase.mask_to_names(self, mask)
-        if self.swap_keys and self.meta_modifier is not None and self.control_modifier is not None:
-            meta_on = bool(mask & META_MASK)
-            meta_set = self.meta_modifier in names
-            control_set = self.control_modifier in names
-            log("mask_to_names names=%s, meta_on=%s, meta_set=%s, control_set=%s", names, meta_on, meta_set, control_set)
-            if meta_on and not control_set:
-                log("mask_to_names swapping meta for control: %s for %s", self.meta_modifier, self.control_modifier)
-                names.append(self.control_modifier)
-                if meta_set:
-                    names.remove(self.meta_modifier)
-            elif control_set and not meta_on:
-                log("mask_to_names swapping control for meta: %s for %s", self.control_modifier, self.meta_modifier)
-                names.remove(self.control_modifier)
-                if not meta_set:
-                    names.append(self.meta_modifier)
-        #deal with numlock:
+        if self.swap_keys:
+            control = self.meta_modifier
+            meta = self.control_modifier
+        else:
+            meta = self.meta_modifier
+            control = self.control_modifier
+        modmap = {
+            SHIFT_MASK      : "shift",
+            LOCK_MASK       : "lock",
+            SUPER_MASK      : self.super_modifier,
+            HYPER_MASK      : self.hyper_modifier,
+            META_MASK       : meta,
+            CONTROL_MASK    : control,
+            }
+        names = []
+        for modmask, modname in modmap.items():
+            if (mask & modmask) and modname:
+                names.append(modname)
+        #don't trust GTK with numlock:
         if self.num_lock_modifier is not None:
             if self.num_lock_state and self.num_lock_modifier not in names:
                 names.append(self.num_lock_modifier)
             elif not self.num_lock_state and self.num_lock_modifier in names:
                 names.remove(self.num_lock_modifier)
-        log("mask_to_names(%s)=%s", mask, names)
+        log("mask_to_names(%s)=%s swap_keys=%s, modmap=%s, num_lock_state=%s, num_lock_modifier=%s", mask, names, self.swap_keys, modmap, self.num_lock_state, self.num_lock_modifier)
         return names
 
     def process_key_event(self, send_key_action_cb, wid, key_event):
-        if self.swap_keys:
+        if self.swap_keys or key_event.keyname in ALWAYS_SWAP:
             trans = self.key_translations.get(key_event.keyname)
             if trans:
                 log("swap keys: translating key '%s' to %s", key_event, trans)
