@@ -23,12 +23,12 @@ class KeyboardConfig(KeyboardConfigBase):
         return []
 
 
-    def get_keycode(self, client_keycode, keyname, modifiers):
+    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers):
         global KEYCODES
         keycode = KEYCODES.get(keyname, -1)
         if keycode==-1:
             keycode = KEYCODES.get(keyname.upper(), -1)
-        log("get_keycode%s=%s", (client_keycode, keyname, modifiers), keycode)
+        log("get_keycode%s=%s", (client_keycode, keyname, pressed, modifiers), keycode)
         return keycode
 
 #we currently assume that all key events are sent using X11 names,
