@@ -1,16 +1,17 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import gobject
-import glib
-
 from xpra.log import Logger
 log = Logger("clipboard", "osx")
 
 from xpra.clipboard.translated_clipboard import TranslatedClipboardProtocolHelper
 from xpra.clipboard.clipboard_base import ClipboardProxy
+from xpra.gtk_common.gobject_compat import import_glib, import_gobject
+glib = import_glib()
+gobject = import_gobject()
+
 
 update_clipboard_change_count = None
 
@@ -67,7 +68,7 @@ def setup_watcher(get_change_count):
                     log("error in change callback %s: %s", x, e)
 
     from xpra.platform.ui_thread_watcher import get_UI_watcher
-    w = get_UI_watcher()
+    w = get_UI_watcher(glib.timeout_add)
     if w is None:
         log.warn("Warning: no UI watcher instance available")
         log.warn(" cannot detect clipboard change events")
@@ -101,7 +102,7 @@ class OSXClipboardProtocolHelper(TranslatedClipboardProtocolHelper):
             return None, None
         return TranslatedClipboardProtocolHelper._do_munge_raw_selection_to_wire(self, target, dtype, dformat, data)
 
-    def _get_clipboard_from_remote_handler(self, proxy, selection, target):
+    def _get_clipboard_from_remote_handler(self, _proxy, _selection, _target):
         #cannot work on osx, the nested mainloop doesn't run :(
         #so we don't even try and rely on the "wants_targets" flag
         #to get the server to send us the data with the token
