@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -18,16 +18,16 @@ def valid_dir(path):
 
 
 #helpers to easily override using env vars:
-def envaslist_or_delegate(env_name, impl):
+def envaslist_or_delegate(env_name, impl, *args):
     env_value = os.environ.get(env_name)
     if env_value is not None:
         return [env_value]
-    return impl()
-def env_or_delegate(env_name, impl):
+    return impl(*args)
+def env_or_delegate(env_name, impl, *args):
     env_value = os.environ.get(env_name)
     if env_value is not None:
         return env_value
-    return impl()
+    return impl(*args)
 
 
 def get_install_prefix():
@@ -37,22 +37,26 @@ def do_get_install_prefix():
 
 
 def get_system_conf_dirs():
-    return envaslist_or_delegate("XPRA_SYSCONF_DIRS", do_get_system_conf_dirs)
+    return envaslist_or_delegate("XPRA_SYSTEM_CONF_DIRS", do_get_system_conf_dirs)
 def do_get_system_conf_dirs():
-    prefix = get_install_prefix()
-    #the system wide configuration directory
-    if prefix == '/usr':
-        #default posix config location:
-        return ['/etc/xpra']
-    #hope the prefix is something like "/usr/local" or "$HOME/.local":
-    return [prefix + '/etc/xpra/']
+    #overriden in all platforms
+    return []
 
-def get_user_conf_dirs():
-    return envaslist_or_delegate("XPRA_USER_CONF_DIRS", do_get_user_conf_dirs)
-def do_get_user_conf_dirs():
-    #per-user configuration location:
-    if os.name!="posix" or os.getuid()>0:
-        return ["~/.xpra"]
+
+def get_ssh_conf_dirs():
+    return envaslist_or_delegate("XPRA_SSH_CONF_DIRS", do_get_ssh_conf_dirs)
+def do_get_ssh_conf_dirs():
+    return ["/etc/ssh", "/usr/local/etc/ssh", "~/.ssh", "~/ssh"]
+
+def get_ssh_known_hosts_files():
+    return envaslist_or_delegate("XPRA_SSH_KNOWN_HOSTS", do_get_ssh_known_hosts_files)
+def do_get_ssh_known_hosts_files():
+    return ("~/.ssh/known_hosts", "~/ssh/known_hosts")
+
+
+def get_user_conf_dirs(uid=None):
+    return envaslist_or_delegate("XPRA_USER_CONF_DIRS", do_get_user_conf_dirs, uid)
+def do_get_user_conf_dirs(_uid):
     return []
 
 def get_default_conf_dirs():
@@ -80,6 +84,27 @@ def do_get_download_dir():
         return "~"
     return d
 
+
+def get_libexec_dir():
+    return env_or_delegate("XPRA_LIBEXEC_DIR", do_get_libexec_dir)
+def do_get_libexec_dir():
+    return get_app_dir()
+
+
+def get_mmap_dir():
+    return env_or_delegate("XPRA_MMAP_DIR", do_get_mmap_dir)
+def do_get_mmap_dir():
+    import tempfile
+    return tempfile.gettempdir()
+
+
+def get_xpra_tmp_dir():
+    return env_or_delegate("XPRA_TMP_DIR", do_get_xpra_tmp_dir)
+def do_get_xpra_tmp_dir():
+    import tempfile
+    return tempfile.gettempdir()
+
+
 def get_script_bin_dirs():
     return envaslist_or_delegate("XPRA_SCRIPT_BIN_DIRS", do_get_script_bin_dirs)
 def do_get_script_bin_dirs():
@@ -88,7 +113,7 @@ def do_get_script_bin_dirs():
 def get_remote_run_xpra_scripts():
     return envaslist_or_delegate("XPRA_REMOTE_RUN_XPRA_SCRIPTS", do_get_remote_run_xpra_scripts)
 def do_get_remote_run_xpra_scripts():
-    return ["~/.xpra/run-xpra", "$XDG_RUNTIME_DIR/xpra/run-xpra", "xpra", "/usr/local/bin/xpra"]
+    return ["$XDG_RUNTIME_DIR/xpra/run-xpra", "xpra", "/usr/local/bin/xpra", "~/.xpra/run-xpra"]
 
 
 def get_sshpass_command():
@@ -97,9 +122,14 @@ def do_get_sshpass_command():
     import os.path
     def is_exe(fpath):
         return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
-    for path in os.environ["PATH"].split(os.pathsep):
+    SSHPASS = "sshpass"
+    from xpra.platform.features import EXECUTABLE_EXTENSION
+    if EXECUTABLE_EXTENSION:
+        SSHPASS = "sshpass.%s" % EXECUTABLE_EXTENSION
+    paths = os.environ["PATH"].split(os.pathsep)
+    for path in paths:
         path = path.strip('"')
-        exe_file = os.path.join(path, "sshpass")
+        exe_file = os.path.join(path, SSHPASS)
         if is_exe(exe_file):
             return exe_file
     return None
@@ -125,9 +155,10 @@ def default_get_app_dir():
                 return adir
     adir = os.path.dirname(inspect.getfile(sys._getframe(1)))
     def root_module(d):
-        pos = d.find("xpra%splatform" % os.path.sep)
-        if pos>=0:
-            return d[:pos]
+        for psep in (os.path.sep, "/", "\\"):
+            pos = d.find("xpra%splatform" % psep)
+            if pos>=0:
+                return d[:pos]
         return d
     if valid_dir(adir):
         return root_module(adir)
@@ -170,7 +201,7 @@ def get_icon_filename(basename=None, ext="png"):
     filename = basename
     fext = os.path.splitext(filename)[1]
     if not fext:
-        filename = "%s.%s" % (filename, ext)
+        filename = "%s.%s" % (basename, ext)
     if not os.path.isabs(filename):
         icon_dir = get_icon_dir()
         filename = os.path.join(icon_dir, filename)
@@ -208,6 +239,8 @@ def get_xpra_command():
         return shlex.split(envvalue)
     return do_get_xpra_command()
 def do_get_xpra_command():
+    return default_do_get_xpra_command()
+def default_do_get_xpra_command():
     #try to use the same "xpra" executable that launched this server:
     if sys.argv and sys.argv[0].lower().endswith("/xpra"):
         return [sys.argv[0]]
@@ -247,10 +280,15 @@ platform_import(globals(), "paths", False,
                 "do_get_install_prefix",
                 "do_get_default_conf_dirs",
                 "do_get_system_conf_dirs",
+                "do_get_ssh_conf_dirs",
+                "do_get_ssh_known_hosts_files",
                 "do_get_user_conf_dirs",
                 "do_get_socket_dirs",
                 "do_get_default_log_dirs",
                 "do_get_download_dir",
+                "do_get_libexec_dir",
+                "do_get_mmap_dir",
+                "do_get_xpra_tmp_dir",
                 "do_get_script_bin_dirs")
 
 def get_info():
@@ -258,11 +296,15 @@ def get_info():
             "install"           : {"prefix" : get_install_prefix()},
             "default_conf"      : {"dirs"   : get_default_conf_dirs()},
             "system_conf"       : {"dirs"   : get_system_conf_dirs()},
+            "ssh_conf"          : {"dirs"   : get_ssh_conf_dirs()},
             "user_conf"         : {"dirs"   : get_user_conf_dirs()},
             "socket"            : {"dirs"   : get_socket_dirs()},
             "log"               : {"dirs"   : get_default_log_dirs()},
             "download"          : {"dir"    : get_download_dir()},
             "app"               : {"dir"    : get_app_dir()},
+            "libexec"           : {"dir"    : get_libexec_dir()},
+            "mmap"              : {"dir"    : get_mmap_dir()},
+            "xpra-tmp"          : {"dir"    : get_xpra_tmp_dir()},
             "app"               : {"default" : {"dir"   : default_get_app_dir()}},
             "resources"         : get_resources_dir(),
             "icons"             : get_icon_dir(),
