@@ -39,20 +39,21 @@ frozen = getattr(sys, 'frozen', False)
 REDIRECT_OUTPUT = envbool("XPRA_REDIRECT_OUTPUT", frozen is True and GetConsoleCP()==0)
 if frozen:
     #cx_freeze paths:
+    PATH = os.environ.get("PATH", "").split(os.pathsep)
     edir = os.path.dirname(sys.executable)
     def jedir(*paths):
-        return "\\".join([edir]+list(paths))
+        return os.path.join(*([edir]+list(paths)))
     def addsyspath(*paths):
         v = jedir(*paths)
         if os.path.exists(v):
-            sys.path.append(v)
+            if v not in sys.path:
+                sys.path.append(v)
+            if os.path.isdir(v) and v not in PATH:
+                PATH.append(v)
     addsyspath('')
-    addsyspath('bin')
-    addsyspath('bin', 'etc')
-    addsyspath('bin', 'lib')
-    addsyspath('bin', 'share')
-    addsyspath('bin', 'library.zip')
+    addsyspath('lib')
     os.environ['GI_TYPELIB_PATH'] = jedir('lib', 'girepository-1.0')
+    os.environ["PATH"] = os.pathsep.join(PATH)
 
 #don't know why this breaks with Python 3 yet...
 FIX_UNICODE_OUT = envbool("XPRA_FIX_UNICODE_OUT", not REDIRECT_OUTPUT and PYTHON2)
@@ -246,6 +247,10 @@ class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
 _wait_for_input = False
 def set_wait_for_input():
     global _wait_for_input
+    wfi = os.environ.get("XPRA_WAIT_FOR_INPUT")
+    if wfi is not None:
+        _wait_for_input = wfi!="0"
+        return
     if is_wine():
         #don't wait for input when running under wine
         #(which usually does not popup a new shell window)
@@ -288,13 +293,14 @@ def do_init():
         #figure out if we want to wait for input at the end:
         set_wait_for_input()
         return
-    from xpra.platform import get_prgname
-    LOG_FILENAME = (get_prgname() or "Xpra")+".log"
-    from xpra.platform.win32.paths import _get_data_dir
-    d = _get_data_dir()
-    log_file = os.path.join(d, LOG_FILENAME)
-    sys.stdout = open(log_file, "a")
-    sys.stderr = sys.stdout
+    if envbool("XPRA_LOG_TO_FILE", True):
+        from xpra.platform import get_prgname
+        LOG_FILENAME = (get_prgname() or "Xpra")+".log"
+        from xpra.platform.win32.paths import _get_data_dir
+        d = _get_data_dir()
+        log_file = os.path.join(d, LOG_FILENAME)
+        sys.stdout = open(log_file, "a")
+        sys.stderr = sys.stdout
 
 
 MB_ICONEXCLAMATION  = 0x00000030
@@ -303,14 +309,14 @@ MB_SYSTEMMODAL      = 0x00001000
 def _show_message(message, uType):
     global prg_name
     #TODO: detect cx_freeze equivallent
-    SHOW_MESSAGEBOX = envbool("XPRA_MESSAGEBOX", True)
-    if SHOW_MESSAGEBOX and REDIRECT_OUTPUT:
+    SHOW_MESSAGEBOX = envbool("XPRA_MESSAGEBOX", REDIRECT_OUTPUT)
+    if SHOW_MESSAGEBOX:
         #try to use an alert box since no console output will be shown:
         try:
             MessageBoxA(0, message.encode(), prg_name.encode(), uType)
             return
-        except:
-            pass
+        except Exception as e:
+            print("Error: cannot show alert box: %s" % (e,))
     print(message)
 
 def command_info(message):
@@ -320,24 +326,6 @@ def command_error(message):
     _show_message(message, MB_ICONEXCLAMATION | MB_SYSTEMMODAL)
 
 
-def get_main_fallback(cmd):
-    set_wait_for_input()
-    global _wait_for_input
-    if _wait_for_input:
-        #something will be shown in a console,
-        #so don't bother showing anything else
-        return None
-    if cmd.find("Xpra_Audio.exe")>=0:
-        #don't show the launcher when trying to use this command
-        return None
-    #try the launcher (better than nothing!)
-    try:
-        from xpra.client.gtk_base.client_launcher import main
-        return main
-    except:
-        return None
-
-
 def do_clean():
     global _wait_for_input
     if _wait_for_input:
