@@ -7,7 +7,7 @@
 import ctypes
 from ctypes.wintypes import HANDLE
 
-from xpra.platform.win32.common import GetKeyState, GetKeyboardLayoutList, GetKeyboardLayout, SystemParametersInfoA
+from xpra.platform.win32.common import GetKeyState, GetKeyboardLayoutList, GetKeyboardLayout, GetIntSystemParametersInfo
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.keyboard_base import KeyboardBase
 from xpra.keyboard.layouts import WIN32_LAYOUTS
@@ -25,21 +25,12 @@ def _GetKeyboardLayoutList():
     count = GetKeyboardLayoutList(max_items, ctypes.byref(handle_list))
     layouts = []
     for i in range(count):
-        layouts.append(handle_list[i])
+        layouts.append(int(handle_list[i]))
     return layouts
 
-def GetIntSystemParametersInfo(key):
-    rv = ctypes.wintypes.INT()
-    SystemParametersInfoA(key, 0, ctypes.byref(rv), 0)
-    return rv.value
-
 
 EMULATE_ALTGR = envbool("XPRA_EMULATE_ALTGR", True)
 EMULATE_ALTGR_CONTROL_KEY_DELAY = envint("XPRA_EMULATE_ALTGR_CONTROL_KEY_DELAY", 50)
-if EMULATE_ALTGR:
-    #needed for altgr emulation timeouts:
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
 
 
 class Keyboard(KeyboardBase):
@@ -122,6 +113,7 @@ class Keyboard(KeyboardBase):
         layouts = []
         variant = None
         variants = None
+        options = ""
         try:
             l = _GetKeyboardLayoutList()
             log("GetKeyboardLayoutList()=%s", csv(hex(v) for v in l))
@@ -133,7 +125,8 @@ class Keyboard(KeyboardBase):
                     if _layout not in layouts:
                         layouts.append(_layout)
         except Exception as e:
-            log.error("Error: failed to detect keyboard layouts: %s", e)
+            log.error("Error: failed to detect keyboard layouts:")
+            log.error(" %s", e)
         try:
             hkl = GetKeyboardLayout(0)
             log("GetKeyboardLayout(0)=%#x", hkl)
@@ -146,8 +139,9 @@ class Keyboard(KeyboardBase):
             else:
                 layouts.append(layout)
         except Exception as e:
-            log.error("Error: failed to detect keyboard layout: %s", e)
-        return layout,layouts,variant,variants
+            log.error("Error: failed to detect keyboard layout:")
+            log.error(" %s", e)
+        return layout,layouts,variant,variants, options
 
     def get_keyboard_repeat(self):
         try:
@@ -190,6 +184,9 @@ class Keyboard(KeyboardBase):
                 if key_event.pressed:
                     if rmenu in (0, 1):
                         self.delayed_event = (send_key_action_cb, wid, key_event)
+                        #needed for altgr emulation timeouts:
+                        from xpra.gtk_common.gobject_compat import import_glib
+                        glib = import_glib()
                         glib.timeout_add(EMULATE_ALTGR_CONTROL_KEY_DELAY, self.send_delayed_key)
                     return
                 if not key_event.pressed and rmenu not in (0, 1):
