@@ -6,35 +6,36 @@
 
 #@PydevCodeAnalysisIgnore
 
-from ctypes import WinDLL, addressof, byref, c_ulong, c_char_p, c_char, c_void_p, cast, string_at
+import errno
+from ctypes import addressof, byref, c_ulong, c_char_p, c_char, c_void_p, cast, string_at
 
+from xpra.os_util import strtobytes
 from xpra.net.bytestreams import Connection
+from xpra.net.common import ConnectionClosedException
+from xpra.platform.win32.common import CloseHandle
 from xpra.platform.win32.namedpipes.common import OVERLAPPED, WAIT_STR, INVALID_HANDLE_VALUE, ERROR_PIPE_BUSY, ERROR_PIPE_NOT_CONNECTED, INFINITE, ERROR_STR, ERROR_BROKEN_PIPE, ERROR_IO_PENDING
+from xpra.platform.win32.namedpipes.common import CreateEventA, CreateFileA, ReadFile, WriteFile, DisconnectNamedPipe, FlushFileBuffers, WaitNamedPipeA, GetLastError, SetNamedPipeHandleState, WaitForSingleObject, GetOverlappedResult
 from xpra.platform.win32.constants import FILE_FLAG_OVERLAPPED, GENERIC_READ, GENERIC_WRITE, OPEN_EXISTING, PIPE_READMODE_BYTE
 
 from xpra.log import Logger
 log = Logger("network", "named-pipe", "win32")
 
-kernel32 = WinDLL("kernel32", use_last_error=True)
-CreateEventA = kernel32.CreateEventA
-CreateFileA = kernel32.CreateFileA
-ReadFile = kernel32.ReadFile
-WriteFile = kernel32.WriteFile
-CloseHandle = kernel32.CloseHandle
-DisconnectNamedPipe = kernel32.DisconnectNamedPipe
-FlushFileBuffers = kernel32.FlushFileBuffers
-WaitNamedPipeA = kernel32.WaitNamedPipeA
-GetLastError = kernel32.GetLastError
-SetNamedPipeHandleState = kernel32.SetNamedPipeHandleState
-WaitForSingleObject = kernel32.WaitForSingleObject
-GetOverlappedResult = kernel32.GetOverlappedResult
-
 BUFSIZE = 65536
 
+CONNECTION_CLOSED_ERRORS = {
+    ERROR_BROKEN_PIPE           : "BROKENPIPE",
+    ERROR_PIPE_NOT_CONNECTED    : "PIPE_NOT_CONNECTED",
+    }
+#some of these may be redundant or impossible to hit? (does not hurt I think)
+for x in ("WSAENETDOWN", "WSAENETUNREACH", "WSAECONNABORTED", "WSAECONNRESET",
+          "WSAENOTCONN", "WSAESHUTDOWN", "WSAETIMEDOUT", "WSAETIMEDOUT",
+          "WSAEHOSTUNREACH", "WSAEDISCON"):
+    CONNECTION_CLOSED_ERRORS[getattr(errno, x)] = x
+
 
 class NamedPipeConnection(Connection):
     def __init__(self, name, pipe_handle):
-        log("NamedPipeConnection(%s, %i)", name, pipe_handle)
+        log("NamedPipeConnection(%s, %#x)", name, pipe_handle)
         Connection.__init__(self, name, "named-pipe")
         self.pipe_handle = pipe_handle
         self.read_buffer = (c_char*BUFSIZE)()
@@ -52,6 +53,17 @@ class NamedPipeConnection(Connection):
         self.write_overlapped.InternalHigh = None
         self.write_overlapped.union.Pointer = None
 
+    def can_retry(self, e):
+        code = e.args[0]
+        if code==errno.WSAEWOULDBLOCK:      #@UndefinedVariable
+            return "WSAEWOULDBLOCK"
+        #convert those to a connection closed:
+        closed = CONNECTION_CLOSED_ERRORS.get(code)
+        if closed:
+            raise ConnectionClosedException(e)
+        return False
+
+
     def untilConcludes(self, fn, *args, **kwargs):
         try:
             return Connection.untilConcludes(self, fn, *args, **kwargs)
@@ -127,7 +139,11 @@ class NamedPipeConnection(Connection):
         self.pipe_handle = None
         def _close_err(fn, e):
             l = log.error
-            code = e[0]
+            try:
+                code = e[0]
+            except:
+                #python3?
+                code = 0
             if code==ERROR_PIPE_NOT_CONNECTED:
                 l = log.debug
             l("Error: %s(%s) %i: %s", fn, ph, code, e)
@@ -161,8 +177,8 @@ def connect_to_namedpipe(pipe_name, timeout=10):
     while True:
         if time.time()-start>=timeout:
             raise Exception("timeout waiting for named pipe '%s'" % pipe_name)
-        pipe_handle = CreateFileA(pipe_name, GENERIC_READ | GENERIC_WRITE, 0, None, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, None)
-        log("CreateFileA(%s)=%s", pipe_name, pipe_handle)
+        pipe_handle = CreateFileA(strtobytes(pipe_name), GENERIC_READ | GENERIC_WRITE, 0, None, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0)
+        log("CreateFileA(%s)=%#x", pipe_name, pipe_handle)
         if pipe_handle!=INVALID_HANDLE_VALUE:
             break
         if GetLastError()!=ERROR_PIPE_BUSY:
@@ -171,7 +187,7 @@ def connect_to_namedpipe(pipe_name, timeout=10):
             raise Exception("timeout waiting for named pipe '%s'" % pipe_name)
     #we have a valid handle!
     dwMode = c_ulong(PIPE_READMODE_BYTE)
-    r = SetNamedPipeHandleState(pipe_handle, byref(dwMode), None, None);
+    r = SetNamedPipeHandleState(pipe_handle, byref(dwMode), None, None)
     log("SetNamedPipeHandleState(..)=%i", r)
     if not r:
         log.warn("Warning: SetNamedPipeHandleState failed")
