@@ -13,17 +13,16 @@ from xpra.log import Logger
 from xpra.os_util import strtobytes
 from xpra.util import envbool, prettify_plug_name, csv
 log = Logger("shadow", "win32")
-traylog = Logger("tray")
 shapelog = Logger("shape")
 cursorlog = Logger("cursor")
-netlog = Logger("network")
 keylog = Logger("keyboard")
+screenlog = Logger("screen")
 
 from collections import namedtuple
 from xpra.util import XPRA_APP_ID, XPRA_IDLE_NOTIFICATION_ID
 from xpra.scripts.config import InitException
 from xpra.server.gtk_server_base import GTKServerBase
-from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
+from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase, MULTI_WINDOW
 from xpra.server.shadow.root_window_model import RootWindowModel
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.gui import get_desktop_name, get_fixed_cursor_size
@@ -45,6 +44,7 @@ from xpra.platform.win32.common import (EnumWindows, EnumWindowsProc, FindWindow
                                         GetIconInfoExW, ICONINFOEXW,
                                         GetObjectA,
                                         GetKeyboardState, SetKeyboardState,
+                                        EnumDisplayMonitors, GetMonitorInfo,
                                         mouse_event)
 
 NOEVENT = object()
@@ -73,6 +73,14 @@ def get_root_window_size():
     h = GetSystemMetrics(win32con.SM_CYVIRTUALSCREEN)
     return w, h
 
+def get_monitors():
+    monitors = []
+    for m in EnumDisplayMonitors():
+        mi = GetMonitorInfo(m)
+        monitors.append(mi)
+    return monitors
+
+
 def get_cursor_data(hCursor):
     #w, h = get_fixed_cursor_size()
     if not hCursor:
@@ -327,12 +335,6 @@ class Win32RootWindowModel(RootWindowModel):
             return shape
         return RootWindowModel.get_property(self, prop)
 
-    def get_dimensions(self):
-        return self.get_root_window_size()
-
-    def get_root_window_size(self):
-        return get_root_window_size()
-
 
 class ShadowServer(GTKShadowServerBase):
 
@@ -402,10 +404,29 @@ class ShadowServer(GTKShadowServerBase):
         return Win32Tray(self, XPRA_APP_ID, self.tray_menu, "Xpra Shadow Server", "server-notconnected", None, self.tray_click_callback, None, self.tray_exit_callback)
 
 
-    def makeRootWindowModels(self):
+    def setup_capture(self):
         w, h = get_root_window_size()
-        self.capture = init_capture(w, h, self.pixel_depth)
-        return (Win32RootWindowModel(self.root, self.capture),)
+        return init_capture(w, h, self.pixel_depth)
+
+    def makeRootWindowModels(self):
+        log("makeRootWindowModels() root=%s", self.root)
+        self.capture = self.setup_capture()
+        if not MULTI_WINDOW:
+            return (RootWindowModel(self.root, self.capture),)
+        models = []
+        monitors = get_monitors()
+        for i, monitor in enumerate(monitors):
+            geom = monitor["Monitor"]
+            x1, y1, x2, y2 = geom
+            assert x1<x2 and y1<y2
+            model = RootWindowModel(self.root, self.capture)
+            model.title = monitor["Device"].lstrip("\\\\.\\")
+            model.geometry = x1, y1, x2-x1, y2-y1
+            screenlog("monitor %i: %10s geometry=%s (from %s)", i, model.title, model.geometry, geom)
+            models.append(model)
+            screenlog("makeRootWindowModels: model(%s)=%s", monitor, model)
+        log("makeRootWindowModels()=%s", models)
+        return models
 
 
     def refresh(self):
@@ -449,12 +470,13 @@ class ShadowServer(GTKShadowServerBase):
     def do_process_mouse_common(self, proto, wid, pointer, *_args):
         #adjust pointer position for offset in client:
         try:
-            x, y = pointer
-            SetPhysicalCursorPos(x, y)
+            x, y = pointer[:2]
+            assert SetPhysicalCursorPos(x, y)
         except Exception as e:
             log("SetPhysicalCursorPos%s failed", pointer, exc_info=True)
             log.error("Error: failed to move the cursor:")
             log.error(" %s", e)
+        return pointer
 
     def clear_keys_pressed(self):
         keystate = (BYTE*256)()
@@ -467,8 +489,13 @@ class ShadowServer(GTKShadowServerBase):
                 if keystate[i]:
                     pressed.append(vknames.get(i, i))
             keylog("keys still pressed: %s", csv(pressed))
-            #clear all: (this does not affect lock modifiers: caps , num, scroll)
-            keystate = (BYTE*256)()
+            for x in (
+                win32con.VK_LSHIFT,     win32con.VK_RSHIFT,     win32con.VK_SHIFT,
+                win32con.VK_LCONTROL,   win32con.VK_RCONTROL,   win32con.VK_CONTROL,
+                win32con.VK_LMENU,      win32con.VK_RMENU,      win32con.VK_MENU,
+                win32con.VK_LWIN,       win32con.VK_RWIN,
+                ):
+                keystate[x] = 0
             SetKeyboardState(keystate)
 
     def get_keyboard_config(self, _props=None):
@@ -480,8 +507,9 @@ class ShadowServer(GTKShadowServerBase):
     def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, *args):
         self._update_modifiers(proto, wid, modifiers)
         pointer = self._process_mouse_common(proto, wid, pointer)
-        self._server_sources.get(proto).user_event()
-        self.button_action(pointer, button, pressed, -1, *args)
+        if pointer:
+            self._server_sources.get(proto).user_event()
+            self.button_action(pointer, button, pressed, -1, *args)
 
     def button_action(self, pointer, button, pressed, deviceid=-1, *args):
         event = BUTTON_EVENTS.get((button, pressed))
@@ -491,7 +519,7 @@ class ShadowServer(GTKShadowServerBase):
         elif event is NOEVENT:
             return
         dwFlags, dwData = event
-        x, y = pointer
+        x, y = pointer[:2]
         mouse_event(dwFlags, x, y, dwData, 0)
 
     def make_hello(self, source):
