@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,6 +12,7 @@ from ctypes import wintypes, byref
 from xpra.log import Logger
 log = Logger("tray", "win32")
 
+from xpra.os_util import monotonic_time
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.gui import EnumDisplayMonitors, GetMonitorInfo
 from xpra.platform.win32.win32_NotifyIcon import win32NotifyIcon
@@ -28,7 +29,7 @@ class Win32Tray(TrayBase):
         self.calculate_offset()
         self.default_icon_extension = "ico"
         icon_filename = get_icon_filename(self.default_icon_filename, "ico")
-        self.tray_widget = win32NotifyIcon(self.app_id, self.tooltip, self.move_cb, self.click_cb, self.exit_cb, None, icon_filename)
+        self.tray_widget = win32NotifyIcon(self.app_id, self.tooltip, self.move_cb, self.click_cb, self.exit_cb, icon_filename)
         el = get_win32_event_listener()
         if el:
             el.add_event_callback(win32con.WM_DISPLAYCHANGE, self.calculate_offset)
@@ -50,8 +51,8 @@ class Win32Tray(TrayBase):
 
     def getHWND(self):
         if self.tray_widget is None:
-            return    None
-        return    self.tray_widget.hwnd
+            return None
+        return self.tray_widget.hwnd
 
     def cleanup(self):
         log("Win32Tray.cleanup() tray_widget=%s", self.tray_widget)
@@ -63,7 +64,7 @@ class Win32Tray(TrayBase):
             el.remove_event_callback(win32con.WM_DISPLAYCHANGE, self.calculate_offset)
         log("Win32Tray.cleanup() ended")
 
-    def calculate_offset(self, *args):
+    def calculate_offset(self, *_args):
         #GTK returns coordinates as unsigned ints, but win32 can give us negative coordinates!
         self.offset_x = 0
         self.offset_y = 0
@@ -86,10 +87,12 @@ class Win32Tray(TrayBase):
     def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
         if self.tray_widget:
             self.tray_widget.set_icon_from_data(pixels, has_alpha, w, h, rowstride, options)
+            self.icon_timestamp = monotonic_time()
 
     def do_set_icon_from_file(self, filename):
         if self.tray_widget:
             self.tray_widget.set_icon(filename)
+            self.icon_timestamp = monotonic_time()
 
     def set_blinking(self, on):
         if self.tray_widget:
