@@ -11,10 +11,13 @@ import os
 from xpra.log import Logger
 log = Logger("tray", "posix")
 
-from xpra.os_util import is_unity
+from xpra.util import envbool
+from xpra.os_util import is_unity, monotonic_time, getUbuntuVersion, PYTHON2
 from xpra.client.tray_base import TrayBase
 from xpra.platform.paths import get_icon_dir, get_icon_filename
 
+DELETE_TEMP_FILE = envbool("XPRA_APPINDICATOR_DELETE_TEMP_FILE", True)
+
 
 _appindicator = False
 def get_appindicator():
@@ -23,8 +26,15 @@ def get_appindicator():
         try:
             import sys
             if "gi" in sys.modules:
-                from gi.repository import AppIndicator3 #@UnresolvedImport @Reimport
-                _appindicator = AppIndicator3
+                if getUbuntuVersion()>=(18,4) and is_unity():
+                    #causes segfaults just by importing it
+                    #shambolic
+                    pass
+                else:
+                    import gi
+                    gi.require_version('AppIndicator3', '0.1')
+                    from gi.repository import AppIndicator3 #@UnresolvedImport @Reimport
+                    _appindicator = AppIndicator3
             else:
                 import appindicator                     #@UnresolvedImport
                 _appindicator = appindicator
@@ -32,6 +42,14 @@ def get_appindicator():
             _appindicator = None
     return _appindicator
 
+def get_application_category(appindicator):
+    v = getattr(appindicator, "CATEGORY_APPLICATION_STATUS", None)
+    if v is None:
+        cat = getattr(appindicator, "IndicatorCategory", None)
+        if cat:
+            v = getattr(cat, "APPLICATION_STATUS", None)
+    return v
+
 def can_use_appindicator():
     return get_appindicator() is not None and is_unity()
 
@@ -44,7 +62,14 @@ class AppindicatorTray(TrayBase):
         self.appindicator = get_appindicator()
         self._has_icon = False
         assert self.appindicator, "appindicator is not available!"
-        self.tray_widget = self.appindicator.Indicator(self.tooltip, filename, self.appindicator.CATEGORY_APPLICATION_STATUS)
+        category = get_application_category(self.appindicator)
+        assert category is not None, "appindicator category is not available!"
+        if PYTHON2:
+            self.tray_widget = self.appindicator.Indicator(self.tooltip, filename, category)
+        else:
+            self.tray_widget = self.appindicator.Indicator()
+            self.tray_widget.set_property("label", self.tooltip)
+            self.tray_widget.set_property("category", category)
         if hasattr(self.tray_widget, "set_icon_theme_path"):
             self.tray_widget.set_icon_theme_path(get_icon_dir())
         self.tray_widget.set_attention_icon("xpra.png")
@@ -55,10 +80,10 @@ class AppindicatorTray(TrayBase):
         if self.menu:
             self.tray_widget.set_menu(self.menu)
 
-    def hide(self, *args):
+    def hide(self, *_args):
         self.tray_widget.set_status(self.appindicator.STATUS_PASSIVE)
 
-    def show(self, *args):
+    def show(self, *_args):
         self.tray_widget.set_status(self.appindicator.STATUS_ACTIVE)
 
     def set_blinking(self, on):
@@ -72,7 +97,7 @@ class AppindicatorTray(TrayBase):
         if not self._has_icon:
             self.tray_widget.set_label(text or "Xpra")
 
-    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
+    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, _options={}):
         #use a temporary file (yuk)
         try:
             from gtk import gdk
@@ -81,13 +106,14 @@ class AppindicatorTray(TrayBase):
             return
         import tempfile
         try:
-            _, filename = tempfile.mkstemp(suffix="png")
+            _, filename = tempfile.mkstemp(suffix=".png")
             log("set_icon_from_data%s using temporary file %s", ("%s pixels" % len(pixels), has_alpha, w, h, rowstride), filename)
             tray_icon = gdk.pixbuf_new_from_data(pixels, gdk.COLORSPACE_RGB, has_alpha, 8, w, h, rowstride)
             tray_icon.save(filename, "png")
             self.do_set_icon_from_file(filename)
         finally:
-            os.unlink(filename)
+            if DELETE_TEMP_FILE:
+                os.unlink(filename)
 
     def do_set_icon_from_file(self, filename):
         if not hasattr(self.tray_widget, "set_icon_theme_path"):
@@ -103,6 +129,7 @@ class AppindicatorTray(TrayBase):
         log("do_set_icon_from_file(%s) setting icon=%s", filename, noext)
         self.tray_widget.set_icon(noext)
         self._has_icon = True
+        self.icon_timestamp = monotonic_time()
 
 
 def main():
@@ -123,7 +150,7 @@ def main():
     item = gtk.MenuItem("Some Menu Item Here")
     menu.append(item)
     menu.show_all()
-    a = AppindicatorTray(menu, "test", "xpra.png", None, None, None, gtk.main_quit)
+    a = AppindicatorTray(None, None, menu, "test", "xpra.png", None, None, None, gtk.main_quit)
     a.show()
     glib.timeout_add(1000*10, gtk.main_quit)
     gtk.main()
