@@ -1,13 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import sys
 import struct
-import binascii
 
 from xpra.log import Logger
 log = Logger("posix")
@@ -17,28 +16,37 @@ dbuslog = Logger("posix", "dbus")
 traylog = Logger("posix", "tray")
 menulog = Logger("posix", "menu")
 mouselog = Logger("posix", "mouse")
+xinputlog = Logger("posix", "xinput")
 
-from xpra.os_util import strtobytes, bytestostr
-from xpra.util import iround, envbool, csv
-from xpra.gtk_common.gobject_compat import get_xid, is_gtk3
+from xpra.os_util import bytestostr, hexstr
+from xpra.util import iround, envbool, envint, csv
+from xpra.gtk_common.gtk_util import get_xwindow
+from xpra.gtk_common.gobject_compat import is_gtk3
+from xpra.os_util import is_X11, is_Wayland
 
-try:
-    from xpra.x11.bindings.window_bindings import X11WindowBindings
-    from xpra.x11.bindings.xi2_bindings import X11XI2Bindings   #@UnresolvedImport
-except Exception as e:
-    log.error("no X11 bindings", exc_info=True)
-    X11WindowBindings = None
-    X11XI2Bindings = None
+X11WindowBindings = None
+X11XI2Bindings = None
+if is_X11():
+    try:
+        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+        from xpra.x11.bindings.xi2_bindings import X11XI2Bindings       #@UnresolvedImport
+    except Exception as e:
+        log.error("no X11 bindings", exc_info=True)
+        del e
 
 device_bell = None
 GTK_MENUS = envbool("XPRA_GTK_MENUS", False)
 RANDR_DPI = envbool("XPRA_RANDR_DPI", True)
 XSETTINGS_DPI = envbool("XPRA_XSETTINGS_DPI", True)
 USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
+XINPUT_WHEEL_DIV = envint("XPRA_XINPUT_WHEEL_DIV", 15)
+DBUS_SCREENSAVER = envbool("XPRA_DBUS_SCREENSAVER", False)
 
 
-def hexstr(v):
-    return binascii.hexlify(strtobytes(v))
+def gl_check():
+    if not is_X11() and is_Wayland():
+        return "disabled under wayland with GTK3 (buggy)"
+    return None
 
 
 def get_native_system_tray_classes():
@@ -54,16 +62,18 @@ def get_native_system_tray_classes():
 
 def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
-    try:
-        wm_check = _get_X11_root_property("_NET_SUPPORTING_WM_CHECK", "WINDOW")
-        if wm_check:
-            xid = struct.unpack("=I", wm_check)[0]
-            traylog("_NET_SUPPORTING_WM_CHECK window=%#x", xid)
-            wm_name = _get_X11_window_property(xid, "_NET_WM_NAME", "UTF8_STRING")
-            traylog("_NET_WM_NAME=%s", wm_name)
-    except Exception as e:
-        traylog.error("Error accessing window manager information:")
-        traylog.error(" %s", e)
+    if is_X11():
+        try:
+            wm_check = _get_X11_root_property("_NET_SUPPORTING_WM_CHECK", "WINDOW")
+            if wm_check:
+                xid = struct.unpack(b"=I", wm_check)[0]
+                traylog("_NET_SUPPORTING_WM_CHECK window=%#x", xid)
+                wm_name = _get_X11_window_property(xid, "_NET_WM_NAME", "UTF8_STRING")
+                traylog("_NET_WM_NAME=%s", wm_name)
+                return wm_name.decode("utf-8")
+        except Exception as e:
+            traylog.error("Error accessing window manager information:")
+            traylog.error(" %s", e)
     return wm_name
 
 def get_native_tray_classes():
@@ -81,12 +91,12 @@ def get_native_tray_classes():
 def get_native_notifier_classes():
     ncs = []
     try:
-        from xpra.client.notifications.dbus_notifier import DBUS_Notifier_factory
+        from xpra.notifications.dbus_notifier import DBUS_Notifier_factory
         ncs.append(DBUS_Notifier_factory)
     except Exception as e:
         dbuslog("cannot load dbus notifier: %s", e)
     try:
-        from xpra.client.notifications.pynotify_notifier import PyNotify_Notifier
+        from xpra.notifications.pynotify_notifier import PyNotify_Notifier
         ncs.append(PyNotify_Notifier)
     except Exception as e:
         log("cannot load pynotify notifier: %s", e)
@@ -121,6 +131,7 @@ def _get_X11_root_property(name, req_type):
         root_xid = X11Window.getDefaultRootWindow()
         return _get_X11_window_property(root_xid, name, req_type)
     except Exception as e:
+        log("_get_X11_root_property(%s, %s)", name, req_type, exc_info=True)
         log.warn("Warning: failed to get X11 root property '%s'", name)
         log.warn(" %s", e)
     return None
@@ -206,7 +217,7 @@ def _get_xsettings_dict():
 
 
 def _get_xsettings_dpi():
-    if XSETTINGS_DPI:
+    if XSETTINGS_DPI and is_X11():
         from xpra.x11.xsettings_prop import XSettingsTypeInteger
         d = _get_xsettings_dict()
         for k,div in {
@@ -218,21 +229,24 @@ def _get_xsettings_dpi():
             if k in d:
                 value_type, value = d.get(k)
                 if value_type==XSettingsTypeInteger:
-                    screenlog("_get_xsettings_dpi() found %s=%s", k, value)
-                    return max(10, min(1000, value/div))
+                    actual_value = max(10, min(1000, value//div))
+                    screenlog("_get_xsettings_dpi() found %s=%s, div=%i, actual value=%i", k, value, div, actual_value)
+                    return actual_value
     return -1
 
 def _get_randr_dpi():
-    if RANDR_DPI:
+    if RANDR_DPI and not is_Wayland():
         try:
+            from xpra.gtk_common.error import xsync
             from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
-            randr_bindings = RandRBindings()
-            wmm, hmm = randr_bindings.get_screen_size_mm()
-            w, h =  randr_bindings.get_screen_size()
-            dpix = iround(w * 25.4 / wmm)
-            dpiy = iround(h * 25.4 / hmm)
-            screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
-            return dpix, dpiy
+            with xsync:
+                randr_bindings = RandRBindings()
+                wmm, hmm = randr_bindings.get_screen_size_mm()
+                w, h =  randr_bindings.get_screen_size()
+                dpix = iround(w * 25.4 / wmm)
+                dpiy = iround(h * 25.4 / hmm)
+                screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
+                return dpix, dpiy
         except Exception as e:
             screenlog.warn("failed to get dpi: %s", e)
     return -1, -1
@@ -251,27 +265,28 @@ def get_ydpi():
 
 
 def get_icc_info():
-    try:
-        data = _get_X11_root_property("_ICC_PROFILE", "CARDINAL")
-        if data:
-            screenlog("_ICC_PROFILE=%s (%s)", type(data), len(data))
-            version = _get_X11_root_property("_ICC_PROFILE_IN_X_VERSION", "CARDINAL")
-            screenlog("get_icc_info() found _ICC_PROFILE_IN_X_VERSION=%s, _ICC_PROFILE=%s", hexstr(version or ""), hexstr(data))
-            icc = {
-                    "source"    : "_ICC_PROFILE",
-                    "data"      : data,
-                    }
-            if version:
-                try:
-                    version = ord(version)
-                except:
-                    pass
-                icc["version"] = version
-            return icc
-    except Exception as e:
-        screenlog.error("Error: cannot access _ICC_PROFILE X11 window property")
-        screenlog.error(" %s", e)
-        screenlog("get_icc_info()", exc_info=True)
+    if not is_Wayland():
+        try:
+            data = _get_X11_root_property("_ICC_PROFILE", "CARDINAL")
+            if data:
+                screenlog("_ICC_PROFILE=%s (%s)", type(data), len(data))
+                version = _get_X11_root_property("_ICC_PROFILE_IN_X_VERSION", "CARDINAL")
+                screenlog("get_icc_info() found _ICC_PROFILE_IN_X_VERSION=%s, _ICC_PROFILE=%s", hexstr(version or ""), hexstr(data))
+                icc = {
+                        "source"    : "_ICC_PROFILE",
+                        "data"      : data,
+                        }
+                if version:
+                    try:
+                        version = ord(version)
+                    except:
+                        pass
+                    icc["version"] = version
+                return icc
+        except Exception as e:
+            screenlog.error("Error: cannot access _ICC_PROFILE X11 window property")
+            screenlog.error(" %s", e)
+            screenlog("get_icc_info()", exc_info=True)
     return {}
 
 
@@ -312,102 +327,111 @@ def get_antialias_info():
 
 def get_current_desktop():
     v = -1
-    d = None
-    try:
-        d = _get_X11_root_property("_NET_CURRENT_DESKTOP", "CARDINAL")
-        if d:
-            v = struct.unpack("=I", d)[0]
-    except Exception as e:
-        log.warn("failed to get current desktop: %s", e)
-    log("get_current_desktop() %s=%s", hexstr(d or ""), v)
+    if not is_Wayland():
+        d = None
+        try:
+            d = _get_X11_root_property("_NET_CURRENT_DESKTOP", "CARDINAL")
+            if d:
+                v = struct.unpack(b"=I", d)[0]
+        except Exception as e:
+            log.warn("failed to get current desktop: %s", e)
+        log("get_current_desktop() %s=%s", hexstr(d or ""), v)
     return v
 
 def get_workarea():
-    try:
-        d = get_current_desktop()
-        if d<0:
-            return None
-        workarea = _get_X11_root_property("_NET_WORKAREA", "CARDINAL")
-        if not workarea:
-            return None
-        screenlog("get_workarea()=%s, len=%s", type(workarea), len(workarea))
-        #workarea comes as a list of 4 CARDINAL dimensions (x,y,w,h), one for each desktop
-        if len(workarea)<(d+1)*4*4:
-            screenlog.warn("get_workarea() invalid _NET_WORKAREA value")
-        else:
-            cur_workarea = workarea[d*4*4:(d+1)*4*4]
-            v = struct.unpack("=IIII", cur_workarea)
-            screenlog("get_workarea() %s=%s", hexstr(cur_workarea), v)
-            return v
-    except Exception as e:
-        screenlog.warn("failed to get workarea: %s", e)
+    if not is_Wayland():
+        try:
+            d = get_current_desktop()
+            if d<0:
+                return None
+            workarea = _get_X11_root_property("_NET_WORKAREA", "CARDINAL")
+            if not workarea:
+                return None
+            screenlog("get_workarea()=%s, len=%s", type(workarea), len(workarea))
+            #workarea comes as a list of 4 CARDINAL dimensions (x,y,w,h), one for each desktop
+            if len(workarea)<(d+1)*4*4:
+                screenlog.warn("get_workarea() invalid _NET_WORKAREA value")
+            else:
+                cur_workarea = workarea[d*4*4:(d+1)*4*4]
+                v = struct.unpack(b"=IIII", cur_workarea)
+                screenlog("get_workarea() %s=%s", hexstr(cur_workarea), v)
+                return v
+        except Exception as e:
+            screenlog.warn("failed to get workarea: %s", e)
     return None
 
 
 def get_number_of_desktops():
-    v = 1
-    d = None
-    try:
-        d = _get_X11_root_property("_NET_NUMBER_OF_DESKTOPS", "CARDINAL")
-        if d:
-            v = struct.unpack("=I", d)[0]
-    except Exception as e:
-        screenlog.warn("failed to get number of desktop: %s", e)
-    v = max(1, v)
-    screenlog("get_number_of_desktops() %s=%s", hexstr(d or ""), v)
+    v = 0
+    if not is_Wayland():
+        d = None
+        try:
+            d = _get_X11_root_property("_NET_NUMBER_OF_DESKTOPS", "CARDINAL")
+            if d:
+                v = struct.unpack(b"=I", d)[0]
+        except Exception as e:
+            screenlog.warn("failed to get number of desktop: %s", e)
+        v = max(1, v)
+        screenlog("get_number_of_desktops() %s=%s", hexstr(d or ""), v)
     return v
 
 def get_desktop_names():
-    v = ["Main"]
-    d = None
-    try:
-        d = _get_X11_root_property("_NET_DESKTOP_NAMES", "UTF8_STRING")
-        if d:
-            v = d.split(b"\0")
-            if len(v)>1 and v[-1]=="":
-                v = v[:-1]
-    except Exception as e:
-        screenlog.warn("failed to get desktop names: %s", e)
-    screenlog("get_desktop_names() %s=%s", hexstr(d or ""), v)
+    v = []
+    if not is_Wayland():
+        v = ["Main"]
+        d = None
+        try:
+            d = _get_X11_root_property("_NET_DESKTOP_NAMES", "UTF8_STRING")
+            if d:
+                v = d.split(b"\0")
+                if len(v)>1 and v[-1]=="":
+                    v = v[:-1]
+        except Exception as e:
+            screenlog.warn("failed to get desktop names: %s", e)
+        screenlog("get_desktop_names() %s=%s", hexstr(d or ""), v)
     return v
 
 
 def get_vrefresh():
-    try:
-        from xpra.x11.bindings.randr_bindings import RandRBindings      #@UnresolvedImport
-        randr = RandRBindings()
-        v = randr.get_vrefresh()
-    except Exception as e:
-        screenlog.warn("failed to get VREFRESH: %s", e)
-        v = -1
-    screenlog("get_vrefresh()=%s", v)
+    v = -1
+    if not is_Wayland():
+        try:
+            from xpra.x11.bindings.randr_bindings import RandRBindings      #@UnresolvedImport
+            randr = RandRBindings()
+            v = randr.get_vrefresh()
+        except Exception as e:
+            log("get_vrefresh()", exc_info=True)
+            log.warn("Warning: failed to query the display vertical refresh rate:")
+            log.warn(" %s", e)
+        screenlog("get_vrefresh()=%s", v)
     return v
 
 
 def _get_xresources():
-    try:
-        from xpra.x11.gtk_x11.prop import prop_get
-        import gtk.gdk
-        root = gtk.gdk.get_default_root_window()
-        v = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
-        log("RESOURCE_MANAGER=%s", v)
-        if v is None:
-            return None
-        value = v.decode("utf-8")
-        #parse the resources into a dict:
-        values={}
-        options = value.split("\n")
-        for option in options:
-            if not option:
-                continue
-            parts = option.split(":\t", 1)
-            if len(parts)!=2:
-                log("skipped invalid option: '%s'", option)
-                continue
-            values[parts[0]] = parts[1]
-        return values
-    except Exception as e:
-        log("_get_xresources error: %s", e)
+    if not is_Wayland():
+        try:
+            from xpra.x11.gtk_x11.prop import prop_get
+            from xpra.gtk_common.gtk_util import get_default_root_window
+            root = get_default_root_window()
+            v = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
+            log("RESOURCE_MANAGER=%s", v)
+            if v is None:
+                return None
+            value = v.decode("utf-8")
+            #parse the resources into a dict:
+            values={}
+            options = value.split("\n")
+            for option in options:
+                if not option:
+                    continue
+                parts = option.split(":\t", 1)
+                if len(parts)!=2:
+                    log("skipped invalid option: '%s'", option)
+                    continue
+                values[parts[0]] = parts[1]
+            return values
+        except Exception as e:
+            log("_get_xresources error: %s", e)
     return None
 
 def get_cursor_size():
@@ -442,19 +466,21 @@ def get_window_frame_sizes():
     return {}
 
 
-def system_bell(window, device, percent, pitch, duration, bell_class, bell_id, bell_name):
-    from xpra.gtk_common.error import XError
+def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id, bell_name):
+    if not is_X11():
+        return False
     global device_bell
     if device_bell is False:
         #failed already
         return False
+    from xpra.gtk_common.error import XError
     def x11_bell():
         global device_bell
         if device_bell is None:
             #try to load it:
             from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings       #@UnresolvedImport
             device_bell = X11KeyboardBindings().device_bell
-        device_bell(get_xid(window), device, bell_class, bell_id, percent, bell_name)
+        device_bell(get_xwindow(window), device, bell_class, bell_id, percent, bell_name)
     try:
         from xpra.gtk_common.error import xsync
         with xsync:
@@ -474,7 +500,7 @@ def _send_client_message(window, message_type, *values):
         X11Window = X11WindowBindings()
         root_xid = X11Window.getDefaultRootWindow()
         if window:
-            xid = get_xid(window)
+            xid = get_xwindow(window)
         else:
             xid = root_xid
         SubstructureNotifyMask = constants["SubstructureNotifyMask"]
@@ -547,8 +573,9 @@ class XI2_Window(object):
         self.XI2 = X11XI2Bindings()
         self.X11Window = X11WindowBindings()
         self.window = window
-        self.xid = window.get_window().xid
+        self.xid = get_xwindow(window.get_window())
         self.windows = ()
+        self.motion_valuators = {}
         window.connect("configure-event", self.configured)
         self.configured()
         #replace event handlers with XI2 version:
@@ -562,18 +589,29 @@ class XI2_Window(object):
     def noop(self, *args):
         pass
 
-    def cleanup(self, *args):
+    def cleanup(self, *_args):
         for window in self.windows:
             self.XI2.disconnect(window)
         self.windows = []
         self.window = None
 
-    def configured(self, *args):
-        self.windows = self.get_parent_windows(self.xid)
+    def configured(self, *_args):
+        from xpra.gtk_common.error import xsync
+        with xsync:
+            self.windows = self.get_parent_windows(self.xid)
         for window in self.windows:
             self.XI2.connect(window, "XI_Motion", self.do_xi_motion)
             self.XI2.connect(window, "XI_ButtonPress", self.do_xi_button)
             self.XI2.connect(window, "XI_ButtonRelease", self.do_xi_button)
+            self.XI2.connect(window, "XI_DeviceChanged", self.do_xi_device_changed)
+            self.XI2.connect(window, "XI_HierarchyChanged", self.do_xi_hierarchy_changed)
+
+    def do_xi_device_changed(self, *_args):
+        self.motion_valuators = {}
+
+    def do_xi_hierarchy_changed(self, *_args):
+        self.motion_valuators = {}
+
 
     def get_parent_windows(self, oxid):
         windows = [oxid]
@@ -584,17 +622,18 @@ class XI2_Window(object):
             if xid==0 or xid==root:
                 break
             windows.append(xid)
-        log("get_parent_windows(%#x)=%s", oxid, csv(hex(x) for x in windows))
+        xinputlog("get_parent_windows(%#x)=%s", oxid, csv(hex(x) for x in windows))
         return windows
 
 
-    def do_xi_button(self, event):
+    def do_xi_button(self, event, device):
         window = self.window
         client = window._client
         if client.readonly:
             return
-        if client.server_input_devices=="xi":
-            #skip synthetic scroll events for two-finger scroll,
+        xinputlog("do_xi_button(%s, %s) server_input_devices=%s", event, device, client.server_input_devices)
+        if client.server_input_devices=="xi" or (client.server_input_devices=="uinput" and client.server_precise_wheel):
+            #skip synthetic scroll events,
             #as the server should synthesize them from the motion events
             #those have the same serial:
             matching_motion = self.XI2.find_event("XI_Motion", event.serial)
@@ -606,26 +645,78 @@ class XI2_Window(object):
         args = self.get_pointer_extra_args(event)
         window._button_action(button, event, depressed, *args)
 
-    def do_xi_motion(self, event):
+    def do_xi_motion(self, event, device):
         window = self.window
         if window.moveresize_event:
+            xinputlog("do_xi_motion(%s, %s) handling as a moveresize event on window %s", event, device, window)
             window.motion_moveresize(event)
             self.do_motion_notify_event(event)
             return
-        if window._client.readonly:
+        client = window._client
+        if client.readonly:
             return
-        #find the motion events in the xi2 event list:
-        pointer, modifiers, buttons = window._pointer_modifiers(event)
+        pointer, relative_pointer, modifiers, buttons = window._pointer_modifiers(event)
         wid = self.window.get_mouse_event_wid(*pointer)
-        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, wid, window._client._focused, self.window._id, event.device, pointer, modifiers, buttons)
-        packet = ["pointer-position", wid, pointer, modifiers, buttons] + self.get_pointer_extra_args(event)
-        window._client.send_mouse_position(packet)
+        #log("server_input_devices=%s, server_precise_wheel=%s", client.server_input_devices, client.server_precise_wheel)
+        valuators = event.valuators
+        unused_valuators = valuators.copy()
+        dx, dy = 0, 0
+        if (valuators and device and device.get("enabled") and 
+            client.server_input_devices=="uinput" and client.server_precise_wheel):
+            XIModeRelative = 0
+            classes = device.get("classes")
+            val_classes = {}
+            for c in classes.values():
+                number = c.get("number")
+                if number is not None and c.get("type")=="valuator" and c.get("mode")==XIModeRelative:
+                    val_classes[number] = c
+            #previous values:
+            mv = self.motion_valuators.setdefault(event.device, {})
+            last_x, last_y = 0, 0
+            wheel_x, wheel_y = 0, 0
+            unused_valuators = {}
+            for number, value in valuators.items():
+                valuator = val_classes.get(number)
+                if valuator:
+                    label = valuator.get("label")
+                    if label:
+                        mouselog("%s: %s", label, value)
+                        if label.lower().find("horiz")>=0:
+                            wheel_x = value
+                            last_x = mv.get(number)
+                            continue
+                        elif label.lower().find("vert")>=0:
+                            wheel_y = value
+                            last_y = mv.get(number)
+                            continue
+                unused_valuators[number] = value
+            #new absolute motion values:
+            #calculate delta if we have both old and new values:
+            if last_x is not None and wheel_x is not None:
+                dx = last_x-wheel_x
+            if last_y is not None and wheel_y is not None:
+                dy = last_y-wheel_y
+            #whatever happens, update our motion cached values:
+            mv.update(event.valuators)
+        #send plain motion first, if any:
+        if unused_valuators:
+            xinputlog("do_xi_motion(%s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, device, wid, window._client._focused, window._id, event.device, pointer, modifiers, buttons)
+            pdata = pointer
+            if client.server_pointer_relative:
+                pdata = list(pointer)+list(relative_pointer)
+            packet = ["pointer-position", wid, pdata, modifiers, buttons] + self.get_pointer_extra_args(event)
+            client.send_mouse_position(packet)
+        #now see if we have anything to send as a wheel event:
+        if dx!=0 or dy!=0:
+            xinputlog("do_xi_motion(%s, %s) wheel deltas: dx=%i, dy=%i", event, device, dx, dy)
+            #normalize (xinput is always using 15 degrees?)
+            client.wheel_event(wid, float(dx)/XINPUT_WHEEL_DIV, float(dy)/XINPUT_WHEEL_DIV, event.device)
 
     def get_pointer_extra_args(self, event):
         def intscaled(f):
             return int(f*1000000), 1000000
-        def dictscaled(v):
-            return dict((k,intscaled(v)) for k,v in v.items())
+        def dictscaled(d):
+            return dict((k,intscaled(v)) for k,v in d.items())
         raw_valuators = {}
         raw_event_name = event.name.replace("XI_", "XI_Raw")    #ie: XI_Motion -> XI_RawMotion
         raw = self.XI2.find_event(raw_event_name, event.serial)
@@ -641,18 +732,22 @@ class XI2_Window(object):
 
 
 class ClientExtras(object):
-    def __init__(self, client, opts):
+    def __init__(self, client, _opts):
         self.client = client
         self._xsettings_watcher = None
         self._root_props_watcher = None
         self.system_bus = None
+        self.session_bus = None
         self.upower_resuming_match = None
         self.upower_sleeping_match = None
         self.login1_match = None
+        self.screensaver_match = None
         self.x11_filter = None
         if client.xsettings_enabled:
             self.setup_xprops()
-        if client.input_devices=="xi":
+        self.xi_setup_failures = 0
+        input_devices = getattr(client, "input_devices", None)
+        if input_devices in ("xi", "auto"):
             #this would trigger warnings with our temporary opengl windows:
             #only enable it after we have connected:
             self.client.after_handshake(self.setup_xi)
@@ -665,16 +760,24 @@ class ClientExtras(object):
         if self.x11_filter:
             return
         try:
-            from xpra.x11.gtk2.gdk_bindings import init_x11_filter  #@UnresolvedImport
+            if is_gtk3():
+                from xpra.x11.gtk3.gdk_bindings import init_x11_filter  #@UnresolvedImport, @UnusedImport
+            else:
+                from xpra.x11.gtk2.gdk_bindings import init_x11_filter  #@UnresolvedImport, @Reimport
             self.x11_filter = init_x11_filter()
             log("x11_filter=%s", self.x11_filter)
-        except:
+        except Exception:
+            log.error("Error: failed to initialize X11 GDK filter:")
+            log.error(" %s", e)
             self.x11_filter = None
 
     def cleanup(self):
         log("cleanup() xsettings_watcher=%s, root_props_watcher=%s", self._xsettings_watcher, self._root_props_watcher)
         if self.x11_filter:
-            from xpra.x11.gtk2.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport
+            if is_gtk3():
+                from xpra.x11.gtk3.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport, @UnusedImport
+            else:
+                from xpra.x11.gtk2.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport, @Reimport
             self.x11_filter = None
             cleanup_x11_filter()
         if self._xsettings_watcher:
@@ -689,10 +792,17 @@ class ClientExtras(object):
             self.system_bus = None
             if self.upower_resuming_match:
                 bus._clean_up_signal_match(self.upower_resuming_match)
+                self.upower_resuming_match = None
             if self.upower_sleeping_match:
                 bus._clean_up_signal_match(self.upower_sleeping_match)
+                self.upower_sleeping_match = None
             if self.login1_match:
                 bus._clean_up_signal_match(self.login1_match)
+                self.login1_match = None
+        if self.session_bus:
+            if self.screensaver_match:
+                self.session_bus._clean_up_signal_match(self.screensaver_match)
+                self.screensaver_match = None
         global WINDOW_METHOD_OVERRIDES
         WINDOW_METHOD_OVERRIDES = {}
 
@@ -715,7 +825,15 @@ class ClientExtras(object):
             dbuslog.info(" no support for power events")
             return
         try:
-            from xpra.dbus.common import init_system_bus
+            from xpra.dbus.common import init_system_bus, init_session_bus
+        except ImportError as e:
+            dbuslog("setup_dbus_signals()", exc_info=True)
+            dbuslog.error("Error: dbus bindings are missing,")
+            dbuslog.error(" cannot setup event listeners:")
+            dbuslog.error(" %s", e)
+            return
+
+        try:
             bus = init_system_bus()
             self.system_bus = bus
             dbuslog("setup_dbus_signals() system bus=%s", bus)
@@ -723,58 +841,88 @@ class ClientExtras(object):
             dbuslog("setup_dbus_signals()", exc_info=True)
             dbuslog.error("Error setting up dbus signals:")
             dbuslog.error(" %s", e)
-            return
+        else:
+            #the UPower signals:
+            try:
+                bus_name    = 'org.freedesktop.UPower'
+                dbuslog("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
+                iface_name  = 'org.freedesktop.UPower'
+                self.upower_resuming_match = bus.add_signal_receiver(self.resuming_callback, 'Resuming', iface_name, bus_name)
+                self.upower_sleeping_match = bus.add_signal_receiver(self.sleeping_callback, 'Sleeping', iface_name, bus_name)
+                dbuslog("listening for 'Resuming' and 'Sleeping' signals on %s", iface_name)
+            except Exception as e:
+                dbuslog("failed to setup UPower event listener: %s", e)
 
-        #the UPower signals:
-        try:
-            bus_name    = 'org.freedesktop.UPower'
-            dbuslog("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
-            iface_name  = 'org.freedesktop.UPower'
-            self.upower_resuming_match = bus.add_signal_receiver(self.resuming_callback, 'Resuming', iface_name, bus_name)
-            self.upower_sleeping_match = bus.add_signal_receiver(self.sleeping_callback, 'Sleeping', iface_name, bus_name)
-            dbuslog("listening for 'Resuming' and 'Sleeping' signals on %s", iface_name)
-        except Exception as e:
-            dbuslog("failed to setup UPower event listener: %s", e)
+            #the "logind" signals:
+            try:
+                bus_name    = 'org.freedesktop.login1'
+                dbuslog("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
+                def sleep_event_handler(suspend):
+                    if suspend:
+                        self.sleeping_callback()
+                    else:
+                        self.resuming_callback()
+                iface_name  = 'org.freedesktop.login1.Manager'
+                self.login1_match = bus.add_signal_receiver(sleep_event_handler, 'PrepareForSleep', iface_name, bus_name)
+                dbuslog("listening for 'PrepareForSleep' signal on %s", iface_name)
+            except Exception as e:
+                dbuslog("failed to setup login1 event listener: %s", e)
+
+        if DBUS_SCREENSAVER:
+            try:
+                session_bus = init_session_bus()
+                self.session_bus = session_bus
+                dbuslog("setup_dbus_signals() session bus=%s", session_bus)
+            except Exception as e:
+                dbuslog("setup_dbus_signals()", exc_info=True)
+                dbuslog.error("Error setting up dbus signals:")
+                dbuslog.error(" %s", e)
+            else:
+                #screensaver signals:
+                try:
+                    bus_name = "org.gnome.ScreenSaver"
+                    iface_name = bus_name
+                    self.screensaver_match = bus.add_signal_receiver(self.ActiveChanged, "ActiveChanged", iface_name, bus_name)
+                    dbuslog("listening for 'ActiveChanged' signal on %s", iface_name)
+                except Exception as e:
+                    dbuslog.warn("Warning: failed to setup screensaver event listener: %s", e)
+
+    def ActiveChanged(self, active):
+        log("ActiveChanged(%s)", active)
+        if active:
+            self.client.suspend()
+        else:
+            self.client.resume()
 
-        #the "logind" signals:
-        try:
-            bus_name    = 'org.freedesktop.login1'
-            dbuslog("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
-            def sleep_event_handler(suspend):
-                if suspend:
-                    self.sleeping_callback()
-                else:
-                    self.resuming_callback()
-            iface_name  = 'org.freedesktop.login1.Manager'
-            self.login1_match = bus.add_signal_receiver(sleep_event_handler, 'PrepareForSleep', iface_name, bus_name)
-            dbuslog("listening for 'PrepareForSleep' signal on %s", iface_name)
-        except Exception as e:
-            dbuslog("failed to setup login1 event listener: %s", e)
 
     def setup_xprops(self):
         #wait for handshake to complete:
-        self.client.after_handshake(self.do_setup_xprops)
+        if not is_Wayland():
+            self.client.after_handshake(self.do_setup_xprops)
 
     def do_setup_xprops(self, *args):
         log("do_setup_xprops(%s)", args)
-        if is_gtk3():
-            log("x11 root properties and XSETTINGS are not supported yet with GTK3")
-            return
         ROOT_PROPS = ["RESOURCE_MANAGER", "_NET_WORKAREA", "_NET_CURRENT_DESKTOP"]
         try:
             self.init_x11_filter()
+            if is_gtk3():
+                #xsettings still need porting
+                return
+            from xpra.gtk_common.gtk_util import get_default_root_window
             from xpra.x11.xsettings import XSettingsWatcher
             from xpra.x11.xroot_props import XRootPropWatcher
+            root = get_default_root_window()
             if self._xsettings_watcher is None:
                 self._xsettings_watcher = XSettingsWatcher()
                 self._xsettings_watcher.connect("xsettings-changed", self._handle_xsettings_changed)
                 self._handle_xsettings_changed()
             if self._root_props_watcher is None:
-                self._root_props_watcher = XRootPropWatcher(ROOT_PROPS)
+                self._root_props_watcher = XRootPropWatcher(ROOT_PROPS, root)
                 self._root_props_watcher.connect("root-prop-changed", self._handle_root_prop_changed)
                 #ensure we get the initial value:
                 self._root_props_watcher.do_notify("RESOURCE_MANAGER")
         except ImportError as e:
+            log("do_setup_xprops%s", args, exc_info=True)
             log.error("failed to load X11 properties/settings bindings: %s - root window properties will not be propagated", e)
 
 
@@ -786,29 +934,44 @@ class ClientExtras(object):
             self.client.send_input_devices("xi", devices)
 
     def setup_xi(self):
-        if self.client.server_input_devices!="xi":
-            log.info("server does not support xi input devices")
+        self.client.timeout_add(100, self.do_setup_xi)
+
+    def do_setup_xi(self):
+        if self.client.server_input_devices not in ("xi", "uinput"):
+            xinputlog.info("server does not support xi input devices")
+            if self.client.server_input_devices:
+                log.info(" server uses: %s", self.client.server_input_devices)
+            return False
         try:
-            from xpra.gtk_common.error import xsync
+            from xpra.gtk_common.error import xsync, XError
+            assert X11WindowBindings, "no X11 window bindings"
+            assert X11XI2Bindings, "no XI2 window bindings"
+            XI2 = X11XI2Bindings()
+            #this may fail when windows are being destroyed,
+            #ie: when another client disconnects because we are stealing the session
+            try:
+                with xsync:
+                    XI2.select_xi2_events()
+            except XError:
+                self.xi_setup_failures += 1
+                xinputlog("select_xi2_events() failed, attempt %i", self.xi_setup_failures,exc_info=True)
+                return self.xi_setup_failures<10    #try again
             with xsync:
-                assert X11WindowBindings, "no X11 window bindings"
-                assert X11XI2Bindings, "no XI2 window bindings"
-                X11XI2Bindings().gdk_inject()
+                XI2.gdk_inject()
                 self.init_x11_filter()
-                XI2 = X11XI2Bindings()
-                XI2.select_xi2_events()
                 if self.client.server_input_devices:
                     XI2.connect(0, "XI_HierarchyChanged", self.do_xi_devices_changed)
                     devices = XI2.get_devices()
                     if devices:
                         self.client.send_input_devices("xi", devices)
         except Exception as e:
-            log("enable_xi2()", exc_info=True)
-            log.error("Error: cannot enable XI2 events")
-            log.error(" %s", e)
+            xinputlog("enable_xi2()", exc_info=True)
+            xinputlog.error("Error: cannot enable XI2 events")
+            xinputlog.error(" %s", e)
         else:
             #register our enhanced event handlers:
             self.add_xi2_method_overrides()
+        return False
 
     def add_xi2_method_overrides(self):
         global WINDOW_ADD_HOOKS
@@ -822,7 +985,7 @@ class ClientExtras(object):
             log.error("failed to get XSETTINGS", exc_info=True)
         return None
 
-    def _handle_xsettings_changed(self, *args):
+    def _handle_xsettings_changed(self, *_args):
         settings = self._get_xsettings()
         log("xsettings_changed new value=%s", settings)
         if settings is not None:
@@ -830,9 +993,9 @@ class ClientExtras(object):
 
     def get_resource_manager(self):
         try:
-            import gtk.gdk
-            root = gtk.gdk.get_default_root_window()
+            from xpra.gtk_common.gtk_util import get_default_root_window
             from xpra.x11.gtk_x11.prop import prop_get
+            root = get_default_root_window()
             value = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
             if value is not None:
                 return value.encode("utf-8")
