@@ -9,6 +9,7 @@ from xpra.platform.keyboard_base import KeyboardBase
 from xpra.keyboard.mask import MODIFIER_MAP
 from xpra.keyboard.layouts import xkbmap_query_tostring
 from xpra.log import Logger
+from xpra.os_util import is_X11
 log = Logger("keyboard", "posix")
 
 
@@ -22,7 +23,7 @@ class Keyboard(KeyboardBase):
             self.keyboard_bindings = X11KeyboardBindings()
         except Exception as e:
             log.error("Error: failed to load posix keyboard bindings")
-            log.error(" %s", e)
+            log.error(" %s", str(e) or type(e))
             self.keyboard_bindings = None
 
 
@@ -85,6 +86,8 @@ class Keyboard(KeyboardBase):
         #parses the "_XKB_RULES_NAMES" X11 property
         #FIXME: a bit ugly to call gtk here...
         #but otherwise we have to call XGetWindowProperty and deal with X11 errors..
+        if not is_X11():
+            return ""
         xkb_rules_names = ""
         from xpra.platform.xposix.gui import _get_X11_root_property
         prop = _get_X11_root_property("_XKB_RULES_NAMES", "STRING")
