@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,7 +11,9 @@ import os
 #before we import xpra.platform
 import platform as python_platform
 assert python_platform
-from xpra.os_util import WIN32, OSX, is_CentOS, is_RedHat, is_Fedora, get_linux_distribution
+from xpra.util import csv
+from xpra.os_util import (WIN32, OSX, PYTHON2, osexpand, getuid, getgid, get_username_for_uid,
+    is_CentOS, is_RedHat, is_Fedora, is_Debian, is_Ubuntu, getUbuntuVersion)
 
 def warn(msg):
     sys.stderr.write(msg+"\n")
@@ -31,6 +33,8 @@ class InitExit(Exception):
         Exception.__init__(self, msg)
 
 
+DEBUG_CONFIG_PROPERTIES = os.environ.get("XPRA_DEBUG_CONFIG_PROPERTIES", "").split()
+
 DEFAULT_XPRA_CONF_FILENAME = os.environ.get("XPRA_CONF_FILENAME", 'xpra.conf')
 DEFAULT_NET_WM_NAME = os.environ.get("XPRA_NET_WM_NAME", "Xpra")
 
@@ -54,6 +58,9 @@ def has_sound_support():
 
 def get_xorg_bin():
     # Detect Xorg Binary
+    if os.uname()[4].startswith("arm") and is_Debian() and os.path.exists("/usr/bin/Xorg"):
+        #Raspbian breaks if we use a different binary..
+        return "/usr/bin/Xorg"
     for p in (
               "/usr/libexec/Xorg",              #fedora 22+
               "/usr/lib/xorg/Xorg",             #ubuntu 16.10
@@ -70,6 +77,15 @@ def get_xorg_bin():
     return None
 
 
+def get_Xdummy_confdir():
+    from xpra.platform.xposix.paths import get_runtime_dir
+    xrd = get_runtime_dir()
+    if xrd:
+        base = "${XDG_RUNTIME_DIR}/xpra"
+    else:
+        base = "${HOME}/.xpra"
+    return base+"/xorg.conf.d/$PID"
+
 def get_Xdummy_command(xorg_cmd="Xorg", log_dir="${XPRA_LOG_DIR}", xorg_conf="/etc/xpra/xorg.conf"):
     cmd = [xorg_cmd]    #ie: ["Xorg"] or ["xpra_Xdummy"] or ["./install/bin/xpra_Xdummy"]
     cmd += [
@@ -82,13 +98,14 @@ def get_Xdummy_command(xorg_cmd="Xorg", log_dir="${XPRA_LOG_DIR}", xorg_conf="/e
           "-logfile", "%s/Xorg.${DISPLAY}.log" % log_dir,
           #must be specified with some Xorg versions (ie: arch linux)
           #this directory can store xorg config files, it does not need to be created:
-          "-configdir", "${HOME}/.xpra/xorg.conf.d",
+          "-configdir", get_Xdummy_confdir(),
           "-config", xorg_conf
           ]
     return cmd
 
 def get_Xvfb_command():
     cmd = ["Xvfb",
+           "+extension", "GLX",
            "+extension", "Composite",
            "-screen", "0", "5760x2560x24+32",
            #better than leaving to vfb after a resize?
@@ -147,7 +164,6 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
     else:
         debug("Xdummy support unspecified, will try to detect")
 
-    from xpra.os_util import is_Ubuntu, getUbuntuVersion
     if is_Ubuntu():
         rnum = getUbuntuVersion()
         if rnum==[16, 10]:
@@ -158,12 +174,6 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
 
 
 def OpenGL_safety_check():
-    #Ubuntu 12.04 will just crash on you if you try:
-    from xpra.os_util import is_Ubuntu, getUbuntuVersion
-    if is_Ubuntu():
-        rnum = getUbuntuVersion()
-        if rnum<=[12, 4]:
-            return "Ubuntu %s is too buggy" % rnum
     #try to detect VirtualBox:
     #based on the code found here:
     #http://spth.virii.lu/eof2/articles/WarGame/vboxdetect.html
@@ -188,11 +198,11 @@ def OpenGL_safety_check():
             return "VirtualBox is present (VBoxMiniRdrDN)"
     return None
 
-OPENGL_DEFAULT = None       #will auto-detect by probing
+OPENGL_DEFAULT = "auto"       #will auto-detect by probing
 def get_opengl_default():
     global OPENGL_DEFAULT
     if OpenGL_safety_check() is not None:
-        OPENGL_DEFAULT = False
+        OPENGL_DEFAULT = "no"
     return OPENGL_DEFAULT
 
 
@@ -255,7 +265,11 @@ def read_config(conf_file):
     if not os.path.isfile(conf_file):
         debug("read_config(%s) is not a file!", conf_file)
         return d
-    with open(conf_file, "rU") as f:
+    if PYTHON2:
+        mode = "rU"
+    else:
+        mode = "r"
+    with open(conf_file, mode) as f:
         lines = []
         no = 0
         for line in f:
@@ -303,6 +317,8 @@ def read_config(conf_file):
         else:
             debug("assigned (new): %s='%s'", name, value)
             d[name] = value
+        if name in DEBUG_CONFIG_PROPERTIES:
+            print("%s=%s (was %s), from %s" % (name, d[name], current_value, conf_file))
     return  d
 
 
@@ -345,7 +361,7 @@ def read_xpra_conf(conf_dir, xpra_conf_filename=DEFAULT_XPRA_CONF_FILENAME):
         d.update(cd)
     return d
 
-def read_xpra_defaults():
+def read_xpra_defaults(username=None, uid=None, gid=None):
     """
         Reads the global <xpra_conf_filename> from the <conf_dir>
         and then the user-specific one.
@@ -365,12 +381,12 @@ def read_xpra_defaults():
     # * user config            (ie: "~/.xpra/" on all Posix, including OSX)
     #                          (ie: "C:\Documents and Settings\Username\Application Data\Xpra" with XP)
     #                          (ie: "C:\Users\<user name>\AppData\Roaming" with Visa onwards)
-    dirs = get_default_conf_dirs() + get_system_conf_dirs() + get_user_conf_dirs()
+    dirs = get_default_conf_dirs() + get_system_conf_dirs() + get_user_conf_dirs(uid)
     defaults = {}
     for d in dirs:
         if not d:
             continue
-        ad = os.path.expanduser(d)
+        ad = osexpand(d, actual_username=username, uid=uid, gid=gid)
         if not os.path.exists(ad):
             debug("read_xpra_defaults: skipping %s", ad)
             continue
@@ -422,19 +438,15 @@ OPTIONS_VALIDATION = {}
 OPTION_TYPES = {
                     #string options:
                     "encoding"          : str,
+                    "opengl"            : str,
                     "title"             : str,
                     "username"          : str,
                     "password"          : str,
-                    "auth"              : str,
-                    "vsock-auth"        : str,
-                    "tcp-auth"          : str,
-                    "ssl-auth"          : str,
                     "wm-name"           : str,
                     "session-name"      : str,
                     "dock-icon"         : str,
                     "tray-icon"         : str,
                     "window-icon"       : str,
-                    "password-file"     : str,
                     "keyboard-raw"      : bool,
                     "keyboard-layout"   : str,
                     "keyboard-layouts"  : list,
@@ -447,6 +459,7 @@ OPTION_TYPES = {
                     "remote-clipboard"  : str,
                     "local-clipboard"   : str,
                     "pulseaudio-command": str,
+                    "bandwidth-limit"   : str,
                     "tcp-encryption"    : str,
                     "tcp-encryption-keyfile": str,
                     "encryption"        : str,
@@ -479,6 +492,7 @@ OPTION_TYPES = {
                     "postscript-printer": str,
                     "debug"             : str,
                     "input-method"      : str,
+                    "video-scaling"     : str,
                     "microphone"        : str,
                     "speaker"           : str,
                     "sound-source"      : str,
@@ -489,6 +503,12 @@ OPTION_TYPES = {
                     "webcam"            : str,
                     "mousewheel"        : str,
                     "input-devices"     : str,
+                    "shortcut-modifiers": str,
+                    "open-files"        : str,
+                    "open-url"          : str,
+                    "file-transfer"     : str,
+                    "printing"          : str,
+                    "challenge-handlers": str,
                     #ssl options:
                     "ssl"               : str,
                     "ssl-key"           : str,
@@ -504,6 +524,7 @@ OPTION_TYPES = {
                     "ssl-server-hostname" : str,
                     "ssl-options"       : str,
                     #int options:
+                    "displayfd"         : int,
                     "pings"             : int,
                     "quality"           : int,
                     "min-quality"       : int,
@@ -511,7 +532,6 @@ OPTION_TYPES = {
                     "min-speed"         : int,
                     "compression_level" : int,
                     "dpi"               : int,
-                    "video-scaling"     : int,
                     "file-size-limit"   : int,
                     "idle-timeout"      : int,
                     "server-idle-timeout" : int,
@@ -520,6 +540,7 @@ OPTION_TYPES = {
                     "uid"               : int,
                     "gid"               : int,
                     "min-port"          : int,
+                    "rfb-upgrade"       : int,
                     #float options:
                     "auto-refresh-delay": float,
                     #boolean options:
@@ -541,6 +562,7 @@ OPTION_TYPES = {
                     "xsettings"         : bool,
                     "system-tray"       : bool,
                     "sharing"           : bool,
+                    "lock"              : bool,
                     "delay-tray"        : bool,
                     "windows"           : bool,
                     "terminate-children": bool,
@@ -549,16 +571,16 @@ OPTION_TYPES = {
                     "exit-ssh"          : bool,
                     "dbus-control"      : bool,
                     "av-sync"           : bool,
-                    "opengl"            : bool,
                     "mdns"              : bool,
-                    "file-transfer"     : bool,
-                    "printing"          : bool,
-                    "open-files"        : bool,
                     "swap-keys"         : bool,
                     "start-new-commands": bool,
                     "proxy-start-sessions": bool,
                     "desktop-fullscreen": bool,
                     "global-menus"      : bool,
+                    "forward-xdg-open"  : bool,
+                    "modal-windows"     : bool,
+                    "bandwidth-detection" : bool,
+                    "ssh-upgrade"       : bool,
                     #arrays of strings:
                     "pulseaudio-configure-commands" : list,
                     "socket-dirs"       : list,
@@ -582,7 +604,22 @@ OPTION_TYPES = {
                     "bind"              : list,
                     "bind-vsock"        : list,
                     "bind-tcp"          : list,
+                    "bind-udp"          : list,
+                    "bind-ws"           : list,
+                    "bind-wss"          : list,
                     "bind-ssl"          : list,
+                    "bind-ssh"          : list,
+                    "bind-rfb"          : list,
+                    "auth"              : list,
+                    "vsock-auth"        : list,
+                    "tcp-auth"          : list,
+                    "udp-auth"          : list,
+                    "ws-auth"           : list,
+                    "wss-auth"          : list,
+                    "ssl-auth"          : list,
+                    "ssh-auth"          : list,
+                    "rfb-auth"          : list,
+                    "password-file"     : list,
                     "start-env"         : list,
                     "env"               : list,
                }
@@ -600,11 +637,17 @@ START_COMMAND_OPTIONS = [
     "start-after-connect", "start-child-after-connect",
     "start-on-connect", "start-child-on-connect",
     ]
-BIND_OPTIONS = ["bind", "bind-tcp", "bind-ssl", "bind-vsock"]
+BIND_OPTIONS = ["bind", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-vsock", "bind-rfb"]
 
 #keep track of the options added since v1,
 #so we can generate command lines that work with older supported versions:
-OPTIONS_ADDED_SINCE_V1 = ["attach", "open-files", "pixel-depth", "uid", "gid", "chdir", "min-port"]
+OPTIONS_ADDED_SINCE_V1 = ["attach", "open-files", "open-url", "pixel-depth", "uid", "gid", "chdir", "min-port", "rfb-upgrade", "bandwidth-limit",
+                    "forward-xdg-open", "modal-windows", "bandwidth-detection",
+                    "bind-ssh", "ssh-auth", "ssh-upgrade",
+                          ]
+OPTIONS_COMPAT_NAMES = {
+    "--compression_level=" : "-z"
+    }
 
 CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "dock-icon", "tray-icon", "window-icon",
@@ -614,7 +657,8 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "systemd-run", "systemd-run-args",
                   "socket-dir", "socket-dirs",
                   "border", "window-close", "max-size", "desktop-scaling",
-                  "file-transfer", "file-size-limit", "download-path", "open-command", "open-files", "printing",
+                  "file-transfer", "file-size-limit", "download-path", "open-command", "open-files", "printing", "open-url",
+                  "challenge-handlers",
                   "dbus-proxy",
                   "remote-logging",
                   "lpadmin", "lpinfo",
@@ -625,9 +669,9 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "quality", "min-quality", "speed", "min-speed",
                   "compression_level",
                   "dpi", "video-scaling", "auto-refresh-delay",
-                  "webcam", "mousewheel", "input-devices", "pings",
+                  "webcam", "mousewheel", "input-devices", "shortcut-modifiers", "pings",
                   "tray", "keyboard-sync", "cursors", "bell", "notifications",
-                  "xsettings", "system-tray", "sharing",
+                  "xsettings", "system-tray", "sharing", "lock",
                   "delay-tray", "windows", "readonly",
                   "av-sync", "swap-keys",
                   "opengl",
@@ -638,7 +682,8 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "key-shortcut",
                   "env"]
 
-CLIENT_ONLY_OPTIONS = ["username"]
+CLIENT_ONLY_OPTIONS = ["username", "swap-keys", "dock-icon",
+                       "tray", "delay-tray", "tray-icon"]
 
 #options that clients can pass to the proxy
 #and which will be forwarded to the new proxy instance process:
@@ -658,13 +703,16 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "idle-timeout", "server-idle-timeout",
     "use-display", "fake-xinerama", "resize_display", "dpi", "pixel-depth",
     "readonly", "keyboard-sync", "cursors", "bell", "notifications", "xsettings",
-    "system-tray", "sharing", "windows", "webcam", "html",
+    "system-tray", "sharing", "lock", "windows", "webcam", "html",
     "terminate-children", "exit-with-children", "exit-with-client",
     "av-sync", "global-menus",
-    "printing", "file-transfer", "open-command", "open-files", "start-new-commands",
+    "forward-xdg-open", "modal-windows", "bandwidth-detection",
+    "ssh-upgrade",
+    "printing", "file-transfer", "open-command", "open-files", "open-url", "start-new-commands",
     "mmap", "mmap-group", "mdns",
-    "auth", "vsock-auth", "tcp-auth", "ssl-auth",
-    "bind", "bind-vsock", "bind-tcp", "bind-ssl",
+    "auth", "vsock-auth", "tcp-auth", "udp-auth", "ws-auth", "wss-auth", "ssl-auth", "ssh-auth", "rfb-auth",
+    "bind", "bind-vsock", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-ssh", "bind-rfb",
+    "rfb-upgrade", "bandwidth-limit",
     "start", "start-child",
     "start-after-connect", "start-child-after-connect",
     "start-on-connect", "start-child-on-connect",
@@ -679,29 +727,30 @@ def get_default_key_shortcuts():
     return [shortcut for e,shortcut in [
                (True,   "Control+Menu:toggle_keyboard_grab"),
                (True,   "Shift+Menu:toggle_pointer_grab"),
-               (True,   "Shift+F11:toggle_fullscreen"),
-               (True,   "Meta+Shift+F1:show_menu"),
-               (True,   "Meta+Shift+F2:show_start_new_command"),
-               (True,   "Meta+Shift+F3:show_bug_report"),
-               (True,   "Meta+Shift+F4:quit"),
-               (True,   "Meta+Shift+F5:increase_quality"),
-               (True,   "Meta+Shift+F6:decrease_quality"),
-               (True,   "Meta+Shift+F7:increase_speed"),
-               (True,   "Meta+Shift+F8:decrease_speed"),
-               (True,   "Meta+Shift+F10:magic_key"),
-               (True,   "Meta+Shift+F11:show_session_info"),
-               (True,   "Meta+Shift+F12:toggle_debug"),
-               (True,   "Meta+Shift+plus:scaleup"),
-               (OSX,    "Meta+Shift+plusminus:scaleup"),
-               (True,   "Meta+Shift+minus:scaledown"),
-               (True,   "Meta+Shift+underscore:scaledown"),
-               (OSX,    "Meta+Shift+emdash:scaledown"),
-               (True,   "Meta+Shift+KP_Add:scaleup"),
-               (True,   "Meta+Shift+KP_Subtract:scaledown"),
-               (True,   "Meta+Shift+KP_Multiply:scalereset"),
-               (True,   "Meta+Shift+bar:scalereset"),
-               (True,   "Meta+Shift+question:scalingoff"),
-               (OSX,    "Meta+Shift+degree:scalereset")]
+               (not OSX,"Shift+F11:toggle_fullscreen"),
+               (OSX,    "Control+F11:toggle_fullscreen"),
+               (True,   "#+F1:show_menu"),
+               (True,   "#+F2:show_start_new_command"),
+               (True,   "#+F3:show_bug_report"),
+               (True,   "#+F4:quit"),
+               (True,   "#+F5:increase_quality"),
+               (True,   "#+F6:decrease_quality"),
+               (True,   "#+F7:increase_speed"),
+               (True,   "#+F8:decrease_speed"),
+               (True,   "#+F10:magic_key"),
+               (True,   "#+F11:show_session_info"),
+               (True,   "#+F12:toggle_debug"),
+               (True,   "#+plus:scaleup"),
+               (OSX,    "#+plusminus:scaleup"),
+               (True,   "#+minus:scaledown"),
+               (True,   "#+underscore:scaledown"),
+               (OSX,    "#+emdash:scaledown"),
+               (True,   "#+KP_Add:scaleup"),
+               (True,   "#+KP_Subtract:scaledown"),
+               (True,   "#+KP_Multiply:scalereset"),
+               (True,   "#+bar:scalereset"),
+               (True,   "#+question:scalingoff"),
+               (OSX,    "#+degree:scalereset")]
                  if e]
 
 def get_default_systemd_run():
@@ -712,16 +761,27 @@ def get_default_systemd_run():
     if is_CentOS() or is_RedHat():
         return "no"
     if is_Fedora():
-        #systemd-run is broken in Fedora 26:
-        #https://github.com/systemd/systemd/issues/3388
-        try:
-            ld = get_linux_distribution()
-            if int(ld[1])>=26:
-                return "no"
-        except:
-            pass
+        return "no"
+    #systemd-run was previously broken in Fedora 26:
+    #https://github.com/systemd/systemd/issues/3388
+    #but with newer kernels, it is working again..
     return "auto"
 
+def get_default_pulseaudio_command():
+    if WIN32 or OSX:
+        return []
+    cmd = ["pulseaudio", "--start", "-n", "--daemonize=false", "--system=false",
+                                    "--exit-idle-time=-1", "--load=module-suspend-on-idle",
+                                    "'--load=module-null-sink sink_name=\"Xpra-Speaker\" sink_properties=device.description=\"Xpra\\ Speaker\"'",
+                                    "'--load=module-null-sink sink_name=\"Xpra-Microphone\" sink_properties=device.description=\"Xpra\\ Microphone\"'",
+                                    "'--load=module-native-protocol-unix socket=$XPRA_PULSE_SERVER'",
+                                    "--load=module-dbus-protocol",
+                                    "--load=module-x11-publish",
+                                    "--log-level=2", "--log-target=stderr"]
+    if not is_Ubuntu() or getUbuntuVersion()>[16, 4]:
+        cmd.append("--enable-memfd=no")
+    return cmd
+
 
 GLOBAL_DEFAULTS = None
 #lowest common denominator here
@@ -730,7 +790,7 @@ def get_defaults():
     global GLOBAL_DEFAULTS
     if GLOBAL_DEFAULTS is not None:
         return GLOBAL_DEFAULTS
-    from xpra.platform.features import DEFAULT_SSH_COMMAND, OPEN_COMMAND, DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS, DEFAULT_PULSEAUDIO_COMMAND, \
+    from xpra.platform.features import DEFAULT_SSH_COMMAND, OPEN_COMMAND, DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS, \
                                         DEFAULT_ENV, CAN_DAEMONIZE, SYSTEM_PROXY_SOCKET
     from xpra.platform.paths import get_download_dir, get_remote_run_xpra_scripts
     try:
@@ -773,23 +833,16 @@ def get_defaults():
     if sys.version_info<(2, 7, 9):
         ssl_protocol = "SSLv23"
 
-    from xpra.os_util import getuid, getgid
-
     GLOBAL_DEFAULTS = {
-                    "encoding"          : "",
+                    "encoding"          : "auto",
                     "title"             : "@title@ on @client-machine@",
                     "username"          : username,
                     "password"          : "",
-                    "auth"              : "",
-                    "vsock-auth"        : "",
-                    "tcp-auth"          : "",
-                    "ssl-auth"          : "",
                     "wm-name"           : DEFAULT_NET_WM_NAME,
                     "session-name"      : "",
                     "dock-icon"         : "",
                     "tray-icon"         : "",
                     "window-icon"       : "",
-                    "password-file"     : "",
                     "keyboard-raw"      : False,
                     "keyboard-layout"   : "",
                     "keyboard-layouts"  : [],
@@ -801,7 +854,8 @@ def get_defaults():
                     "clipboard-filter-file" : "",
                     "remote-clipboard"  : "CLIPBOARD",
                     "local-clipboard"   : "CLIPBOARD",
-                    "pulseaudio-command": " ".join(DEFAULT_PULSEAUDIO_COMMAND),
+                    "pulseaudio-command": " ".join(get_default_pulseaudio_command()),
+                    "bandwidth-limit"   : "auto",
                     "encryption"        : "",
                     "tcp-encryption"    : "",
                     "encryption-keyfile": "",
@@ -823,7 +877,7 @@ def get_defaults():
                     "display"           : "",
                     "tcp-proxy"         : "",
                     "download-path"     : get_download_dir(),
-                    "open-command"      : OPEN_COMMAND,
+                    "open-command"      : " ".join(OPEN_COMMAND),
                     "remote-logging"    : "both",
                     "lpadmin"           : "/usr/sbin/lpadmin",
                     "lpinfo"            : "/usr/sbin/lpinfo",
@@ -840,6 +894,12 @@ def get_defaults():
                     "webcam"            : ["auto", "no"][OSX],
                     "mousewheel"        : "on",
                     "input-devices"     : "auto",
+                    "shortcut-modifiers": "auto",
+                    "open-files"        : "auto",
+                    "open-url"          : "auto",
+                    "file-transfer"     : "auto",
+                    "printing"          : "yes",
+                    "challenge-handlers": "all",
                     #ssl options:
                     "ssl"               : "auto",
                     "ssl-key"           : "",
@@ -860,7 +920,6 @@ def get_defaults():
                     "min-speed"         : 30,
                     "compression_level" : 1,
                     "dpi"               : 0,
-                    "video-scaling"     : 1,
                     "file-size-limit"   : 100,
                     "idle-timeout"      : 0,
                     "server-idle-timeout" : 0,
@@ -869,9 +928,10 @@ def get_defaults():
                     "uid"               : getuid(),
                     "gid"               : getgid(),
                     "min-port"          : 1024,
+                    "rfb-upgrade"       : 5,
                     "auto-refresh-delay": 0.15,
                     "daemon"            : CAN_DAEMONIZE,
-                    "start-via-proxy"   : None,
+                    "start-via-proxy"   : False,
                     "attach"            : None,
                     "use-display"       : False,
                     "fake-xinerama"     : not OSX and not WIN32,
@@ -883,15 +943,18 @@ def get_defaults():
                     "mmap-group"        : False,
                     "speaker"           : ["disabled", "on"][has_sound_support()],
                     "microphone"        : ["disabled", "off"][has_sound_support()],
+                    "video-scaling"     : "auto",
                     "readonly"          : False,
                     "keyboard-sync"     : True,
+                    "displayfd"         : 0,
                     "pings"             : 5,
                     "cursors"           : True,
                     "bell"              : True,
                     "notifications"     : True,
                     "xsettings"         : not OSX and not WIN32,
                     "system-tray"       : True,
-                    "sharing"           : False,
+                    "sharing"           : None,
+                    "lock"              : None,
                     "delay-tray"        : False,
                     "windows"           : True,
                     "terminate-children": False,
@@ -905,12 +968,13 @@ def get_defaults():
                     "dbus-control"      : not WIN32 and not OSX,
                     "opengl"            : get_opengl_default(),
                     "mdns"              : not WIN32,
-                    "file-transfer"     : True,
-                    "printing"          : True,
-                    "open-files"        : False,
                     "swap-keys"         : OSX,  #only used on osx
                     "desktop-fullscreen": False,
                     "global-menus"      : True,
+                    "forward-xdg-open"  : True,
+                    "modal-windows"     : True,
+                    "bandwidth-detection" : True,
+                    "ssh-upgrade"       : True,
                     "pulseaudio-configure-commands"  : [" ".join(x) for x in DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS],
                     "socket-dirs"       : [],
                     "remote-xpra"       : get_remote_run_xpra_scripts(),
@@ -927,7 +991,22 @@ def get_defaults():
                     "bind"              : bind_dirs,
                     "bind-vsock"        : [],
                     "bind-tcp"          : [],
+                    "bind-udp"          : [],
+                    "bind-ws"           : [],
+                    "bind-wss"          : [],
                     "bind-ssl"          : [],
+                    "bind-ssh"          : [],
+                    "bind-rfb"          : [],
+                    "auth"              : [],
+                    "vsock-auth"        : [],
+                    "tcp-auth"          : [],
+                    "udp-auth"          : [],
+                    "ws-auth"           : [],
+                    "wss-auth"          : [],
+                    "ssl-auth"          : [],
+                    "ssh-auth"          : [],
+                    "rfb-auth"          : [],
+                    "password-file"     : [],
                     "start"             : [],
                     "start-child"       : [],
                     "start-after-connect"       : [],
@@ -945,18 +1024,19 @@ CLONES = {}
 NO_FILE_OPTIONS = ["daemon"]
 
 
-TRUE_OPTIONS = ["yes", "true", "1", "on", True]
-FALSE_OPTIONS = ["no", "false", "0", "off", False]
-def parse_bool(k, v):
+TRUE_OPTIONS = ("yes", "true", "1", "on", True)
+FALSE_OPTIONS = ("no", "false", "0", "off", False)
+OFF_OPTIONS = ("off", )
+def parse_bool(k, v, auto=None):
     if type(v)==str:
-        v = v.lower()
+        v = v.lower().strip()
     if v in TRUE_OPTIONS:
         return True
     elif v in FALSE_OPTIONS:
         return False
     elif v in ["auto", None]:
         #keep default - which may be None!
-        return None
+        return auto
     else:
         try:
             return bool(int(v))
@@ -1002,6 +1082,37 @@ def print_number(i, auto_value=0):
         return "auto"
     return str(i)
 
+def parse_with_unit(numtype, v, subunit="bps", min_value=250000):
+    #special case for bandwidth-limit, which can be specified using units:
+    try:
+        v = str(v).lower().strip()
+        import re
+        if not v or v in FALSE_OPTIONS:
+            return 0
+        elif v=="auto":
+            return None
+        else:
+            r = re.match('([0-9\.]*)(.*)', v)
+            assert r
+            f = float(r.group(1))
+            unit = r.group(2).lower()
+            if unit.endswith(subunit):
+                unit = unit[:-len(subunit)]     #ie: 10mbps -> 10m
+            if unit=="b":
+                pass
+            elif unit=="k":
+                f *= 1000
+            elif unit=="m":
+                f *= 1000000
+            elif unit=="g":
+                f *= 1000000000
+            if min_value is not None:
+                assert f>=min_value, "value is too low"
+            return int(f)
+    except Exception as e:
+        raise InitException("invalid value for %s '%s': %s" % (numtype, v, e))
+
+
 def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_validation={}):
     """
         Validates all the options given in a dict with fields as keys and
@@ -1060,9 +1171,11 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_valid
     return nd
 
 
-def make_defaults_struct(extras_defaults={}, extras_types={}, extras_validation={}):
+def make_defaults_struct(extras_defaults={}, extras_types={}, extras_validation={}, username="", uid=getuid(), gid=getgid()):
     #populate config with default values:
-    defaults = read_xpra_defaults()
+    if not username and uid:
+        username = get_username_for_uid(uid)
+    defaults = read_xpra_defaults(username, uid, gid)
     return dict_to_validated_config(defaults, extras_defaults, extras_types, extras_validation)
 
 def dict_to_validated_config(d={}, extras_defaults={}, extras_types={}, extras_validation={}):
@@ -1108,26 +1221,35 @@ def _nodupes(s):
     return remove_dupes(x.strip().lower() for x in s.split(","))
 
 def fixup_video_all_or_none(options):
-    from xpra.codecs.video_helper import ALL_VIDEO_ENCODER_OPTIONS as aveco
-    from xpra.codecs.video_helper import ALL_CSC_MODULE_OPTIONS as acsco
-    from xpra.codecs.video_helper import ALL_VIDEO_DECODER_OPTIONS as avedo
+    #we import below, but only if we really need to access
+    #one of the lists, because those are expensive to probe
+    #(we have to load the codec, which may load other libraries)
+    #
+    #from xpra.codecs.video_helper import (
+    # ALL_VIDEO_ENCODER_OPTIONS,
+    # ALL_CSC_MODULE_OPTIONS,
+    # ALL_VIDEO_DECODER_OPTIONS,
+    # HARDWARE_ENCODER_OPTIONS,
+    #)
+    def getlist(strarg, help_txt, all_list_name):
+        if strarg=="help":
+            from xpra.codecs import video_helper
+            raise InitInfo("the following %s may be available: %s" % (help_txt, csv(getattr(video_helper, all_list_name))))
+        elif strarg=="none":
+            return []
+        elif strarg=="all":
+            from xpra.codecs import video_helper    #@Reimport
+            return getattr(video_helper, all_list_name)
+        else:
+            return [x for x in _nodupes(strarg) if x]
     vestr   = _csvstr(options.video_encoders)
     cscstr  = _csvstr(options.csc_modules)
     vdstr   = _csvstr(options.video_decoders)
     pvestr  = _csvstr(options.proxy_video_encoders)
-    def getlist(strarg, help_txt, all_list):
-        if strarg=="help":
-            raise InitInfo("the following %s may be available: %s" % (help_txt, ", ".join(all_list)))
-        elif strarg=="none":
-            return []
-        elif strarg=="all":
-            return all_list
-        else:
-            return [x for x in _nodupes(strarg) if x]
-    options.video_encoders  = getlist(vestr,    "video encoders",   aveco)
-    options.csc_modules     = getlist(cscstr,   "csc modules",      acsco)
-    options.video_decoders  = getlist(vdstr,    "video decoders",   avedo)
-    options.proxy_video_encoders = getlist(pvestr, "proxy video encoders", avedo)
+    options.video_encoders  = getlist(vestr,    "video encoders",   "ALL_VIDEO_ENCODER_OPTIONS")
+    options.csc_modules     = getlist(cscstr,   "csc modules",      "ALL_CSC_MODULE_OPTIONS")
+    options.video_decoders  = getlist(vdstr,    "video decoders",   "ALL_VIDEO_DECODER_OPTIONS")
+    options.proxy_video_encoders = getlist(pvestr, "proxy video encoders", "HARDWARE_ENCODER_OPTIONS")
 
 def fixup_socketdirs(options, defaults):
     if not options.socket_dirs:
@@ -1155,7 +1277,10 @@ def fixup_pings(options):
         options.pings = 5
 
 def fixup_encodings(options):
-    from xpra.codecs.loader import PREFERED_ENCODING_ORDER
+    try:
+        from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
+    except ImportError:
+        return
     RENAME = {"jpg" : "jpeg"}
     if options.encoding:
         options.encoding = RENAME.get(options.encoding, options.encoding)
@@ -1184,7 +1309,7 @@ def fixup_compression(options):
         compressors = _nodupes(cstr)
         unknown = [x for x in compressors if x and x not in compression.ALL_COMPRESSORS]
         if unknown:
-            warn("warning: invalid compressor(s) specified: %s" % (", ".join(unknown)))
+            warn("warning: invalid compressor(s) specified: %s" % csv(unknown))
     options.compressors = compressors
 
 def fixup_packetencoding(options):
@@ -1197,7 +1322,7 @@ def fixup_packetencoding(options):
         packet_encoders = _nodupes(pestr)
         unknown = [x for x in packet_encoders if x and x not in packet_encoding.ALL_ENCODERS]
         if unknown:
-            warn("warning: invalid packet encoder(s) specified: %s" % (", ".join(unknown)))
+            warn("warning: invalid packet encoder(s) specified: %s" % csv(unknown))
     options.packet_encoders = packet_encoders
 
 def fixup_keyboard(options):
@@ -1231,9 +1356,6 @@ def fixup_clipboard(options):
         warn(" specify 'to-server', 'to-client' or 'both'")
         options.clipboard_direction = "disabled"
 
-def fixup_bool(options):
-    options.sharing = parse_bool("sharing", options.sharing) or False
-
 def abs_paths(options):
     #convert to absolute paths before we daemonize
     for k in ("clipboard-filter-file",
@@ -1259,7 +1381,6 @@ def fixup_options(options, defaults={}):
     fixup_socketdirs(options, defaults)
     fixup_clipboard(options)
     fixup_keyboard(options)
-    fixup_bool(options)
     abs_paths(options)
     #remote-xpra is meant to be a list, but the user can specify a string using the command line,
     #in which case we replace all the default values with this single entry:
@@ -1275,7 +1396,7 @@ def main():
             if ot==bool and v is None:
                 v = "Auto"
             if type(v)==list:
-                v = ", ".join(str(x) for x in v)
+                v = csv(str(x) for x in v)
             print("* %-32s : %s" % (k, nonl(v)))
     from xpra.platform import program_context
     from xpra.log import enable_color
