@@ -15,5 +15,5 @@ class Authenticator(SysAuthenticator):
     def get_password(self):
         return None
 
-    def authenticate(self, challenge_response, client_salt):
+    def authenticate(self, _challenge_response=None, _client_salt=None):
         return True
