@@ -1,4 +1,4 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -56,6 +56,9 @@ class ArgsControlCommand(ControlCommand):
             self.raise_error("too many arguments, %i maximum" % self.max_args)
         args = list(args)
         for i,validation in enumerate(self.validation):
+            if i>=len(args):
+                #argument not supplied
+                continue
             v = args[i]
             log("running '%s' validation for argument %i: %s (value=%s, type=%s)", self.name, i, validation, v, type(v))
             if not validation:
@@ -76,7 +79,7 @@ class FixedMessageCommand(ControlCommand):
         super(FixedMessageCommand, self).__init__(name, help_text)
         self.message = message
 
-    def run(self, *args):
+    def run(self, *_args):
         return self.message
 
 
@@ -95,7 +98,7 @@ class HelpCommand(ArgsControlCommand):
 
     def run(self, *args):
         if len(args)==0:
-            return "control supports: %s" % (", ".join(self.control_commands))
+            return "control supports: %s" % csv(self.control_commands)
         name = args[0]
         command = self.control_commands.get(name)
         if not command:
