@@ -6,7 +6,9 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from time import sleep
 from threading import Event
+from collections import deque
 
 from xpra.log import Logger
 log = Logger("server")
@@ -21,26 +23,64 @@ netlog = Logger("network")
 bandwidthlog = Logger("bandwidth")
 
 
+from xpra.make_thread import start_thread
+from xpra.os_util import Queue, monotonic_time
+from xpra.util import merge_dicts, flatten_dict, notypedict, envbool, envint, typedict, AtomicInteger
 from xpra.server.source.source_stats import GlobalPerformanceStatistics
-from xpra.server.source.audio_mixin import AudioMixin
-from xpra.server.source.mmap_connection import MMAP_Connection
-from xpra.server.source.fileprint_mixin import FilePrintMixin
-from xpra.server.source.clipboard_connection import ClipboardConnection
-from xpra.server.source.networkstate_mixin import NetworkStateMixin
+
 from xpra.server.source.clientinfo_mixin import ClientInfoMixin
-from xpra.server.source.dbus_mixin import DBUS_Mixin
-from xpra.server.source.windows_mixin import WindowsMixin
-from xpra.server.source.encodings_mixin import EncodingsMixin
+CC_BASES = [ClientInfoMixin]
+from xpra.server import server_features
+#TODO: notifications mixin
+if server_features.clipboard:
+    from xpra.server.source.clipboard_connection import ClipboardConnection
+    CC_BASES.append(ClipboardConnection)
+if server_features.audio:
+    from xpra.server.source.audio_mixin import AudioMixin
+    CC_BASES.append(AudioMixin)
+if server_features.webcam:
+    from xpra.server.source.webcam_mixin import WebcamMixin
+    CC_BASES.append(WebcamMixin)
+if server_features.fileprint:
+    from xpra.server.source.fileprint_mixin import FilePrintMixin
+    CC_BASES.append(FilePrintMixin)
+if server_features.mmap:
+    from xpra.server.source.mmap_connection import MMAP_Connection
+    CC_BASES.append(MMAP_Connection)
+if server_features.input_devices:
+    from xpra.server.source.input_mixin import InputMixin
+    CC_BASES.append(InputMixin)
+if server_features.dbus:
+    from xpra.server.source.dbus_mixin import DBUS_Mixin
+    CC_BASES.append(DBUS_Mixin)
+if server_features.network_state:
+    from xpra.server.source.networkstate_mixin import NetworkStateMixin
+    CC_BASES.append(NetworkStateMixin)
+if server_features.display:
+    from xpra.server.source.clientdisplay_mixin import ClientDisplayMixin
+    CC_BASES.append(ClientDisplayMixin)
+if server_features.windows:
+    from xpra.server.source.windows_mixin import WindowsMixin
+    CC_BASES.append(WindowsMixin)
+    #must be after windows mixin so it can assume "self.send_windows" is set
+    if server_features.encoding:
+        from xpra.server.source.encodings_mixin import EncodingsMixin
+        CC_BASES.append(EncodingsMixin)
+    if server_features.audio and server_features.av_sync:
+        from xpra.server.source.avsync_mixin import AVSyncMixin
+        CC_BASES.append(AVSyncMixin)
 from xpra.server.source.idle_mixin import IdleMixin
-from xpra.server.source.input_mixin import InputMixin
-from xpra.server.source.avsync_mixin import AVSyncMixin
-from xpra.server.source.clientdisplay_mixin import ClientDisplayMixin
-from xpra.server.source.webcam_mixin import WebcamMixin
-from xpra.os_util import monotonic_time
-from xpra.util import merge_dicts, flatten_dict, notypedict, envbool, envint, AtomicInteger
+CC_BASES.append(IdleMixin)
+CC_BASES = tuple(CC_BASES)
+ClientConnectionClass = type('ClientConnectionClass', CC_BASES, {})
+log("ClientConnectionClass%s", CC_BASES)
+
 
 BANDWIDTH_DETECTION = envbool("XPRA_BANDWIDTH_DETECTION", True)
 MIN_BANDWIDTH = envint("XPRA_MIN_BANDWIDTH", 5*1024*1024)
+AUTO_BANDWIDTH_PCT = envint("XPRA_AUTO_BANDWIDTH_PCT", 80)
+assert AUTO_BANDWIDTH_PCT>1 and AUTO_BANDWIDTH_PCT<=100, "invalid value for XPRA_AUTO_BANDWIDTH_PCT: %i" % AUTO_BANDWIDTH_PCT
+NOYIELD = not envbool("XPRA_YIELD", False)
 
 counter = AtomicInteger()
 
@@ -62,72 +102,40 @@ adds the damage pixels ready for processing to the encode_work_queue,
 items are picked off by the separate 'encode' thread (see 'encode_loop')
 and added to the damage_packet_queue.
 """
-class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkStateMixin, ClientInfoMixin, DBUS_Mixin, WindowsMixin, EncodingsMixin, MMAP_Connection, IdleMixin, InputMixin, AVSyncMixin, ClientDisplayMixin, WebcamMixin):
 
-    def __init__(self, protocol, disconnect_cb, session_name,
-                 idle_add, timeout_add, source_remove, setting_changed,
-                 idle_timeout,
-                 socket_dir, unix_socket_paths, log_disconnect, dbus_control,
-                 get_transient_for, get_focus, get_cursor_data_cb,
-                 get_window_id,
-                 window_filters,
-                 file_transfer,
-                 supports_mmap, mmap_filename, min_mmap_size,
-                 bandwidth_limit,
-                 av_sync,
-                 core_encodings, encodings, default_encoding, scaling_control,
-                 webcam_enabled, webcam_device, webcam_encodings,
-                 sound_properties,
-                 sound_source_plugin,
-                 supports_speaker, supports_microphone,
-                 speaker_codecs, microphone_codecs,
-                 default_quality, default_min_quality,
-                 default_speed, default_min_speed):
-        log("ServerSource%s", (protocol, disconnect_cb, session_name,
-                 idle_add, timeout_add, source_remove, setting_changed,
-                 idle_timeout,
-                 socket_dir, unix_socket_paths, log_disconnect, dbus_control,
-                 get_transient_for, get_focus,
-                 get_window_id,
-                 window_filters,
-                 file_transfer,
-                 supports_mmap, mmap_filename, min_mmap_size,
-                 bandwidth_limit,
-                 av_sync,
-                 core_encodings, encodings, default_encoding, scaling_control,
-                 webcam_enabled, webcam_device, webcam_encodings,
-                 sound_properties,
-                 sound_source_plugin,
-                 supports_speaker, supports_microphone,
-                 speaker_codecs, microphone_codecs,
-                 default_quality, default_min_quality,
-                 default_speed, default_min_speed))
+class ClientConnection(ClientConnectionClass):
 
+    def __init__(self, protocol, disconnect_cb, session_name, server,
+                 idle_add, timeout_add, source_remove,
+                 setting_changed,
+                 socket_dir, unix_socket_paths, log_disconnect, bandwidth_limit, bandwidth_detection,
+                 ):
         global counter
         self.counter = counter.increase()
         self.protocol = protocol
         self.connection_time = monotonic_time()
         self.close_event = Event()
+        self.disconnect = disconnect_cb
         self.session_name = session_name
 
-        AudioMixin.__init__(self, sound_properties, sound_source_plugin,
-                 supports_speaker, supports_microphone, speaker_codecs, microphone_codecs)
-        MMAP_Connection.__init__(self, supports_mmap, mmap_filename, min_mmap_size)
-        ClipboardConnection.__init__(self)
-        FilePrintMixin.__init__(self, file_transfer)
-        NetworkStateMixin.__init__(self)
-        ClientInfoMixin.__init__(self)
-        DBUS_Mixin.__init__(self, dbus_control)
-        WindowsMixin.__init__(self, get_transient_for, get_focus, get_cursor_data_cb, get_window_id, window_filters)
-        EncodingsMixin.__init__(self, core_encodings, encodings, default_encoding, scaling_control, default_quality, default_min_quality, default_speed, default_min_speed)
-        IdleMixin.__init__(self, idle_timeout)
-        InputMixin.__init__(self)
-        AVSyncMixin.__init__(self, av_sync)
-        ClientDisplayMixin.__init__(self)
-        WebcamMixin.__init__(self, webcam_enabled, webcam_device, webcam_encodings)
+        for bc in CC_BASES:
+            try:
+                bc.__init__(self)
+                bc.init_from(self, protocol, server)
+            except Exception as e:
+                raise Exception("failed to initialize %s: %s" % (bc, e))
 
+        self.packet_queue = deque()                 #holds actual packets ready for sending (already encoded)
+                                                    #these packets are picked off by the "protocol" via 'next_packet()'
+                                                    #format: packet, wid, pixels, start_send_cb, end_send_cb
+                                                    #(only packet is required - the rest can be 0/None for clipboard packets)
+        # the encode work queue is used by mixins that need to encode data before sending it,
+        # ie: encodings and clipboard
+        self.encode_work_queue = None               #this queue will hold functions to call to compress data (pixels, clipboard)
+                                                    #items placed in this queue are picked off by the "encode" thread,
+                                                    #the functions should add the packets they generate to the 'packet_queue'
+        self.encode_thread = None
         self.ordinary_packets = []
-        self.disconnect = disconnect_cb
         self.socket_dir = socket_dir
         self.unix_socket_paths = unix_socket_paths
         self.log_disconnect = log_disconnect
@@ -138,6 +146,7 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
         self.setting_changed = setting_changed
         # network constraints:
         self.server_bandwidth_limit = bandwidth_limit
+        self.bandwidth_detection = bandwidth_detection
 
         #these statistics are shared by all WindowSource instances:
         self.statistics = GlobalPerformanceStatistics()
@@ -173,7 +182,7 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
         self.wants_display = True
         self.wants_events = False
         self.wants_default_cursor = False
-        for c in ClientConnection.__bases__:
+        for c in CC_BASES:
             c.init_state(self)
 
 
@@ -182,14 +191,17 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
 
     def close(self):
         log("%s.close()", self)
-        for c in ClientConnection.__bases__:
+        for c in CC_BASES:
             c.cleanup(self)
         self.close_event.set()
         self.protocol = None
 
 
     def update_bandwidth_limits(self):
-        if self.mmap_size>0:
+        if not self.bandwidth_detection:
+            return
+        mmap_size = getattr(self, "mmap_size", 0)
+        if mmap_size>0:
             return
         #calculate soft bandwidth limit based on send congestion data:
         bandwidth_limit = 0
@@ -208,7 +220,7 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
         bandwidthlog("update_bandwidth_limits() bandwidth_limit=%s, soft bandwidth limit=%s", self.bandwidth_limit, bandwidth_limit)
         #figure out how to distribute the bandwidth amongst the windows,
         #we use the window size,
-        #(we should actually use the number of bytes actually sent: framerate, compression, etc..)
+        #(we should use the number of bytes actually sent: framerate, compression, etc..)
         window_weight = {}
         for wid, ws in self.window_sources.items():
             weight = 0
@@ -238,23 +250,26 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
         self.wants_features = c.boolget("wants_features", True)
         self.wants_default_cursor = c.boolget("wants_default_cursor", False)
 
-        for mixin in ClientConnection.__bases__:
+        for mixin in CC_BASES:
             mixin.parse_client_caps(self, c)
 
         #general features:
         self.info_namespace = c.boolget("info-namespace")
         self.send_notifications = c.boolget("notifications")
         self.send_notifications_actions = c.boolget("notifications.actions")
+        log("notifications=%s, actions=%s", self.send_notifications, self.send_notifications_actions)
         self.share = c.boolget("share")
         self.lock = c.boolget("lock")
         self.control_commands = c.strlistget("control_commands")
         bandwidth_limit = c.intget("bandwidth-limit", 0)
-        if self.server_bandwidth_limit<=0:
-            self.bandwidth_limit = bandwidth_limit
-        else:
-            self.bandwidth_limit = min(self.server_bandwidth_limit, bandwidth_limit)
-        bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s", self.server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit)
-        log("cursors=%s (encodings=%s), bell=%s, notifications=%s", self.send_cursors, self.cursor_encodings, self.send_bell, self.send_notifications)
+        server_bandwidth_limit = self.server_bandwidth_limit
+        if self.server_bandwidth_limit is None:
+            server_bandwidth_limit = self.get_socket_bandwidth_limit() or bandwidth_limit
+        self.bandwidth_limit = min(server_bandwidth_limit, bandwidth_limit)
+        self.bandwidth_detection = c.boolget("bandwidth-detection", True)
+        cd = typedict(c.dictget("connection-data"))
+        self.jitter = cd.intget("jitter", 0)
+        bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s, detection=%s", server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit, self.bandwidth_detection)
 
         cinfo = self.get_connect_info()
         for i,ci in enumerate(cinfo):
@@ -265,19 +280,102 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
             if msg:
                 proxylog.warn("Warning: proxy version may not be compatible: %s", msg)
         self.update_connection_data(c.dictget("connection-data"))
-        if self.mmap_size>0:
+        if getattr(self, "mmap_size", 0)>0:
             log("mmap enabled, ignoring bandwidth-limit")
             self.bandwidth_limit = 0
         #adjust max packet size if file transfers are enabled:
-        if self.file_transfer:
+        #TODO: belongs in mixin:
+        file_transfer = getattr(self, "file_transfer", None)
+        if file_transfer:
             self.protocol.max_packet_size = max(self.protocol.max_packet_size, self.file_size_limit*1024*1024)
 
+    def get_socket_bandwidth_limit(self):
+        p = self.protocol
+        if not p:
+            return 0
+        #auto-detect:
+        pinfo = p.get_info()
+        socket_speed = pinfo.get("socket", {}).get("device", {}).get("speed")
+        if not socket_speed:
+            return 0
+        bandwidthlog("get_socket_bandwidth_limit() socket_speed=%s", socket_speed)
+        #auto: use 80% of socket speed if we have it:
+        return socket_speed*AUTO_BANDWIDTH_PCT//100 or 0
+
 
     def startup_complete(self):
         log("startup_complete()")
         self.send("startup-complete")
 
 
+    #
+    # The encode thread loop management:
+    #
+    def queue_encode(self, item):
+        #start the encode work queue:
+        self.encode_work_queue = Queue()            #holds functions to call to compress data (pixels, clipboard)
+                                                    #items placed in this queue are picked off by the "encode" thread,
+                                                    #the functions should add the packets they generate to the 'packet_queue'
+        self.queue_encode = self.encode_work_queue.put
+        self.encode_work_queue.put(item)
+        self.encode_thread = start_thread(self.encode_loop, "encode")
+
+    def encode_queue_size(self):
+        ewq = self.encode_work_queue
+        if ewq is None:
+            return 0
+        return ewq.qsize()
+
+    def call_in_encode_thread(self, *fn_and_args):
+        """
+            This is used by WindowSource to queue damage processing to be done in the 'encode' thread.
+            The 'encode_and_send_cb' will then add the resulting packet to the 'packet_queue' via 'queue_packet'.
+        """
+        self.statistics.compression_work_qsizes.append((monotonic_time(), self.encode_queue_size()))
+        self.queue_encode(fn_and_args)
+
+    def queue_packet(self, packet, wid=0, pixels=0, start_send_cb=None, end_send_cb=None, fail_cb=None, wait_for_more=False):
+        """
+            Add a new 'draw' packet to the 'packet_queue'.
+            Note: this code runs in the non-ui thread
+        """
+        now = monotonic_time()
+        self.statistics.packet_qsizes.append((now, len(self.packet_queue)))
+        if wid>0:
+            self.statistics.damage_packet_qpixels.append((now, wid, sum(x[2] for x in tuple(self.packet_queue) if x[1]==wid)))
+        self.packet_queue.append((packet, wid, pixels, start_send_cb, end_send_cb, fail_cb, wait_for_more))
+        p = self.protocol
+        if p:
+            p.source_has_more()
+
+    def encode_loop(self):
+        """
+            This runs in a separate thread and calls all the function callbacks
+            which are added to the 'encode_work_queue'.
+            Must run until we hit the end of queue marker,
+            to ensure all the queued items get called,
+            those that are marked as optional will be skipped when is_closed()
+        """
+        while True:
+            fn_and_args = self.encode_work_queue.get(True)
+            if fn_and_args is None:
+                return              #empty marker
+            #some function calls are optional and can be skipped when closing:
+            #(but some are not, like encoder clean functions)
+            optional_when_closing = fn_and_args[0]
+            if optional_when_closing and self.is_closed():
+                continue
+            try:
+                fn_and_args[1](*fn_and_args[2:])
+            except Exception as e:
+                if self.is_closed():
+                    log("ignoring encoding error in %s as source is already closed:", fn_and_args[0])
+                    log(" %s", e)
+                else:
+                    log.error("Error during encoding:", exc_info=True)
+                del e
+            NOYIELD or sleep(0)
+
     ######################################################################
     # network:
     def next_packet(self):
@@ -318,13 +416,8 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
 
     def send_hello(self, server_capabilities):
         capabilities = server_capabilities.copy()
-        merge_dicts(capabilities, AudioMixin.get_caps(self))
-        merge_dicts(capabilities, MMAP_Connection.get_caps(self))
-        merge_dicts(capabilities, WindowsMixin.get_caps(self))
-        merge_dicts(capabilities, EncodingsMixin.get_caps(self))
-        merge_dicts(capabilities, InputMixin.get_caps(self))
-        merge_dicts(capabilities, AVSyncMixin.get_caps(self))
-        merge_dicts(capabilities, ClientDisplayMixin.get_caps(self))
+        for bc in CC_BASES:
+            merge_dicts(capabilities, bc.get_caps(self))
         self.send("hello", capabilities)
         self.hello_sent = True
 
@@ -338,8 +431,10 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
                 "elapsed_time"      : int(monotonic_time()-self.connection_time),
                 "counter"           : self.counter,
                 "hello-sent"        : self.hello_sent,
+                "jitter"            : self.jitter,
                 "bandwidth-limit"   : {
                     "setting"       : self.bandwidth_limit or 0,
+                    "detection"     : self.bandwidth_detection,
                     "actual"        : self.soft_bandwidth_limit or 0,
                     }
                 }
@@ -349,17 +444,13 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
                          "connection"       : p.get_info(),
                          })
         info.update(self.get_features_info())
-        merge_dicts(info, FilePrintMixin.get_info(self))
-        merge_dicts(info, AudioMixin.get_info(self))
-        merge_dicts(info, MMAP_Connection.get_info(self))
-        merge_dicts(info, NetworkStateMixin.get_info(self))
-        merge_dicts(info, ClientInfoMixin.get_info(self))
-        merge_dicts(info, WindowsMixin.get_info(self))
-        merge_dicts(info, EncodingsMixin.get_info(self))
-        merge_dicts(info, AVSyncMixin.get_info(self))
-        merge_dicts(info, ClientDisplayMixin.get_info(self))
-        merge_dicts(info, IdleMixin.get_info(self))
-        merge_dicts(info, ClipboardConnection.get_info(self))
+        for bc in CC_BASES:
+            try:
+                merge_dicts(info, bc.get_info(self))
+            except Exception as e:
+                log("merge_dicts on %s", bc, exc_info=True)
+                log.error("Error: cannot add information from %s:", bc)
+                log.error(" %s", e)
         return info
 
     def get_features_info(self):
@@ -400,7 +491,7 @@ class ClientConnection(AudioMixin, ClipboardConnection, FilePrintMixin, NetworkS
             notifylog("not sending notification: %s", e)
         else:
             icon_filename = get_icon_filename(icon_name)
-            icon = parse_image_path(icon_filename)
+            icon = parse_image_path(icon_filename) or ""
             self.notify("", nid, "Xpra", 0, "", summary, body, actions, hints, expire_timeout, icon, user_callback)
 
     def notify(self, dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon, user_callback=None):
