@@ -1,4 +1,4 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -26,6 +26,8 @@ def std_unit_dec(v):
     unit, value = to_std_unit(v*10.0)
     if value>=100:
         return "%s%s" % (int(value//10), unit)
+    elif int(value)%10==0:
+        return "%s%s" % (int(value/10), unit)
     return "%s%s" % (int(value)/10.0, unit)
 
 
@@ -49,6 +51,8 @@ def values_to_scaled_values(data, scale_unit=10, min_scaled_value=10, num_values
     max_v = max(data)
     #pad with None values so we have at least num_values:
     if len(data)<num_values:
+        if isinstance(data, tuple):
+            data = list(data)
         for _ in range(num_values-len(data)):
             data.insert(0, None)
     scale = 1
@@ -115,7 +119,7 @@ def find_invpow(x, n):
 def get_list_stats(in_values, show_percentile=[5, 8, 9], show_dev=False):
     #this may be backed by a deque/list whichi is used by other threads
     #so make a copy before use:
-    values = list(in_values)
+    values = tuple(in_values)
     if len(values)==0:
         return  {}
     #arithmetic mean
