@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -42,7 +42,7 @@ GRACE_PERIOD = envint("XPRA_SOUND_GRACE_PERIOD", 2000)
 #percentage: from 0 for no margin, to 200% which triples the buffer target
 MARGIN = max(0, min(200, envint("XPRA_SOUND_MARGIN", 50)))
 #how high we push up the min-level to prevent underruns:
-UNDERRUN_MIN_LEVEL = max(0, envint("XPRA_SOUND_UNDERRUN_MIN_LEVEL", 50))
+UNDERRUN_MIN_LEVEL = max(0, envint("XPRA_SOUND_UNDERRUN_MIN_LEVEL", 150))
 
 
 GST_FORMAT_BYTES = 2
@@ -91,12 +91,13 @@ class SoundSink(SoundPipeline):
         self.last_data = None
         self.last_underrun = 0
         self.last_overrun = 0
+        self.refill = True
         self.last_max_update = monotonic_time()
         self.last_min_update = monotonic_time()
         self.level_lock = Lock()
         pipeline_els = []
         appsrc_el = ["appsrc",
-                     "do-timestamp=1",
+                     #"do-timestamp=1",
                      "name=src",
                      "emit-signals=0",
                      "block=0",
@@ -111,7 +112,6 @@ class SoundSink(SoundPipeline):
             pipeline_els.append(decoder_str)
         pipeline_els.append("audioconvert")
         pipeline_els.append("audioresample")
-        pipeline_els.append("volume name=volume volume=0")
         if QUEUE_TIME>0:
             pipeline_els.append(" ".join(["queue",
                                           "name=queue",
@@ -120,6 +120,7 @@ class SoundSink(SoundPipeline):
                                           "max-size-bytes=0",
                                           "max-size-time=%s" % QUEUE_TIME,
                                           "leaky=%s" % QUEUE_LEAK]))
+        pipeline_els.append("volume name=volume volume=0")
         sink_attributes = SINK_SHARED_DEFAULT_ATTRIBUTES.copy()
         #anything older than this may cause problems (ie: centos 6.x)
         #because the attributes may not exist
@@ -150,10 +151,8 @@ class SoundSink(SoundPipeline):
         return "SoundSink('%s' - %s)" % (self.pipeline_str, self.state)
 
     def cleanup(self):
-        if self.volume_timer!=0:
-            glib.source_remove(self.volume_timer)
-            self.volume_timer = 0
         SoundPipeline.cleanup(self)
+        self.cancel_volume_timer()
         self.sink_type = ""
         self.src = None
 
@@ -161,12 +160,18 @@ class SoundSink(SoundPipeline):
         SoundPipeline.start(self)
         self.timeout_add(UNMUTE_DELAY, self.start_adjust_volume)
 
+
     def start_adjust_volume(self, interval=100):
         if self.volume_timer!=0:
             glib.source_remove(self.volume_timer)
         self.volume_timer = self.timeout_add(interval, self.adjust_volume)
         return False
 
+    def cancel_volume_timer(self):
+        if self.volume_timer!=0:
+            glib.source_remove(self.volume_timer)
+            self.volume_timer = 0
+
 
     def adjust_volume(self):
         if not self.volume:
@@ -184,35 +189,42 @@ class SoundSink(SoundPipeline):
         return True
 
 
-    def queue_pushing(self, *args):
+    def queue_pushing(self, *_args):
         gstlog("queue_pushing")
+        self.queue_state = "pushing"
         self.emit_info()
         return True
 
-    def queue_running(self, *args):
+    def queue_running(self, *_args):
         gstlog("queue_running")
         self.queue_state = "running"
-        self.set_min_level()
-        self.set_max_level()
         self.emit_info()
         return True
 
-    def queue_underrun(self, *args):
+    def queue_underrun(self, *_args):
         now = monotonic_time()
         if self.queue_state=="starting" or 1000*(now-self.start_time)<GRACE_PERIOD:
             gstlog("ignoring underrun during startup")
             return 1
+        self.underruns += 1
         gstlog("queue_underrun")
         self.queue_state = "underrun"
-        if now-self.last_underrun>2:
-            self.last_underrun = now
-            self.set_min_level()
+        if now-self.last_underrun>5:
+            #only count underruns when we're back to no min time:
+            qmin = self.queue.get_property("min-threshold-time")//MS_TO_NS
+            clt = self.queue.get_property("current-level-time")//MS_TO_NS
+            gstlog("queue_underrun level=%3i, min=%3i", clt, qmin)
+            if qmin==0 and clt<10:
+                self.last_underrun = now
+                self.refill = True
+                self.set_max_level()
+                self.set_min_level()
         self.emit_info()
         return 1
 
     def get_level_range(self, mintime=2, maxtime=10):
         now = monotonic_time()
-        filtered = [v for t,v in list(self.levels) if (now-t)>=mintime and (now-t)<=maxtime]
+        filtered = [v for t,v in tuple(self.levels) if (now-t)>=mintime and (now-t)<=maxtime]
         if len(filtered)>=10:
             maxl = max(filtered)
             minl = min(filtered)
@@ -220,18 +232,18 @@ class SoundSink(SoundPipeline):
             return maxl-minl
         return 0
 
-    def queue_overrun(self, *args):
+    def queue_overrun(self, *_args):
         now = monotonic_time()
         if self.queue_state=="starting" or 1000*(now-self.start_time)<GRACE_PERIOD:
             gstlog("ignoring overrun during startup")
             return 1
         clt = self.queue.get_property("current-level-time")//MS_TO_NS
-        log("overrun level=%ims", clt)
+        log("queue_overrun level=%ims", clt)
         now = monotonic_time()
         #grace period of recording overruns:
         #(because when we record an overrun, we lower the max-time,
         # which causes more overruns!)
-        if self.last_overrun is None or now-self.last_overrun>2:
+        if now-self.last_overrun>2:
             self.last_overrun = now
             self.set_max_level()
             self.overrun_events.append(now)
@@ -243,30 +255,30 @@ class SoundSink(SoundPipeline):
             return
         now = monotonic_time()
         elapsed = now-self.last_min_update
+        lrange = self.get_level_range()
+        log("set_min_level() lrange=%i, elapsed=%i", lrange, elapsed)
         if elapsed<1:
             #not more than once a second
             return
-        lrange = self.get_level_range()
-        if lrange==0:
-            #not enough data
+        if self.refill:
+            #need to have a gap between min and max,
+            #so we cannot go higher than mst-50:
+            mst = self.queue.get_property("max-size-time")//MS_TO_NS
+            mrange = max(lrange+100, UNDERRUN_MIN_LEVEL)
+            mtt = min(mst-50, mrange)
+            gstlog("set_min_level mtt=%3i, max-size-time=%3i, lrange=%s, mrange=%s (UNDERRUN_MIN_LEVEL=%s)", mtt, mst, lrange, mrange, UNDERRUN_MIN_LEVEL)
+        else:
+            mtt = 0
+        cmtt = self.queue.get_property("min-threshold-time")//MS_TO_NS
+        if cmtt==mtt:
             return
         if not self.level_lock.acquire(False):
-            log("cannot get level lock for setting min-threshold-time")
+            gstlog("cannot get level lock for setting min-threshold-time")
             return
         try:
-            cmtt = self.queue.get_property("min-threshold-time")//MS_TO_NS
-            #from 100% down to 0% in 2 seconds after underrun:
-            now = monotonic_time()
-            pct = max(0, int((self.last_underrun+2-now)*50))
-            #cannot go higher than mst-50:
-            mst = self.queue.get_property("max-size-time")
-            mrange = max(lrange+100, 150)
-            mtt = min(mst-50, pct*max(UNDERRUN_MIN_LEVEL, mrange)//200)
-            log("set_min_level pct=%2i, cmtt=%3i, lrange=%s (UNDERRUN_MIN_LEVEL=%s)", pct, cmtt, lrange, UNDERRUN_MIN_LEVEL)
-            if cmtt!=mtt:
-                self.queue.set_property("min-threshold-time", mtt*MS_TO_NS)
-                log("set_min_level min-threshold-time=%s", mtt)
-                self.last_min_update = now
+            self.queue.set_property("min-threshold-time", mtt*MS_TO_NS)
+            gstlog("set_min_level min-threshold-time=%s", mtt)
+            self.last_min_update = now
         finally:
             self.level_lock.release()
 
@@ -278,30 +290,36 @@ class SoundSink(SoundPipeline):
         if elapsed<1:
             #not more than once a second
             return
+        lrange = self.get_level_range(mintime=0)
+        log("set_max_level lrange=%3i, elapsed=%is", lrange, int(elapsed))
+        cmst = self.queue.get_property("max-size-time")//MS_TO_NS
+        #overruns in the last minute:
+        olm = len([x for x in tuple(self.overrun_events) if now-x<60])
+        #increase target if we have more than 5 overruns in the last minute:
+        target_mst = lrange*(100 + MARGIN + min(100, olm*20))//100
+        #from 100% down to 0% in 2 seconds after underrun:
+        pct = max(0, int((self.last_overrun+2-now)*50))
+        #use this last_overrun percentage value to temporarily decrease the target
+        #(causes overruns that drop packets and lower the buffer level)
+        target_mst = max(50, int(target_mst - pct*lrange//100))
+        mst = (cmst + target_mst)//2
+        if self.refill:
+            #temporarily raise max level during underruns,
+            #so set_min_level has more room for manoeuver:
+            mst += UNDERRUN_MIN_LEVEL
+        #cap it at 1 second:
+        mst = min(mst, 1000)
+        log("set_max_level overrun count=%-2i, margin=%3i, pct=%2i, cmst=%3i, target=%3i, mst=%3i", olm, MARGIN, pct, cmst, target_mst, mst)
+        if abs(cmst-mst)<=max(50, lrange//2):
+            #not enough difference
+            return
         if not self.level_lock.acquire(False):
-            log("cannot get level lock for setting max-size-time")
+            gstlog("cannot get level lock for setting max-size-time")
             return
         try:
-            lrange = self.get_level_range(mintime=0)
-            log("set_max_level lrange=%3i, elapsed=%is", lrange, int(elapsed))
-            cmst = self.queue.get_property("max-size-time")//MS_TO_NS
-            #overruns in the last minute:
-            olm = len([x for x in list(self.overrun_events) if now-x<60])
-            #increase target if we have more than 5 overruns in the last minute:
-            target_mst = lrange*(100 + MARGIN + min(100, olm*20))//100
-            #from 100% down to 0% in 2 seconds after underrun:
-            pct = max(0, int((self.last_overrun+2-now)*50))
-            #use this last_overrun percentage value to temporarily decrease the target
-            #(causes overruns that drop packets and lower the buffer level)
-            target_mst = max(50, int(target_mst - pct*lrange//100))
-            mst = (cmst + target_mst)//2
-            #cap it at 1 second:
-            mst = min(mst, 1000)
-            log("set_max_level overrun count=%-2i, margin=%3i, pct=%2i, cmst=%3i", olm, MARGIN, pct, cmst)
-            if abs(cmst-mst)>=max(50, lrange//2):
-                self.queue.set_property("max-size-time", mst*MS_TO_NS)
-                log("set_max_level max-size-time=%s", mst)
-                self.last_max_update = now
+            self.queue.set_property("max-size-time", mst*MS_TO_NS)
+            log("set_max_level max-size-time=%s", mst)
+            self.last_max_update = now
         finally:
             self.level_lock.release()
 
@@ -360,15 +378,22 @@ class SoundSink(SoundPipeline):
             self.do_add_data(x)
         if self.do_add_data(data, metadata):
             self.rec_queue_level(data)
-            if self.queue_state=="pushing":
-                self.set_min_level()
-                self.set_max_level()
+            self.set_max_level()
+            self.set_min_level()
+            #drop back down quickly if the level has reached min:
+            if self.refill:
+                clt = self.queue.get_property("current-level-time")//MS_TO_NS
+                qmin = self.queue.get_property("min-threshold-time")//MS_TO_NS
+                gstlog("add_data: refill=%s, level=%i, min=%i", self.refill, clt, qmin)
+                if qmin>0 and clt>qmin:
+                    self.refill = False
         self.emit_info()
 
     def do_add_data(self, data, metadata=None):
         #having a timestamp causes problems with the queue and overruns:
         log("do_add_data(%s bytes, %s) queue_state=%s", len(data), metadata, self.queue_state)
-        buf = gst.new_buffer(data)
+        buf = gst.Buffer.new_allocate(None, len(data), None)
+        buf.fill(0, data)
         if metadata:
             #having a timestamp causes problems with the queue and overruns:
             #ts = metadata.get("timestamp")
@@ -387,10 +412,13 @@ class SoundSink(SoundPipeline):
         return False
 
     def rec_queue_level(self, data):
-        if self.queue:
-            clt = self.queue.get_property("current-level-time")//MS_TO_NS
-            log("pushed %5i bytes, new buffer level: %3ims, queue state=%s", len(data), clt, self.queue_state)
-            self.levels.append((monotonic_time(), clt))
+        q = self.queue
+        if not q:
+            return
+        clt = q.get_property("current-level-time")//MS_TO_NS
+        log("pushed %5i bytes, new buffer level: %3ims, queue state=%s", len(data), clt, self.queue_state)
+        now = monotonic_time()
+        self.levels.append((now, clt))
 
     def push_buffer(self, buf):
         #buf.size = size
@@ -400,7 +428,7 @@ class SoundSink(SoundPipeline):
         #buf.offset_end = offset_end
         #buf.set_caps(gst.caps_from_string(caps))
         r = self.src.emit("push-buffer", buf)
-        if r!=gst.FLOW_OK:
+        if r!=gst.FlowReturn.OK:
             if self.queue_state != "error":
                 log.error("Error pushing buffer: %s", r)
                 self.update_state("error")
@@ -455,7 +483,7 @@ def main():
         QUEUE_SILENT = True
         ss = SoundSink(codecs=codecs)
         def eos(*args):
-            print("eos")
+            print("eos%s" % (args,))
             glib.idle_add(glib_mainloop.quit)
         ss.connect("eos", eos)
         ss.start()
@@ -463,10 +491,10 @@ def main():
         glib_mainloop = glib.MainLoop()
 
         import signal
-        def deadly_signal(*args):
+        def deadly_signal(*_args):
             glib.idle_add(ss.stop)
             glib.idle_add(glib_mainloop.quit)
-            def force_quit(sig, frame):
+            def force_quit(_sig, _frame):
                 sys.exit()
             signal.signal(signal.SIGINT, force_quit)
             signal.signal(signal.SIGTERM, force_quit)
@@ -475,7 +503,7 @@ def main():
             signal.signal(signal.SIGINT, deadly_signal)
         signal.signal(signal.SIGTERM, deadly_signal)
 
-        def check_for_end(*args):
+        def check_for_end(*_args):
             qtime = ss.queue.get_property("current-level-time")//MS_TO_NS
             if qtime<=0:
                 log.info("underrun (end of stream)")
