@@ -1,26 +1,54 @@
 # This file is part of Xpra.
-# Copyright (C) 2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2015-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-import time
 import sys
 
-from xpra.sound.gstreamer_util import parse_sound_source, get_source_plugins, parse_element_options, format_element_options, \
+from collections import namedtuple
+from xpra.sound.gstreamer_util import parse_sound_source, get_source_plugins, get_sink_plugins, get_default_sink, get_default_source, \
+                            import_gst, format_element_options, \
                             can_decode, can_encode, get_muxers, get_demuxers, get_all_plugin_names
 from xpra.net.subprocess_wrapper import subprocess_caller, subprocess_callee, exec_kwargs, exec_env
 from xpra.platform.paths import get_sound_command
-from xpra.util import AdHocStruct, typedict
+from xpra.os_util import WIN32, OSX, POSIX, BITS, monotonic_time, bytestostr
+from xpra.util import typedict, parse_simple_dict, envint, envbool
 from xpra.scripts.config import InitExit, InitException
 from xpra.log import Logger
 log = Logger("sound")
 
-DEBUG_SOUND = os.environ.get("XPRA_SOUND_DEBUG", "0")=="1"
-SUBPROCESS_DEBUG = os.environ.get("XPRA_SOUND_SUBPROCESS_DEBUG", "").split(",")
-FAKE_START_FAILURE = os.environ.get("XPRA_SOUND_FAKE_START_FAILURE", "0")=="1"
-FAKE_EXIT = int(os.environ.get("XPRA_SOUND_FAKE_EXIT", "0"))
-FAKE_CRASH = int(os.environ.get("XPRA_SOUND_FAKE_CRASH", "0"))
+DEBUG_SOUND = envbool("XPRA_SOUND_DEBUG", False)
+SUBPROCESS_DEBUG = [x.strip() for x in os.environ.get("XPRA_SOUND_SUBPROCESS_DEBUG", "").split(",") if x.strip()]
+FAKE_START_FAILURE = envbool("XPRA_SOUND_FAKE_START_FAILURE", False)
+FAKE_EXIT = envbool("XPRA_SOUND_FAKE_EXIT", False)
+FAKE_CRASH = envbool("XPRA_SOUND_FAKE_CRASH", False)
+SOUND_START_TIMEOUT = envint("XPRA_SOUND_START_TIMEOUT", 5000)
+BUNDLE_METADATA = envbool("XPRA_SOUND_BUNDLE_METADATA", True)
+
+DEFAULT_SOUND_COMMAND_ARGS = os.environ.get("XPRA_DEFAULT_SOUND_COMMAND_ARGS", "--windows=no --video-encoders=none --csc-modules=none --video-decoders=none --proxy-video-encoders=none").split(" ")
+
+
+def get_full_sound_command():
+    return get_sound_command()+DEFAULT_SOUND_COMMAND_ARGS
+
+
+def get_sound_wrapper_env():
+    env = {
+        #no need to add our ssl hooks for the sound subprocess:
+        "XPRA_SSL_PEEK" : "0"
+        }
+    if WIN32:
+        #disable bencoder to skip warnings with the py3k Sound subapp
+        env["XPRA_USE_BENCODER"] = "0"
+    elif POSIX and not OSX:
+        try:
+            from xpra.sound.pulseaudio.pulseaudio_util import add_audio_tagging_env
+            add_audio_tagging_env(env)
+        except ImportError as e:
+            log.warn("Warning: failed to set pulseaudio tagging:")
+            log.warn(" %s", e)
+    return env
 
 
 #this wrapper takes care of launching src.py or sink.py
@@ -72,11 +100,12 @@ class sound_subprocess(subprocess_callee):
 
     def cleanup(self):
         wo = self.wrapped_object
-        log("stop() wrapped object=%s", wo)
+        log("cleanup() wrapped object=%s", wo)
         if wo:
             #this will stop the sound pipeline:
             self.wrapped_object = None
             wo.cleanup()
+        self.timeout_add(1000, self.do_stop)
 
     def export_info(self):
         wo = self.wrapped_object
@@ -91,7 +120,7 @@ class sound_record(sound_subprocess):
         from xpra.sound.src import SoundSource
         sound_pipeline = SoundSource(*pipeline_args)
         sound_subprocess.__init__(self, sound_pipeline, [], ["new-stream", "new-buffer"])
-        self.large_packets = ["new-buffer"]
+        self.large_packets = [b"new-buffer"]
 
 class sound_play(sound_subprocess):
     """ wraps SoundSink as a subprocess """
@@ -105,84 +134,90 @@ def run_sound(mode, error_cb, options, args):
     """ this function just parses command line arguments to feed into the sound subprocess class,
         which in turn just feeds them into the sound pipeline class (sink.py or src.py)
     """
-    #we have to import gstreamer before init() on OSX,
-    #because init will end up import gobject,
-    #which means choosing between gi and gtk2 bindings
-    from xpra.sound.gstreamer_util import import_gst, get_pygst_version
-    gst = import_gst()
     from xpra.gtk_common.gobject_compat import want_gtk3
-    want_gtk3(get_pygst_version()[0]>0)
-    from xpra.platform import init
-    init("Xpra")
-    log("run_sound(%s, %s, %s, %s) gst=%s", mode, error_cb, options, args, gst)
+    want_gtk3(True)
+    gst = import_gst()
     if not gst:
         return 1
-    if mode=="_sound_record":
-        subproc = sound_record
-        info = "record"
-    elif mode=="_sound_play":
-        subproc = sound_play
-        info = "play"
-    elif mode=="_sound_query":
-        plugins = get_all_plugin_names()
-        sources = [x for x in get_source_plugins() if x in plugins]
-        from xpra.sound.gstreamer_util import gst_version, pygst_version
-        d = {"encoders"         : can_encode(),
-             "decoders"         : can_decode(),
-             "sources"          : sources,
-             "muxers"           : get_muxers(),
-             "demuxers"         : get_demuxers(),
-             "gst.version"      : gst_version,
-             "pygst.version"    : pygst_version,
-             "plugins"          : plugins,
-             "python.version"   : sys.version_info[:3],
-            }
-        for k,v in d.items():
-            print("%s=%s" % (k, ",".join(str(x) for x in v)))
-        return 0
-    else:
-        log.error("unknown mode: %s" % mode)
-        return 1
-    assert len(args)>=6, "not enough arguments"
+    info = mode.replace("_sound_", "")  #ie: "_sound_record" -> "record"
+    from xpra.platform import program_context
+    with program_context("Xpra-Audio-%s" % info, "Xpra Audio %s" % info):
+        log("run_sound(%s, %s, %s, %s) gst=%s", mode, error_cb, options, args, gst)
+        if mode=="_sound_record":
+            subproc = sound_record
+        elif mode=="_sound_play":
+            subproc = sound_play
+        elif mode=="_sound_query":
+            plugins = get_all_plugin_names()
+            sources = [x for x in get_source_plugins() if x in plugins]
+            sinks = [x for x in get_sink_plugins() if x in plugins]
+            from xpra.sound.gstreamer_util import get_gst_version, get_pygst_version
+            d = {
+                 "encoders"         : can_encode(),
+                 "decoders"         : can_decode(),
+                 "sources"          : sources,
+                 "source.default"   : get_default_source() or "",
+                 "sinks"            : sinks,
+                 "sink.default"     : get_default_sink() or "",
+                 "muxers"           : get_muxers(),
+                 "demuxers"         : get_demuxers(),
+                 "gst.version"      : [int(x) for x in get_gst_version()],
+                 "pygst.version"    : get_pygst_version(),
+                 "plugins"          : plugins,
+                 "python.version"   : sys.version_info[:3],
+                 "python.bits"      : BITS,
+                }
+            if BUNDLE_METADATA:
+                d["bundle-metadata"] = True
+            for k,v in d.items():
+                if type(v) in (list, tuple):
+                    v = ",".join(str(x) for x in v)
+                print("%s=%s" % (k, v))
+            return 0
+        else:
+            log.error("unknown mode: %s" % mode)
+            return 1
+        assert len(args)>=6, "not enough arguments"
 
-    #the plugin to use (ie: 'pulsesrc' for src.py or 'autoaudiosink' for sink.py)
-    plugin = args[2]
-    #plugin options (ie: "device=monitor_device,something=value")
-    options = parse_element_options(args[3])
-    #codecs:
-    codecs = [x.strip() for x in args[4].split(",")]
-    #codec options:
-    codec_options = parse_element_options(args[5])
-    #volume (optional):
-    try:
-        volume = int(args[6])
-    except:
-        volume = 1.0
+        #the plugin to use (ie: 'pulsesrc' for src.py or 'autoaudiosink' for sink.py)
+        plugin = args[2]
+        #plugin options (ie: "device=monitor_device,something=value")
+        options = parse_simple_dict(args[3])
+        #codecs:
+        codecs = [x.strip() for x in args[4].split(",")]
+        #codec options:
+        codec_options = parse_simple_dict(args[5])
+        #volume (optional):
+        try:
+            volume = int(args[6])
+        except:
+            volume = 1.0
 
-    ss = None
-    try:
-        ss = subproc(plugin, options, codecs, codec_options, volume)
-        ss.start()
-        return 0
-    except InitExit as e:
-        log.error("%s: %s", info, e)
-        return e.status
-    except InitException as e:
-        log.error("%s: %s", info, e)
-        return 1
-    except Exception:
-        log.error("run_sound%s error", (mode, error_cb, options, args), exc_info=True)
-        return 1
-    finally:
-        if ss:
-            ss.stop()
+        ss = None
+        try:
+            ss = subproc(plugin, options, codecs, codec_options, volume)
+            ss.start()
+            return 0
+        except InitExit as e:
+            log.error("%s: %s", info, e)
+            return e.status
+        except InitException as e:
+            log.error("%s: %s", info, e)
+            return 1
+        except Exception:
+            log.error("run_sound%s error", (mode, error_cb, options, args), exc_info=True)
+            return 1
+        finally:
+            if ss:
+                ss.stop()
 
 
 def _add_debug_args(command):
     from xpra.log import debug_enabled_categories
     debug = SUBPROCESS_DEBUG[:]
-    if (DEBUG_SOUND or "sound" in debug_enabled_categories) and ("sound" not in debug):
-        debug.append("sound")
+    for f in ("sound", "gstreamer"):
+        if (DEBUG_SOUND or f in debug_enabled_categories) and (f not in debug):
+            debug.append(f)
     if debug:
         #forward debug flags:
         command += ["-d", ",".join(debug)]
@@ -200,7 +235,7 @@ class sound_subprocess_wrapper(subprocess_caller):
         self.state = "stopped"
         self.codec = "unknown"
         self.codec_description = ""
-        self.last_info = {}
+        self.info = {}
         #hook some default packet handlers:
         self.connect("state-changed", self.state_changed)
         self.connect("info", self.info_update)
@@ -208,19 +243,14 @@ class sound_subprocess_wrapper(subprocess_caller):
 
     def get_env(self):
         env = subprocess_caller.get_env(self)
-        try:
-            from xpra.sound.pulseaudio_util import add_audio_tagging_env
-            add_audio_tagging_env(env)
-        except Exception as e:
-            log.warn("Warning: failed to set pulseaudio tagging:")
-            log.warn(" %s", e)
+        env.update(get_sound_wrapper_env())
         return env
 
     def start(self):
         self.state = "starting"
         subprocess_caller.start(self)
         log("start() %s subprocess(%s)=%s", self.description, self.command, self.process.pid)
-        self.timeout_add(2500, self.verify_started)
+        self.timeout_add(SOUND_START_TIMEOUT, self.verify_started)
 
 
     def cleanup(self):
@@ -228,28 +258,29 @@ class sound_subprocess_wrapper(subprocess_caller):
         self.send("cleanup")
         #cleanup should cause the process to exit
         self.timeout_add(500, self.send, "stop")
-        self.timeout_add(1000, self.stop)
+        self.timeout_add(1000, self.send, "exit")
+        self.timeout_add(1500, self.stop)
 
 
     def verify_started(self):
         p = self.process
-        log("verify_started() process=%s, last_info=%s, codec=%s", p, self.last_info, self.codec)
+        log("verify_started() process=%s, info=%s, codec=%s", p, self.info, self.codec)
         if p is None or p.poll() is not None:
             #process has terminated already
             return
         #if we don't get an "info" packet, then the pipeline must have failed to start
-        if not self.last_info:
-            log.warn("the %s process has failed to start, stopping it", self.description)
+        if not self.info:
+            log.warn("Warning: the %s process has failed to start", self.description)
             self.cleanup()
 
 
-    def subprocess_signal(self, wrapper, proc):
+    def subprocess_signal(self, _wrapper, proc):
         log("subprocess_signal: %s", proc)
         #call via idle_add to prevent deadlocks on win32!
         self.idle_add(self.stop_protocol)
 
 
-    def state_changed(self, wrapper, new_state):
+    def state_changed(self, _wrapper, new_state):
         self.state = new_state
 
     def get_state(self):
@@ -257,15 +288,15 @@ class sound_subprocess_wrapper(subprocess_caller):
 
 
     def get_info(self):
-        return self.last_info
+        return self.info
 
-    def info_update(self, wrapper, info):
+    def info_update(self, _wrapper, info):
         log("info_update: %s", info)
-        self.last_info = info
-        self.last_info["time"] = int(time.time())
+        self.info.update(info)
+        self.info["time"] = int(monotonic_time())
         p = self.process
         if p and not p.poll():
-            self.last_info["pid"] = p.pid
+            self.info["pid"] = p.pid
         self.codec_description = info.get("codec_description")
 
 
@@ -273,15 +304,15 @@ class sound_subprocess_wrapper(subprocess_caller):
         self.send("set_volume", int(v*100))
 
     def get_volume(self):
-        return self.last_info.get("volume", 100)/100.0
+        return self.info.get("volume", 100)/100.0
 
 
 class source_subprocess_wrapper(sound_subprocess_wrapper):
 
     def __init__(self, plugin, options, codecs, volume, element_options):
         sound_subprocess_wrapper.__init__(self, "sound source")
-        self.large_packets = ["new-buffer"]
-        self.command = get_sound_command()+["_sound_record", "-", "-", plugin or "", format_element_options(element_options), ",".join(codecs), "", str(volume)]
+        self.large_packets = [b"new-buffer"]
+        self.command = get_full_sound_command()+["_sound_record", "-", "-", plugin or "", format_element_options(element_options), ",".join(codecs), "", str(volume)]
         _add_debug_args(self.command)
 
     def __repr__(self):
@@ -293,17 +324,17 @@ class source_subprocess_wrapper(sound_subprocess_wrapper):
 
 class sink_subprocess_wrapper(sound_subprocess_wrapper):
 
-    def __init__(self, plugin, options, codec, volume, element_options):
+    def __init__(self, plugin, codec, volume, element_options):
         sound_subprocess_wrapper.__init__(self, "sound output")
-        self.large_packets = ["add_data"]
+        self.large_packets = [b"add_data"]
         self.codec = codec
-        self.command = get_sound_command()+["_sound_play", "-", "-", plugin or "", format_element_options(element_options), codec, "", str(volume)]
+        self.command = get_full_sound_command()+["_sound_play", "-", "-", plugin or "", format_element_options(element_options), codec, "", str(volume)]
         _add_debug_args(self.command)
 
-    def add_data(self, data, metadata):
+    def add_data(self, data, metadata={}, packet_metadata=()):
         if DEBUG_SOUND:
-            log("add_data(%s bytes, %s) forwarding to %s", len(data), metadata, self.protocol)
-        self.send("add_data", data, dict(metadata))
+            log("add_data(%s bytes, %s, %s) forwarding to %s", len(data), metadata, len(packet_metadata), self.protocol)
+        self.send("add_data", data, dict(metadata), packet_metadata)
 
     def __repr__(self):
         try:
@@ -312,15 +343,15 @@ class sink_subprocess_wrapper(sound_subprocess_wrapper):
             return "sink_subprocess_wrapper(%s)" % self.process
 
 
-def start_sending_sound(plugins, sound_source_plugin, codec, volume, want_monitor_device, remote_decoders, remote_pulseaudio_server, remote_pulseaudio_id):
-    log("start_sending_sound%s", (plugins, sound_source_plugin, codec, volume, want_monitor_device, remote_decoders, remote_pulseaudio_server, remote_pulseaudio_id))
+def start_sending_sound(plugins, sound_source_plugin, device, codec, volume, want_monitor_device, remote_decoders, remote_pulseaudio_server, remote_pulseaudio_id):
+    log("start_sending_sound%s", (plugins, sound_source_plugin, device, codec, volume, want_monitor_device, remote_decoders, remote_pulseaudio_server, remote_pulseaudio_id))
     try:
         #info about the remote end:
-        remote = AdHocStruct()
-        remote.pulseaudio_server = remote_pulseaudio_server
-        remote.pulseaudio_id = remote_pulseaudio_id
-        remote.remote_decoders = remote_decoders
-        plugin, options = parse_sound_source(plugins, sound_source_plugin, want_monitor_device, remote)
+        PAInfo = namedtuple("PAInfo", "pulseaudio_server,pulseaudio_id,remote_decoders")
+        remote = PAInfo(pulseaudio_server=remote_pulseaudio_server,
+                        pulseaudio_id=remote_pulseaudio_id,
+                        remote_decoders=remote_decoders)
+        plugin, options = parse_sound_source(plugins, sound_source_plugin, device, want_monitor_device, remote)
         if not plugin:
             log.error("failed to setup '%s' sound stream source", (sound_source_plugin or "auto"))
             return  None
@@ -336,29 +367,35 @@ def start_sending_sound(plugins, sound_source_plugin, codec, volume, want_monito
 def start_receiving_sound(codec):
     log("start_receiving_sound(%s)", codec)
     try:
-        return sink_subprocess_wrapper(None, {}, codec, 1.0, {})
+        return sink_subprocess_wrapper(None, codec, 1.0, {})
     except:
         log.error("failed to start sound sink", exc_info=True)
         return None
 
 def query_sound():
     import subprocess
-    command = get_sound_command()+["_sound_query"]
+    command = get_full_sound_command()+["_sound_query"]
     _add_debug_args(command)
     kwargs = exec_kwargs()
     env = exec_env()
+    env.update(get_sound_wrapper_env())
     log("query_sound() command=%s, env=%s, kwargs=%s", command, env, kwargs)
-    proc = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=sys.stderr.fileno(), env=env, **kwargs)
+    proc = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, env=env, **kwargs)
     out, err = proc.communicate(None)
     log("query_sound() process returned %s", proc.returncode)
     log("query_sound() out=%s, err=%s", out, err)
     if proc.returncode!=0:
         return typedict()
     d = typedict()
-    for x in out.decode("utf8").splitlines():
-        kv = x.split("=", 1)
+    for x in out.splitlines():
+        kv = x.split(b"=", 1)
         if len(kv)==2:
             #ie: kv = ["decoders", "mp3,vorbis"]
-            d[kv[0].encode()] = [x.encode() for x in kv[1].split(",")]     #d["decoders"] = ["mp3", "vorbis"]
+            k,v = kv
+            #fugly warning: all the other values are lists.. but this one is not:
+            if k!=b"python.bits":
+                v = [bytestostr(x) for x in v.split(b",")]
+            #d["decoders"] = ["mp3", "vorbis"]
+            d[bytestostr(k)] = v
     log("query_sound()=%s", d)
     return d
