@@ -3,8 +3,17 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.gtk_common.gobject_compat import import_gdk
-gdk = import_gdk()
+#if you want to use a virtual screen bigger than 32767x32767
+#you will need to change those values, but some broken toolkits
+#will then misbehave (they use signed shorts instead of signed ints..)
+MAX_WINDOW_SIZE = 2**15-1
+
+class Unmanageable(Exception):
+    pass
+
+
+REPR_FUNCTIONS = {}
+
 
 # Just to make it easier to pass around and have a helpful debug logging.
 # Really, just a python objects where we can stick random bags of attributes.
@@ -19,10 +28,7 @@ class X11Event(object):
                 continue
             elif k=="serial":
                 d[k] = "%#x" % v
-            elif v and type(v)==gdk.Window:
-                d[k] = "%#x" % v.xid
-            elif v and type(v)==gdk.Display:
-                d[k] = "%s" % v.get_name()
             else:
-                d[k] = v
+                fn = REPR_FUNCTIONS.get(type(v), str)
+                d[k] = fn(v)
         return "<X11:%s %r>" % (self.name, d)
