@@ -1,22 +1,25 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2016-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2016-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-import gtk.gdk
+from gtk import gdk
 import gobject
 import socket
 
-from xpra.util import updict, log_screen_sizes, envbool
-from xpra.os_util import get_generic_os_name
+from xpra.os_util import get_generic_os_name, PYTHON3
+if PYTHON3:
+    raise ImportError("desktop server needs porting to python3 / gtk3")
+from xpra.util import updict, log_screen_sizes
 from xpra.platform.paths import get_icon
 from xpra.platform.gui import get_wm_name
+from xpra.server import server_features
 from xpra.gtk_common.gobject_util import one_arg_signal, no_arg_signal
 from xpra.gtk_common.gobject_compat import import_glib
 from xpra.gtk_common.error import xswallow
-from xpra.gtk_common.gtk_util import get_screen_sizes, get_root_size
+from xpra.gtk_common.gtk_util import get_screen_sizes, get_root_size, get_xwindow
 from xpra.x11.gtk2.models.model_stub import WindowModelStub
 from xpra.x11.gtk2.gdk_bindings import (
                                add_catchall_receiver,       #@UnresolvedImport
@@ -27,6 +30,7 @@ from xpra.x11.gtk2.gdk_bindings import (
                                cleanup_all_event_receivers  #@UnresolvedImport
                                )
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+from xpra.x11.xroot_props import XRootPropWatcher
 from xpra.x11.gtk2.window_damage import WindowDamageHandler
 X11Window = X11WindowBindings()
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
@@ -40,16 +44,13 @@ from xpra.log import Logger
 log = Logger("server")
 windowlog = Logger("server", "window")
 geomlog = Logger("server", "window", "geometry")
-traylog = Logger("server", "tray")
 settingslog = Logger("x11", "xsettings")
 metadatalog = Logger("x11", "metadata")
 screenlog = Logger("screen")
-
+iconlog = Logger("icon")
 
 glib = import_glib()
 
-FORCE_SCREEN_MISMATCH = envbool("XPRA_FORCE_SCREEN_MISMATCH", False)
-
 
 class DesktopModel(WindowModelStub, WindowDamageHandler):
     __gsignals__ = {}
@@ -71,20 +72,28 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
         "size-hints": (gobject.TYPE_PYOBJECT,
                        "Client hints on constraining its size", "",
                        gobject.PARAM_READABLE),
+        "wm-name": (gobject.TYPE_PYOBJECT,
+                       "The name of the window manager or session manager", "",
+                       gobject.PARAM_READABLE),
+        "icon": (gobject.TYPE_PYOBJECT,
+                       "The icon of the window manager or session manager", "",
+                       gobject.PARAM_READABLE),
         }
 
 
     _property_names         = ["xid", "client-machine", "window-type", "shadow", "size-hints", "class-instance", "focused", "title", "depth", "icon"]
-    _dynamic_property_names = ["size-hints"]
+    _dynamic_property_names = ["size-hints", "title", "icon"]
 
-    def __init__(self, root):
+    def __init__(self, root, resize_exact=False):
         WindowDamageHandler.__init__(self, root)
         WindowModelStub.__init__(self)
-        self.resize_timer = None
-        self.resize_value = None
+        self.root_prop_watcher = XRootPropWatcher(["WINDOW_MANAGER", "_NET_SUPPORTING_WM_CHECK"], root)
+        self.root_prop_watcher.connect("root-prop-changed", self.root_prop_changed)
+        self.update_icon()
+        self.resize_exact = resize_exact
 
     def __repr__(self):
-        return "DesktopModel(%#x)" % (self.client_window.xid)
+        return "DesktopModel(%#x)" % get_xwindow(self.client_window)
 
 
     def setup(self):
@@ -100,10 +109,34 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
         WindowDamageHandler.destroy(self)
         WindowModelStub.unmanage(self, exiting)
         self._managed = False
-        rt = self.resize_timer
-        if rt:
-            self.resize_timer = None
-            glib.source_remove(rt)
+        rpw = self.root_prop_watcher
+        if rpw:
+            self.root_prop_watcher = None
+            rpw.cleanup()
+
+    def root_prop_changed(self, watcher, prop):
+        iconlog("root_prop_changed(%s, %s)", watcher, prop)
+        if self.update_wm_name():
+            self.update_icon()
+
+    def update_wm_name(self):
+        wm_name = ""
+        try:
+            wm_name = get_wm_name()
+        except:
+            pass
+        iconlog("update_wm_name() wm-name=%s", wm_name)
+        return self._updateprop("wm-name", wm_name)
+
+    def update_icon(self):
+        icon = None
+        try:
+            icon_name = (get_wm_name() or "").lower()+".png"
+            icon = get_icon(icon_name)
+            iconlog("get_icon(%s)=%s", icon_name, icon)
+        except:
+            iconlog("failed to return window icon")
+        return self._updateprop("icon", icon)
 
 
     def get_geometry(self):
@@ -123,7 +156,7 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
 
     def get_property(self, prop):
         if prop=="xid":
-            return self.client_window.xid
+            return int(self.client_window.xid)
         elif prop=="depth":
             return self._depth
         elif prop=="title":
@@ -136,71 +169,9 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
             return True
         elif prop=="class-instance":
             return ("xpra-desktop", "Xpra-Desktop")
-        elif prop=="icon":
-            try:
-                icon_name = get_wm_name()+".png"
-                icon = get_icon(icon_name)
-                log("get_icon(%s)=%s", icon_name, icon)
-                return icon
-            except:
-                log("failed to return window icon")
-                return None
         else:
             return gobject.GObject.get_property(self, prop)
 
-    def resize(self, w, h):
-        geomlog("resize(%i, %i)", w, h)
-        if not RandR.has_randr():
-            geomlog.error("Error: cannot honour resize request,")
-            geomlog.error(" not RandR support on display")
-            return
-        #FIXME: small race if the user resizes with randr,
-        #at the same time as he resizes the window..
-        self.resize_value = (w, h)
-        if not self.resize_timer:
-            self.resize_timer = glib.timeout_add(250, self.do_resize)
-
-    def do_resize(self):
-        self.resize_timer = None
-        try:
-            w, h = self.resize_value
-            with xsync:
-                screen_sizes = RandR.get_screen_sizes()
-                #hack: force mistmatch
-                if FORCE_SCREEN_MISMATCH:
-                    screen_sizes = [(sw,sh) for sw,sh in screen_sizes if (sw!=w and sh!=h)]
-                geomlog("screen sizes=%s", screen_sizes)
-                if (w,h) not in screen_sizes:
-                    geomlog.warn("Warning: invalid screen size %ix%i", w, h)
-                    #find the nearest:
-                    distances = {}
-                    lower_distances = {}        #for sizes lower than requested
-                    for sw, sh in screen_sizes:
-                        distance = abs(sw*sh - w*h)
-                        distances.setdefault(distance, []).append((sw, sh))
-                        if sw<=w and sh<=h:
-                            lower_distances.setdefault(distance, []).append((sw, sh))
-                    geomlog("lower distances=%s", distances)
-                    if lower_distances:
-                        nearest = lower_distances[sorted(lower_distances.keys())[0]]
-                    else:
-                        geomlog("distances=%s", distances)
-                        nearest = distances[sorted(distances.keys())[0]]
-                    geomlog("nearest matches: %s", nearest)
-                    w, h = nearest[0]
-                    geomlog.warn(" using %ix%i instead", w, h)
-                    if RandR.get_screen_size()==(w,h):
-                        #this is already the resolution we have,
-                        #but the client has other ideas,
-                        #so tell the client we ain't budging:
-                        self.emit("resized")
-                        return
-                RandR.set_screen_size(w, h)
-        except Exception as e:
-            geomlog("resize(%i, %i)", w, h, exc_info=True)
-            geomlog.error("Error: failed to resize desktop display to %ix%i:", w, h)
-            geomlog.error(" %s", e)
-
     def _screen_size_changed(self, screen):
         w, h = screen.get_width(), screen.get_height()
         screenlog("screen size changed: new size %ix%i", w, h)
@@ -212,24 +183,65 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
     def update_size_hints(self, screen):
         w, h = screen.get_width(), screen.get_height()
         screenlog("screen dimensions: %ix%i", w, h)
-        if RandR.has_randr():
-            #TODO: get all of this from randr:
-            #screen_sizes = RandR.get_screen_sizes()
-            size_hints = {
-                "maximum-size"  : (8192, 4096),
-                "minimum-size"  : (640, 640),
-                "base-size"     : (640, 640),
-                "increment"     : (128, 128),
-                "minimum-aspect-ratio"  : (1, 3),
-                "maximum-aspect-ratio"  : (3, 1),
-                }
-        else:
+        size_hints = {}
+        def use_fixed_size():
             size = w, h
-            size_hints = {
+            size_hints.update({
                 "maximum-size"  : size,
                 "minimum-size"  : size,
                 "base-size"     : size,
-                }
+                })
+        if RandR.has_randr():
+            if self.resize_exact:
+                #assume resize_exact is enabled
+                #no size restrictions
+                size_hints = {}
+            else:
+                try:
+                    with xsync:
+                        screen_sizes = RandR.get_xrr_screen_sizes()
+                except:
+                    screenlog("failed to query screen sizes", exc_info=True)
+                else:
+                    if len(screen_sizes)==0:
+                        use_fixed_size()
+                    else:
+                        #find the maximum size supported:
+                        max_size = {}
+                        for tw, th in screen_sizes:
+                            max_size[tw*th] = (tw, th)
+                        max_pixels = sorted(max_size.keys())[-1]
+                        size_hints["maximum-size"] = max_size[max_pixels]
+                        #find the best increment we can use:
+                        inc_hits = {}
+                        #we should also figure out what the potential increments are,
+                        #rather than hardcoding them here:
+                        INC_VALUES = (16, 32, 64, 128, 256)
+                        for inc in INC_VALUES:
+                            hits = 0
+                            for tsize in screen_sizes:
+                                tw, th = tsize
+                                if (tw+inc, th+inc) in screen_sizes:
+                                    hits += 1
+                            inc_hits[inc] = hits
+                        screenlog("size increment hits: %s", inc_hits)
+                        max_hits = max(inc_hits.values())
+                        if max_hits>16:
+                            #find the first increment value matching the max hits
+                            for inc in INC_VALUES:
+                                if inc_hits[inc]==max_hits:
+                                    break
+                            #TODO: also get these values from the screen sizes:
+                            size_hints.update({
+                                "base-size"             : (640, 640),
+                                "minimum-size"          : (640, 640),
+                                "increment"             : (128, 128),
+                                "minimum-aspect-ratio"  : (1, 3),
+                                "maximum-aspect-ratio"  : (3, 1),
+                                })
+        else:
+            use_fixed_size()
+        screenlog("size-hints=%s", size_hints)
         self._updateprop("size-hints", size_hints)
 
 
@@ -239,11 +251,22 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
 gobject.type_register(DesktopModel)
 
 
+
+DESKTOPSERVER_BASES = [gobject.GObject]
+if server_features.rfb:
+    from xpra.server.rfb.rfb_server import RFBServer
+    DESKTOPSERVER_BASES.append(RFBServer)
+DESKTOPSERVER_BASES.append(X11ServerBase)
+DESKTOPSERVER_BASES = tuple(DESKTOPSERVER_BASES)
+DesktopServerBaseClass = type('DesktopServerBaseClass', DESKTOPSERVER_BASES, {})
+log("DesktopServerBaseClass%s", DESKTOPSERVER_BASES)
+
+
 """
     A server class for RFB / VNC-like desktop displays,
     used with the "start-desktop" subcommand.
 """
-class XpraDesktopServer(gobject.GObject, X11ServerBase):
+class XpraDesktopServer(DesktopServerBaseClass):
     __gsignals__ = {
         "xpra-xkb-event"        : one_arg_signal,
         "xpra-cursor-event"     : one_arg_signal,
@@ -251,13 +274,21 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         }
 
     def __init__(self):
-        gobject.GObject.__init__(self)
-        X11ServerBase.__init__(self)
+        for c in DESKTOPSERVER_BASES:
+            c.__init__(self)
+        self.session_type = "desktop"
+        self.resize_timer = None
+        self.resize_value = None
+
+    def init(self, opts):
+        for c in DESKTOPSERVER_BASES:
+            if c!=gobject.GObject:
+                c.init(self, opts)
 
     def x11_init(self):
         X11ServerBase.x11_init(self)
         assert init_x11_filter() is True
-        display = gtk.gdk.display_get_default()
+        display = gdk.display_get_default()
         screens = display.get_n_screens()
         for n in range(screens):
             screen = display.get_screen(n)
@@ -267,7 +298,8 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         add_catchall_receiver("xpra-xkb-event", self)
         X11Keyboard.selectBellNotification(True)
 
-    def do_cleanup(self, *args):
+    def do_cleanup(self):
+        self.cancel_resize_timer()
         X11ServerBase.do_cleanup(self)
         remove_catchall_receiver("xpra-motion-event", self)
         cleanup_x11_filter()
@@ -275,8 +307,11 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             cleanup_all_event_receivers()
 
 
+    def notify_dpi_warning(self, body):
+        pass
+
     def print_screen_info(self):
-        X11ServerBase.print_screen_info(self)
+        super(XpraDesktopServer, self).print_screen_info()
         root_w, root_h = get_root_size()
         sss = get_screen_sizes()
         log_screen_sizes(root_w, root_h, sss)
@@ -307,11 +342,48 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             return root_w, root_h
         w, h = requested_size
         screenlog("client requested desktop mode resolution is %sx%s (current server resolution is %sx%s)", w, h, root_w, root_h)
-        if w<=0 or h<=0:
+        if w<=0 or h<=0 or w>=32768 or h>=32768:
             screenlog("configure_best_screen_size() client requested an invalid desktop mode size: %s", requested_size)
             return root_w, root_h
         return self.set_screen_size(w, h)
 
+    def cancel_resize_timer(self):
+        rt = self.resize_timer
+        if rt:
+            self.resize_timer = None
+            glib.source_remove(rt)
+
+    def resize(self, w, h):
+        geomlog("resize(%i, %i)", w, h)
+        if not RandR.has_randr():
+            geomlog.error("Error: cannot honour resize request,")
+            geomlog.error(" no RandR support on this display")
+            return
+        #FIXME: small race if the user resizes with randr,
+        #at the same time as he resizes the window..
+        self.resize_value = (w, h)
+        if not self.resize_timer:
+            self.resize_timer = glib.timeout_add(250, self.do_resize)
+
+    def do_resize(self):
+        self.resize_timer = None
+        rw, rh = self.resize_value
+        try:
+            with xsync:
+                ow, oh = RandR.get_screen_size()
+            w, h = self.set_screen_size(rw, rh, False)
+            if (ow, oh) == (w, h):
+                #this is already the resolution we have,
+                #but the client has other ideas,
+                #so tell the client we ain't budging:
+                for win in self._window_to_id.keys():
+                    win.emit("resized")
+        except Exception as e:
+            geomlog("do_resize() %ix%i", rw, rh, exc_info=True)
+            geomlog.error("Error: failed to resize desktop display to %ix%i:", rw, rh)
+            geomlog.error(" %s", e)
+
+
     def set_desktop_geometry_attributes(self, w, h):
         #geometry is not synced with the client's for desktop servers
         pass
@@ -340,20 +412,21 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
     def load_existing_windows(self):
         #at present, just one  window is forwarded:
         #the root window covering the whole display
-        display = gtk.gdk.display_get_default()
+        display = gdk.display_get_default()
         screens = display.get_n_screens()
-        for n in range(screens):
-            screen = display.get_screen(n)
-            root = screen.get_root_window()
-            model = DesktopModel(root)
-            model.setup()
-            windowlog("adding root window model %s", model)
-            X11ServerBase._add_new_window_common(self, model)
-            model.managed_connect("client-contents-changed", self._contents_changed)
-            model.managed_connect("resized", self._window_resized_signaled)
+        with xsync:
+            for n in range(screens):
+                screen = display.get_screen(n)
+                root = screen.get_root_window()
+                model = DesktopModel(root, self.randr_exact_size)
+                model.setup()
+                windowlog("adding root window model %s", model)
+                super(XpraDesktopServer, self)._add_new_window_common(model)
+                model.managed_connect("client-contents-changed", self._contents_changed)
+                model.managed_connect("resized", self._window_resized_signaled)
 
 
-    def _window_resized_signaled(self, window, *args):
+    def _window_resized_signaled(self, window):
         #the vfb has been resized
         wid = self._window_to_id[window]
         x, y, w, h = window.get_geometry()
@@ -372,7 +445,7 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         windowlog("send_initial_windows(%s, %s) will send: %s", ss, sharing, self._id_to_window)
         for wid,window in sorted(self._id_to_window.items()):
             x, y, w, h = window.get_geometry()
-            wprops = self.client_properties.get("%s|%s" % (wid, ss.uuid))
+            wprops = self.client_properties.get(wid, {}).get(ss.uuid)
             ss.new_window("new-window", wid, window, x, y, w, h, wprops)
             ss.damage(wid, window, 0, 0, w, h)
 
@@ -388,7 +461,7 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
     def _set_window_state(self, proto, wid, window, new_window_state):
         if not new_window_state:
             return []
-        metadatalog("set_window_state%s", (wid, window, new_window_state))
+        metadatalog("set_window_state%s", (proto, wid, window, new_window_state))
         changes = []
         #boolean: but not a wm_state and renamed in the model... (iconic vs iconified!)
         iconified = new_window_state.get("iconified")
@@ -406,7 +479,7 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         model = self._id_to_window.get(wid)
         return model.client_window.get_screen().get_number()
 
-    def get_window_position(self, window):
+    def get_window_position(self, _window):
         #we export the whole desktop as a window:
         return 0, 0
 
@@ -418,11 +491,11 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             windowlog("cannot map window %s: already removed!", wid)
             return
         geomlog("client mapped window %s - %s, at: %s", wid, window, (x, y, w, h))
+        self._window_mapped_at(proto, wid, window, (x, y, w, h))
         if len(packet)>=8:
             self._set_window_state(proto, wid, window, packet[7])
         if len(packet)>=7:
             self._set_client_properties(proto, wid, window, packet[6])
-        self._window_mapped_at(proto, wid, window, (x, y, w, h))
         self._damage(window, 0, 0, w, h)
 
 
@@ -437,19 +510,19 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             #during iconification events:
             self._set_window_state(proto, wid, window, packet[3])
         assert not window.is_OR()
-        self._window_mapped_at(proto, wid, window, None)
+        self._window_mapped_at(proto, wid, window)
         #TODO: handle inconification?
         #iconified = len(packet)>=3 and bool(packet[2])
 
 
     def _process_configure_window(self, proto, packet):
         wid, x, y, w, h = packet[1:6]
-        if len(packet)>=13:
+        if len(packet)>=13 and server_features.input_devices and not self.readonly:
             pwid = packet[10]
             pointer = packet[11]
             modifiers = packet[12]
-            self._update_modifiers(proto, wid, modifiers)
-            self._process_mouse_common(proto, pwid, pointer)
+            if self._process_mouse_common(proto, pwid, pointer):
+                self._update_modifiers(proto, wid, modifiers)
         #some "configure-window" packets are only meant for metadata updates:
         skip_geometry = len(packet)>=10 and packet[9]
         window = self._id_to_window.get(wid)
@@ -463,7 +536,8 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         if not skip_geometry:
             owx, owy, oww, owh = window.get_geometry()
             geomlog("_process_configure_window(%s) old window geometry: %s", packet[1:], (owx, owy, oww, owh))
-            window.resize(w, h)
+            if oww!=w or owh!=h:
+                self.resize(w, h)
         if len(packet)>=7:
             cprops = packet[6]
             if cprops:
@@ -474,6 +548,21 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             self._damage(window, 0, 0, w, h)
 
 
+    def _adjust_pointer(self, proto, wid, pointer):
+        window = self._id_to_window.get(wid)
+        if not window:
+            self.suspend_cursor(proto)
+            return None
+        pointer = super(XpraDesktopServer, self)._adjust_pointer(proto, wid, pointer)
+        #maybe the pointer is off-screen:
+        ww, wh = window.get_dimensions()
+        x, y = pointer[:2]
+        if x<0 or x>=ww or y<0 or y>=wh:
+            self.suspend_cursor(proto)
+            return None
+        self.restore_cursor(proto)
+        return pointer
+
     def _move_pointer(self, wid, pos, *args):
         if wid>=0:
             window = self._id_to_window.get(wid)
@@ -482,7 +571,8 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             else:
                 #TODO: just like shadow server, adjust for window position
                 pass
-        X11ServerBase._move_pointer(self, wid, pos, *args)
+        with xsync:
+            X11ServerBase._move_pointer(self, wid, pos, -1, *args)
 
 
     def _process_close_window(self, proto, packet):
@@ -503,6 +593,10 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
         self.dbus_server = X11_DBUS_Server(self, os.environ.get("DISPLAY", "").lstrip(":"))
 
 
+    def show_all_windows(self):
+        log.warn("Warning: show_all_windows not implemented for desktop server")
+
+
     def do_make_screenshot_packet(self):
         log("grabbing screenshot")
         regions = []
@@ -536,5 +630,4 @@ class XpraDesktopServer(gobject.GObject, X11ServerBase):
             offset_y += 0
         return self.make_screenshot_packet_from_regions(regions)
 
-
 gobject.type_register(XpraDesktopServer)
