@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -29,6 +29,7 @@ from xpra.gtk_common.gtk2.gdk_bindings import (
                 get_display_for,                            #@UnresolvedImport
                 calc_constrained_size,                      #@UnresolvedImport
                )
+from xpra.gtk_common.gtk_util import get_default_root_window
 
 from xpra.log import Logger
 log = Logger("x11", "window")
@@ -73,6 +74,11 @@ CLAMP_OVERLAP = envint("XPRA_WINDOW_CLAMP_OVERLAP", 20)
 assert CLAMP_OVERLAP>=0
 
 
+#aliases to reduce the number of pydev warnings:
+PARAM_READABLE = gobject.PARAM_READABLE
+PARAM_READWRITE = gobject.PARAM_READWRITE
+
+
 class WindowModel(BaseWindowModel):
     """This represents a managed client window.  It allows one to produce
     widgets that view that client window in various ways."""
@@ -87,19 +93,19 @@ class WindowModel(BaseWindowModel):
     __gproperties__.update({
         "owner": (gobject.TYPE_PYOBJECT,
                   "Owner", "",
-                  gobject.PARAM_READABLE),
+                  PARAM_READABLE),
         # Interesting properties of the client window, that will be
         # automatically kept up to date:
         "requested-position": (gobject.TYPE_PYOBJECT,
                                "Client-requested position on screen", "",
-                               gobject.PARAM_READABLE),
+                               PARAM_READABLE),
         "requested-size": (gobject.TYPE_PYOBJECT,
                            "Client-requested size on screen", "",
-                           gobject.PARAM_READABLE),
+                           PARAM_READABLE),
         "set-initial-position": (gobject.TYPE_BOOLEAN,
                                  "Should the requested position be honoured?", "",
                                  False,
-                                 gobject.PARAM_READWRITE),
+                                 PARAM_READWRITE),
         # Toggling this property does not actually make the window iconified,
         # i.e. make it appear or disappear from the screen -- it merely
         # updates the various window manager properties that inform the world
@@ -107,28 +113,28 @@ class WindowModel(BaseWindowModel):
         "iconic": (gobject.TYPE_BOOLEAN,
                    "ICCCM 'iconic' state -- any sort of 'not on desktop'.", "",
                    False,
-                   gobject.PARAM_READWRITE),
+                   PARAM_READWRITE),
         #from WM_NORMAL_HINTS
         "size-hints": (gobject.TYPE_PYOBJECT,
                        "Client hints on constraining its size", "",
-                       gobject.PARAM_READABLE),
+                       PARAM_READABLE),
         #from _NET_WM_ICON_NAME or WM_ICON_NAME
         "icon-title": (gobject.TYPE_PYOBJECT,
                        "Icon title (unicode or None)", "",
-                       gobject.PARAM_READABLE),
+                       PARAM_READABLE),
         #from _NET_WM_ICON
         "icon": (gobject.TYPE_PYOBJECT,
                  "Icon (local Cairo surface)", "",
-                 gobject.PARAM_READABLE),
+                 PARAM_READABLE),
         #from _NET_WM_ICON
         "icon-pixmap": (gobject.TYPE_PYOBJECT,
                         "Icon (server Pixmap)", "",
-                        gobject.PARAM_READABLE),
+                        PARAM_READABLE),
         #from _MOTIF_WM_HINTS.decorations
         "decorations": (gobject.TYPE_INT,
                        "Should the window decorations be shown", "",
                        -1, 65535, -1,
-                       gobject.PARAM_READABLE),
+                       PARAM_READABLE),
         })
     __gsignals__ = dict(BaseWindowModel.__common_signals__)
     __gsignals__.update({
@@ -173,13 +179,13 @@ class WindowModel(BaseWindowModel):
     def setup(self):
         super(WindowModel, self).setup()
 
-        x, y, w, h, _ = self.client_window.get_geometry()
+        ox, oy, ow, oh, _ = self.client_window.get_geometry()
         # We enable PROPERTY_CHANGE_MASK so that we can call
         # x11_get_server_time on this window.
         # clamp this window to the desktop size:
-        x, y = self._clamp_to_desktop(x, y, w, h)
+        x, y = self._clamp_to_desktop(ox, oy, ow, oh)
         self.corral_window = gdk.Window(self.parking_window,
-                                        x=x, y=y, width=w, height=h,
+                                        x=x, y=y, width=ow, height=oh,
                                         window_type=gdk.WINDOW_CHILD,
                                         wclass=gdk.INPUT_OUTPUT,
                                         event_mask=gdk.PROPERTY_CHANGE_MASK,
@@ -219,8 +225,11 @@ class WindowModel(BaseWindowModel):
             raise Unmanageable("window constrained size is too large: %sx%s (from client geometry: %s,%s with size hints=%s)" % (nw, nh, w, h, hints))
         self._updateprop("geometry", (x, y, nw, nh))
         geomlog("setup() resizing windows to %sx%s", nw, nh)
-        self.corral_window.resize(nw, nh)
-        self.client_window.resize(nw, nh)
+        #don't trigger a resize unless we have to:
+        if ow!=nw or oh!=nh:
+            self.corral_window.resize(nw, nh)
+        if w!=nw or h!=nh:
+            self.client_window.resize(nw, nh)
         self.client_window.show_unraised()
         #this is here to trigger X11 errors if any are pending
         #or if the window is deleted already:
@@ -307,7 +316,7 @@ class WindowModel(BaseWindowModel):
                 for prop in WindowModel.SCRUB_PROPERTIES:
                     X11Window.XDeleteProperty(self.xid, prop)
             if self.client_reparented:
-                self.client_window.reparent(gdk.get_default_root_window(), 0, 0)
+                self.client_window.reparent(get_default_root_window(), 0, 0)
                 self.client_reparented = False
             self.client_window.set_events(self.client_window_saved_events)
             #it is now safe to destroy the corral window:
@@ -454,12 +463,12 @@ class WindowModel(BaseWindowModel):
 
     def _do_update_client_geometry(self, window_size_cb, window_position_cb):
         allocated_w, allocated_h = window_size_cb()
-        geomlog("_do_update_client_geometry: allocated %ix%i", allocated_w, allocated_h)
+        geomlog("_do_update_client_geometry: allocated %ix%i (from %s)", allocated_w, allocated_h, window_size_cb)
         hints = self.get_property("size-hints")
         w, h = calc_constrained_size(allocated_w, allocated_h, hints)
         geomlog("_do_update_client_geometry: size(%s)=%ix%i", hints, w, h)
         x, y = window_position_cb(w, h)
-        geomlog("_do_update_client_geometry: position=%ix%i", x, y)
+        geomlog("_do_update_client_geometry: position=%ix%i (from %s)", x, y, window_position_cb)
         self.corral_window.move_resize(x, y, w, h)
         self._updateprop("geometry", (x, y, w, h))
         with xswallow:
@@ -486,36 +495,35 @@ class WindowModel(BaseWindowModel):
         try:
             #workaround applications whose windows disappear from underneath us:
             with xsync:
-                self.resize_corral_window(event.x, event.y, event.width, event.height, event.border_width)
+                #event.border_width unused
+                self.resize_corral_window(event.x, event.y, event.width, event.height)
         except XError as e:
             geomlog("do_xpra_configure_event(%s)", event, exc_info=True)
             geomlog.warn("Warning: failed to resize corral window %#x", self.corral_window.xid)
             geomlog.warn(" %s", e)
 
-    def resize_corral_window(self, x, y, w, h, _border):
+    def resize_corral_window(self, x, y, w, h):
         #the client window may have been resized or moved (generally programmatically)
         #so we may need to update the corral_window to match
         cox, coy, cow, coh = self.corral_window.get_geometry()[:4]
         #size changes (and position if any):
         hints = self.get_property("size-hints")
         w, h = calc_constrained_size(w, h, hints)
-        geomlog("resize_corral_window() new constrained size=%ix%i", w, h)
         cx, cy, cw, ch = self.get_property("geometry")
-        if cow!=w or coh!=h:
-            #at least resized, check for move:
-            if (x, y) != (0, 0):
+        resized = cow!=w or coh!=h
+        moved = (x, y) != (0, 0)
+        if resized:
+            if moved:
                 self._internal_set_property("set-initial-position", True)
                 geomlog("resize_corral_window() move and resize from %s to %s", (cox, coy, cow, coh), (x, y, w, h))
                 self.corral_window.move_resize(x, y, w, h)
                 self.client_window.move(0, 0)
                 self._updateprop("geometry", (x, y, w, h))
             else:
-                #just resize:
                 geomlog("resize_corral_window() resize from %s to %s", (cow, coh), (w, h))
                 self.corral_window.resize(w, h)
                 self._updateprop("geometry", (cx, cy, w, h))
-        #just position change:
-        elif (x, y) != (0, 0):
+        elif moved:
             self._internal_set_property("set-initial-position", True)
             geomlog("resize_corral_window() moving corral window from %s to %s", (cox, coy), (x, y))
             self.corral_window.move(x, y)
@@ -609,7 +617,7 @@ class WindowModel(BaseWindowModel):
             if motif_hints.flags & (2**MotifWMHints.DECORATIONS_BIT):
                 self._updateprop("decorations", motif_hints.decorations)
             if motif_hints.flags & (2**MotifWMHints.INPUT_MODE_BIT):
-                self._updateprop("modal", bool(motif_hints.input_mode))
+                self._updateprop("modal", int(motif_hints.input_mode))
 
 
     def _handle_wm_normal_hints_change(self):
