@@ -1,10 +1,11 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import gtk
+from gtk import gdk
+
 import gobject
 import os
 
@@ -13,8 +14,9 @@ from xpra.gtk_common.error import xsync, xswallow
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
 from xpra.x11.window_info import window_name, window_info
 from xpra.gtk_common.gobject_util import no_arg_signal, one_arg_signal
+from xpra.gtk_common.gtk_util import get_xwindow, get_default_root_window, display_get_default
 
-from xpra.x11.gtk2 import Unmanageable
+from xpra.x11.common import Unmanageable
 from xpra.x11.gtk2.selection import ManagerSelection
 from xpra.x11.gtk2.world_window import WorldWindow
 from xpra.x11.gtk2.window import WindowModel, configure_bits
@@ -96,8 +98,6 @@ DEFAULT_NET_SUPPORTED = [
         "_NET_CLOSE_WINDOW",
 
         # These aren't supported in any particularly meaningful way, but hey.
-        "_NET_FRAME_EXTENTS",
-
         "_NET_WM_WINDOW_TYPE",
         "_NET_WM_WINDOW_TYPE_NORMAL",
         "_NET_WM_WINDOW_TYPE_DESKTOP",
@@ -142,6 +142,10 @@ DEFAULT_NET_SUPPORTED = [
         # Not at all yet:
         #"_NET_RESTACK_WINDOW",
         ]
+FRAME_EXTENTS = envbool("XPRA_FRAME_EXTENTS", True)
+if FRAME_EXTENTS:
+    DEFAULT_NET_SUPPORTED.append("_NET_FRAME_EXTENTS")
+
 NET_SUPPORTED = [x for x in DEFAULT_NET_SUPPORTED if x not in NO_NET_SUPPORTED]
 
 
@@ -165,7 +169,7 @@ def wm_check(display, wm_name, upgrading=False):
             continue
         def xid(w):
             if w:
-                return "%#x" % w.xid
+                return "%#x" % get_xwindow(w)
             return None
         log("_NET_SUPPORTING_WM_CHECK for screen %i: %s (root=%s)", i, xid(ewmh_wm), xid(root))
         if ewmh_wm:
@@ -222,7 +226,7 @@ class Wm(gobject.GObject):
         gobject.GObject.__init__(self)
 
         if display is None:
-            display = gtk.gdk.display_manager_get().get_default_display()
+            display = display_get_default()
         self._display = display
         self._root = self._display.get_default_screen().get_root_window()
         self._wm_name = wm_name
@@ -249,10 +253,10 @@ class Wm(gobject.GObject):
         # Set up the necessary EWMH properties on the root window.
         self._setup_ewmh_window()
         # Start with just one desktop:
-        self.set_desktop_list([u"Main"])
+        self.set_desktop_list((u"Main", ))
         self.set_current_desktop(0)
         # Start with the full display as workarea:
-        root_w, root_h = gtk.gdk.get_default_root_window().get_size()
+        root_w, root_h = get_default_root_window().get_size()
         self.root_set("_NET_SUPPORTED", ["atom"], NET_SUPPORTED)
         self.set_workarea(0, 0, root_w, root_h)
         self.set_desktop_geometry(root_w, root_h)
@@ -267,13 +271,17 @@ class Wm(gobject.GObject):
         # the existing clients.
         add_event_receiver(self._root, self)
         add_fallback_receiver("xpra-client-message-event", self)
+        #when reparenting, the events may get sent
+        #to a window that is already destroyed
+        #and we don't want to miss those events, so:
+        add_fallback_receiver("child-map-request-event", self)
         X11Window.substructureRedirect(self._root.xid)
 
         for w in get_children(self._root):
             # Checking for FOREIGN here filters out anything that we've
             # created ourselves (like, say, the world window), and checking
             # for mapped filters out any withdrawn windows.
-            if (w.get_window_type() == gtk.gdk.WINDOW_FOREIGN
+            if (w.get_window_type() == gdk.WINDOW_FOREIGN
                 and not X11Window.is_override_redirect(w.xid)
                 and X11Window.is_mapped(w.xid)):
                 log("Wm managing pre-existing child window %#x", w.xid)
@@ -366,7 +374,7 @@ class Wm(gobject.GObject):
             self._update_window_list()
             self.emit("new-window", win)
 
-    def _handle_client_unmanaged(self, window, wm_exiting):
+    def _handle_client_unmanaged(self, window, _wm_exiting):
         gdkwindow = window.get_property("client-window")
         assert gdkwindow in self._windows
         del self._windows[gdkwindow]
@@ -374,7 +382,7 @@ class Wm(gobject.GObject):
         self._update_window_list()
         self.notify("windows")
 
-    def _update_window_list(self, *args):
+    def _update_window_list(self, *_args):
         # Ignore errors because not all the windows may still exist; if so,
         # then it's okay to leave the lists out of date for a moment, because
         # in a moment we'll get a signal telling us about the window that
@@ -398,19 +406,19 @@ class Wm(gobject.GObject):
         if event.message_type=="_NET_SHOWING_DESKTOP":
             show = bool(event.data[0])
             self.emit("show-desktop", show)
-        elif event.message_type=="_NET_REQUEST_FRAME_EXTENTS":
+        elif event.message_type=="_NET_REQUEST_FRAME_EXTENTS" and FRAME_EXTENTS:
             #if we're here, that means the window model does not exist
             #(or it would have processed the event)
             #so this must be a an unmapped window
             frame = (0, 0, 0, 0)
-            if not X11Window.is_override_redirect(event.window.xid):
-                #use the global default:
-                frame = prop_get(self._root, "DEFAULT_NET_FRAME_EXTENTS", ["u32"], ignore_errors=True)
-            if not frame:
-                #fallback:
-                frame = (0, 0, 0, 0)
-            log("_NET_REQUEST_FRAME_EXTENTS: setting _NET_FRAME_EXTENTS=%s on %#x", frame, event.window.xid)
             with xswallow:
+                if not X11Window.is_override_redirect(event.window.xid):
+                    #use the global default:
+                    frame = prop_get(self._root, "DEFAULT_NET_FRAME_EXTENTS", ["u32"], ignore_errors=True)
+                if not frame:
+                    #fallback:
+                    frame = (0, 0, 0, 0)
+                framelog("_NET_REQUEST_FRAME_EXTENTS: setting _NET_FRAME_EXTENTS=%s on %#x", frame, event.window.xid)
                 prop_set(event.window, "_NET_FRAME_EXTENTS", ["u32"], frame)
 
     def _lost_wm_selection(self, selection):
@@ -422,7 +430,8 @@ class Wm(gobject.GObject):
 
     def cleanup(self):
         remove_fallback_receiver("xpra-client-message-event", self)
-        for win in list(self._windows.itervalues()):
+        remove_fallback_receiver("child-map-request-event", self)
+        for win in tuple(self._windows.itervalues()):
             win.unmanage(True)
 
     def do_child_map_request_event(self, event):
@@ -494,12 +503,12 @@ class Wm(gobject.GObject):
         # clobber any XSelectInput calls that *we* might have wanted to make
         # on this window.)  Also, GDK might silently swallow all events that
         # are detected on it, anyway.
-        self._ewmh_window = gtk.gdk.Window(self._root,
+        self._ewmh_window = gdk.Window(self._root,
                                            width=1,
                                            height=1,
-                                           window_type=gtk.gdk.WINDOW_TOPLEVEL,
+                                           window_type=gdk.WINDOW_TOPLEVEL,
                                            event_mask=0, # event mask
-                                           wclass=gtk.gdk.INPUT_ONLY,
+                                           wclass=gdk.INPUT_ONLY,
                                            title=self._wm_name)
         prop_set(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK",
                  "window", self._ewmh_window)
