@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,7 +11,6 @@ from xpra.x11.gtk_x11.gdk_display_source import init_display_source
 init_display_source()
 
 from xpra.util import std, csv
-from xpra.keyboard.layouts import parse_xkbmap_query
 from xpra.gtk_common.error import xsync
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 X11Keyboard = X11KeyboardBindings()
@@ -50,7 +49,7 @@ def clean_keyboard_state():
 ################################################################################
 # keyboard layouts
 
-def do_set_keymap(xkbmap_layout, xkbmap_variant,
+def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
                   xkbmap_print, xkbmap_query, xkbmap_query_struct={}):
     """ xkbmap_layout is the generic layout name (used on non posix platforms)
         xkbmap_variant is the layout variant (may not be set)
@@ -63,6 +62,7 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
     #First we try to use data from setxkbmap -query,
     #preferably as structured data:
     if xkbmap_query and not xkbmap_query_struct:
+        from xpra.keyboard.layouts import parse_xkbmap_query
         xkbmap_query_struct = parse_xkbmap_query(xkbmap_query)
     if xkbmap_query_struct:
         log("do_set_keymap using xkbmap_query struct=%s", xkbmap_query_struct)
@@ -73,7 +73,6 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
             "layout"      : "gb",
             "options"     : "grp:shift_caps_toggle",
             }
-        (we execute the options separately in case that fails..)
         """
         #parse the data into a dict:
         rules = xkbmap_query_struct.get("rules")
@@ -82,26 +81,12 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
         variant = xkbmap_query_struct.get("variant")
         options = xkbmap_query_struct.get("options")
         if layout:
-            log.info("setting keymap: %s", ", ".join(["%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items() if k in ["rules", "model", "layout"] and v]))
-            try:
-                X11Keyboard.setxkbmap(rules, model, layout, variant, options)
-                return
-            except:
-                log.warn("failed to set exact keymap using %s", xkbmap_query_struct)
-            if options:
-                #try again with no options:
-                try:
-                    X11Keyboard.setxkbmap(rules, model, layout, variant, "")
-                    return
-                except:
-                    log.error("failed to set exact keymap even without applying options")
+            log.info("setting keymap: %s", csv("%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items() if k in ["rules", "model", "layout"] and v))
+            safe_setxkbmap(rules, model, layout, variant, options)
         else:
-            try:
-                X11Keyboard.setxkbmap(rules, model, "", "", "")
-                return
-            except:
-                log.warn("failed to set exact keymap using %s", xkbmap_query_struct)
+            safe_setxkbmap(rules, model, "", "", "")
     if xkbmap_print:
+        #TODO: this is no longer used with any client, remove it
         log("do_set_keymap using xkbmap_print")
         #try to guess the layout by parsing "setxkbmap -print"
         try:
@@ -111,14 +96,31 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
                 if m:
                     layout = std(m.group(1))
                     log.info("guessing keyboard layout='%s'" % layout)
-                    X11Keyboard.setxkbmap("", "pc104", layout, "", "")
+                    safe_setxkbmap("evdev", "pc105", layout, "", xkbmap_options)
                     return
         except Exception as e:
             log.info("error setting keymap: %s" % e)
-    #fallback:
+    #fallback for non X11 clients:
     layout = xkbmap_layout or "us"
     log.info("setting keyboard layout to '%s'", std(layout))
-    X11Keyboard.setxkbmap("", "", layout, xkbmap_variant, "")
+    safe_setxkbmap("evdev", "pc105", layout, xkbmap_variant, xkbmap_options)
+
+def safe_setxkbmap(rules, model, layout, variant, options):
+    #(we execute the options separately in case that fails..)
+    try:
+        X11Keyboard.setxkbmap(rules, model, layout, variant, options)
+        return
+    except:
+        log.warn("Warning: failed to set exact keymap,")
+        log.warn(" rules=%s, model=%s, layout=%s, variant=%s, options=%s", rules, model, layout, variant, options)
+    if options:
+        #try again with no options:
+        try:
+            X11Keyboard.setxkbmap(rules, model, layout, variant, "")
+            return
+        except:
+            log.error("Error: failed to set exact keymap")
+            log.error(" even without applying any options..")
 
 
 ################################################################################
@@ -129,7 +131,7 @@ def apply_xmodmap(instructions):
         with xsync:
             unset = X11Keyboard.set_xmodmap(instructions)
     except:
-        log.error("apply_xmodmap", exc_info=True)
+        log.error("Error configuring modifier map", exc_info=True)
         unset = instructions
     if unset is None:
         #None means an X11 error occurred, re-do all:
@@ -147,10 +149,13 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
         keycodes = indexed_mappings(xkbmap_x11_keycodes)
     else:
         keycodes = gtk_keycodes_to_mappings(xkbmap_keycodes)
+    log("set_keycode_translation(%s, %s)", xkbmap_x11_keycodes, xkbmap_keycodes)
+    log(" keycodes=%s", keycodes)
     #keycodes = {
     #     9: set([('', 1), ('Escape', 4), ('', 3), ('Escape', 0), ('Escape', 2)]),
     #     10: set([('onesuperior', 4), ('onesuperior', 8), ('exclam', 1), ('1', 6), ('exclam', 3), ('1', 2), ('exclamdown', 9), ('exclamdown', 5), ('1', 0), ('exclam', 7)]),
     x11_keycodes = X11Keyboard.get_keycode_mappings()
+    log(" x11_keycodes=%s", x11_keycodes)
     #x11_keycodes = {
     #    8: ['Mode_switch', '', 'Mode_switch', '', 'Mode_switch'],
     #    9: ['Escape', '', 'Escape', '', 'Escape'],
@@ -158,9 +163,15 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
     x11_keycodes_for_keysym = {}
     for keycode, keysyms in x11_keycodes.items():
         for keysym in keysyms:
-            x11_keycodes_for_keysym.setdefault(keysym, []).append(keycode)
+            x11_keycodes_for_keysym.setdefault(keysym, set()).add(keycode)
     def find_keycode(kc, keysym, i):
-        keycodes = x11_keycodes_for_keysym.get(keysym)
+        keycodes = tuple(x11_keycodes_for_keysym.get(keysym, set()))
+        if keysym in DEBUG_KEYSYMS:
+            log.info("set_keycode_translation: find_keycode%s x11 keycodes=%s", (kc, keysym, i), keycodes)
+        def rlog(v, msg):
+            if keysym in DEBUG_KEYSYMS:
+                log.info("set_keycode_translation: find_keycode%s=%s (%s)", (kc, keysym, i), v, msg)
+            return v
         if not keycodes:
             return None
         #no other option, use it:
@@ -168,21 +179,38 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
             return keycodes[0]
         for keycode in keycodes:
             defs = x11_keycodes.get(keycode)
+            if keysym in DEBUG_KEYSYMS:
+                log.info("x11 keycode %i: %s", keycode, defs)
             assert defs, "bug: keycode %i not found in %s" % (keycode, x11_keycodes)
-            if (keysym, i) in defs:
-                return keycode
+            if len(defs)>i and defs[i]==keysym:
+                return rlog(keycode, "exact index match")
         #if possible, use the same one:
         if kc in keycodes:
-            return kc
-        return keycodes[0]
+            return rlog(kc, "using same keycode as client")
+        return rlog(keycodes[0], "using first match")
     #generate the translation map:
     trans = {}
     for keycode, defs in keycodes.items():
-        for keysym,i in list(defs):             #ie: ('1', 0)
+        for keysym,i in tuple(defs):             #ie: ('1', 0) or ('A', 1), etc
             x11_keycode = find_keycode(keycode, keysym, i)
             if x11_keycode:
                 trans[(keycode, keysym)] = x11_keycode
+                trans[(keysym, i)] = x11_keycode
                 trans[keysym] = x11_keycode
+    if not xkbmap_x11_keycodes:
+        #now add all the keycodes we may not have mapped yet
+        #(present in x11_keycodes but not keycodes)
+        for keycode, keysyms in x11_keycodes.items():
+            for i, keysym in enumerate(keysyms):
+                if keysym not in trans:
+                    if keysym in DEBUG_KEYSYMS:
+                        log.info("x11 keycode %s: %s", keycode, keysym)
+                    trans[keysym] = keycode
+                key = (keysym, i)
+                if key not in trans:
+                    if keysym in DEBUG_KEYSYMS:
+                        log.info("x11 keycode %s: %s", keycode, key)
+                    trans[key] = keycode
     log("set_keycode_translation(..)=%s", trans)
     return trans
 
@@ -234,20 +262,20 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
         invalid_keysyms = set()
         def estr(entries):
             try:
-                return csv(list(set(x[0] for x in entries)))
+                return csv(tuple(set(x[0] for x in entries)))
             except:
-                return csv(list(entries))
+                return csv(tuple(entries))
         for keycode, entries in mappings.items():
             mods = modifiers_for(entries)
             if len(mods)>1:
                 log.warn("Warning: keymapping changed:")
-                log.warn(" keycode %s points to %i modifiers: %s", keycode, len(mods), csv(list(mods)))
+                log.warn(" keycode %s points to %i modifiers: %s", keycode, len(mods), csv(tuple(mods)))
                 log.warn(" from definition: %s", estr(entries))
                 for mod in mods:
                     emod = [entry for entry in entries if mod in modifiers_for([entry])]
                     log.warn(" %s: %s", mod, estr(emod))
                 #keep just the first one:
-                mods = list(mods)[:1]
+                mods = tuple(mods)[:1]
                 mod = mods[0]
                 entries = [entry for entry in entries if mod in modifiers_for([entry])]
                 log.warn(" keeping: %s for %s", estr(entries), mod)
@@ -443,7 +471,7 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
         keysyms = set([keysym for keysym, _ in entries])
         if len(keysyms)==0:
             return 0
-        if len(keysyms)==1 and list(keysyms)[0]=='0xffffff':
+        if len(keysyms)==1 and tuple(keysyms)[0]=='0xffffff':
             log("skipped invalid keysym: %s / %s", client_keycode, entries)
             return 0
         l = log
@@ -455,11 +483,11 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
             return do_assign(client_keycode, client_keycode, entries)
         if len(keysyms)==1:
             #only one keysym, replace with single entry
-            entries = set([(list(keysyms)[0], 0)])
+            entries = set([(tuple(keysyms)[0], 0)])
 
         #the candidate preserve entries: those that have at least one of the keysyms:
         preserve_keycode_matches = {}
-        for keysym in list(keysyms):
+        for keysym in tuple(keysyms):
             keycodes = preserve_keysyms_map.get(keysym, [])
             for keycode in keycodes:
                 v = preserve_keycode_entries.get(keycode)
@@ -555,9 +583,9 @@ def keymap_to_xmodmap(trans_keycodes):
     for server_keycode, entries in trans_keycodes.items():
         keysyms = [None]*keysyms_per_keycode
         names = [""]*keysyms_per_keycode
-        for name, index in entries:
+        sentries = sorted(entries, key=lambda x:x[1])
+        for name, index in sentries:
             assert 0<=index and index<keysyms_per_keycode
-            names[index] = name
             try:
                 keysym = X11Keyboard.parse_keysym(name)
             except:
@@ -567,11 +595,30 @@ def keymap_to_xmodmap(trans_keycodes):
                     missing_keysyms.append(name)
             else:
                 if keysyms[index] is not None:
-                    log.warn("we already have a keysym for %s at index %s: %s, entries=%s", server_keycode, index, keysyms[index], entries)
-                else:
-                    keysyms[index] = keysym
-                    if keysym in DEBUG_KEYSYMS:
-                        log.info("keymap_to_xmodmap: keysyms[%s]=%s", index, keysym)
+                    #if the client provides multiple keysyms for the same index,
+                    #replace with the new one if the old one exists elsewhere,
+                    #or skip it if we have another entry for it
+                    can_override = any(True for i,v in enumerate(keysyms) if i<index and v==keysyms[index])
+                    can_skip = any(True for i,v in enumerate(keysyms) if i!=index and v==keysym)
+                    if can_override or can_skip:
+                        l = log.debug
+                    else:
+                        l = log.warn
+                    l("Warning: more than one keysym for keycode %-3i at index %i:", server_keycode, index)
+                    l(" entries=%s", csv(tuple(sentries)))
+                    l(" keysyms=%s", csv(keysyms))
+                    l(" assigned keysym=%s", names[index])
+                    l(" wanted keysym=%s", name)
+                    if can_override:
+                        l(" current value also found at another index, overriding it")
+                    elif can_skip:
+                        l(" new value also found elsewhere, skipping it")
+                    else:
+                        continue
+                names[index] = name
+                keysyms[index] = keysym
+                if name in DEBUG_KEYSYMS:
+                    log.info("keymap_to_xmodmap: keysyms[%s]=%s (%s)", index, keysym, name)
         #remove empty keysyms:
         while len(keysyms)>0 and keysyms[0] is None:
             keysyms = keysyms[1:]
@@ -590,7 +637,7 @@ def keymap_to_xmodmap(trans_keycodes):
 ################################################################################
 # modifiers
 
-def clear_modifiers(modifiers):
+def clear_modifiers(_modifiers):
     instructions = []
     for i in range(0, 8):
         instructions.append(("clear", i))
@@ -642,7 +689,7 @@ def get_modifiers_from_meanings(xkbmap_mod_meanings):
     log("get_modifiers_from_meanings(%s) modifier dict=%s", xkbmap_mod_meanings, modifiers)
     return modifiers
 
-def get_modifiers_from_keycodes(xkbmap_keycodes):
+def get_modifiers_from_keycodes(xkbmap_keycodes, add_default_modifiers=True):
     """
         Some platforms can't tell us about modifier mappings
         So we try to find matches from the defaults below:
@@ -651,7 +698,6 @@ def get_modifiers_from_keycodes(xkbmap_keycodes):
     pref = DEFAULT_MODIFIER_MEANINGS
     #keycodes are: {keycode : (keyval, name, keycode, group, level)}
     matches = {}
-    log("get_modifiers_from_keycodes(%s...)", str(xkbmap_keycodes))
     log("get_modifiers_from_keycodes(%s...)", str(xkbmap_keycodes)[:160])
     all_keynames = set()
     for entry in xkbmap_keycodes:
@@ -661,18 +707,45 @@ def get_modifiers_from_keycodes(xkbmap_keycodes):
             keynames = matches.setdefault(modifier, set())
             keynames.add(keyname)
             all_keynames.add(keyname)
-    #try to add missings ones (magic!)
-    defaults = {}
-    for keyname, modifier in DEFAULT_MODIFIER_MEANINGS.items():
-        if keyname in all_keynames:
-            continue            #aleady defined
-        if modifier not in matches:
-            #define it since it is completely missing
-            defaults.setdefault(modifier, set()).add(keyname)
-        elif modifier in ["shift", "lock", "control", "mod1", "mod2"] or keyname=="ISO_Level3_Shift":
-            #these ones we always add them, even if a record for this modifier already exists
-            matches.setdefault(modifier, set()).add(keyname)
-    log("get_modifiers_from_keycodes(...) adding defaults: %s", defaults)
-    matches.update(defaults)
+    if add_default_modifiers:
+        #try to add missings ones (magic!)
+        defaults = {}
+        for keyname, modifier in DEFAULT_MODIFIER_MEANINGS.items():
+            if keyname in all_keynames:
+                continue            #aleady defined
+            if modifier not in matches:
+                #define it since it is completely missing
+                defaults.setdefault(modifier, set()).add(keyname)
+            elif modifier in ["shift", "lock", "control", "mod1", "mod2"] or keyname=="ISO_Level3_Shift":
+                #these ones we always add them, even if a record for this modifier already exists
+                matches.setdefault(modifier, set()).add(keyname)
+        log("get_modifiers_from_keycodes(...) adding defaults: %s", defaults)
+        matches.update(defaults)
     log("get_modifiers_from_keycodes(...)=%s", matches)
     return matches
+
+def map_missing_modifiers(keynames_for_mod):
+    x11_keycodes = X11Keyboard.get_keycode_mappings()
+    min_keycode, max_keycode = X11Keyboard.get_minmax_keycodes()
+    free_keycodes = [x for x in range(min_keycode, max_keycode) if x not in x11_keycodes]
+    log("map_missing_modifiers(%s) min_keycode=%i max_keycode=%i, free_keycodes=%s", keynames_for_mod, min_keycode, max_keycode, free_keycodes)
+    keysyms_to_keycode = {}
+    for keycode, keysyms in x11_keycodes.items():
+        for keysym in keysyms:
+            keysyms_to_keycode.setdefault(keysym, []).append(keycode)
+    xmodmap_changes = []
+    for mod, keysyms in keynames_for_mod.items():
+        missing = []
+        for keysym in keysyms:
+            if keysym not in keysyms_to_keycode:
+                missing.append(keysym)
+        if missing:
+            log("map_missing_modifiers: no keycode found for modifier keys %s (%s)", csv(missing), mod)
+            if not free_keycodes:
+                log.warn("Warning: keymap is full, cannot add '%s' for modifier '%s'", keysym, mod)
+            else:
+                keycode = free_keycodes.pop()
+                xmodmap_changes.append(("keycode", keycode, missing))
+    if xmodmap_changes:
+        log("xmodmap_changes=%s", xmodmap_changes)
+        X11Keyboard.set_xmodmap(xmodmap_changes)
